@@ -1,6 +1,6 @@
 /* abermud: An example implementation of AberMUD for gzochi
  * abermud.c: Main loop implementation for gzochi abermud example game
- * Copyright (C) 2014 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * This is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -164,7 +164,7 @@ handle_input (abermud_context *context)
       if (context->input_buffer->pos > 0)
 	{
 	  context->input_buffer->pos--;
-	  strncpy
+	  memmove
 	    (context->input_buffer->data + context->input_buffer->pos, 
 	     context->input_buffer->data + context->input_buffer->pos + 1, 
 	     strlen (context->input_buffer->data) - 
