@@ -117,14 +117,11 @@
 
     (gzochi:set-binding! 
      "motd" (gzochi:make-managed-serializable 
-	     motd 
-	     (g:@ (gzochi io) gzochi:write-string) 
-	     (g:@ (gzochi io) gzochi:read-string)))
+	     motd (g:<-> gzochi:write-string gzochi:read-string)))
+    
     (gzochi:set-binding!
      "help" (gzochi:make-managed-serializable
-	     help 
-	     (g:@ (gzochi io) gzochi:write-string) 
-	     (g:@ (gzochi io) gzochi:read-string)))
+	     help (g:<-> gzochi:write-string gzochi:read-string)))
 
     (gzochi:log-info "Creator Of Legends: Initialised"))
 
