@@ -1,6 +1,6 @@
 ;; data.scm --- Data structures for gzochi AberMUD example game
 
-;; Copyright (C) 2014 Julian Graham
+;; Copyright (C) 2018 Julian Graham
 ;;
 ;; This software is provided 'as-is', without any express or implied
 ;; warranty. In no event will the authors be held liable for any damages
@@ -27,11 +27,19 @@
 	  abermud:find-noun
 	  abermud:find-adjective
 	  abermud:word-equal?
+	  abermud:write-word
+	  abermud:read-word
+	  abermud:write-words
+	  abermud:read-words
 
 	  abermud:make-term
 	  abermud:term?
 	  abermud:term-adjective
 	  abermud:term-noun
+	  abermud:term-hash
+	  abermud:term-equal?
+	  abermud:write-term
+	  abermud:read-term
 
           abermud:make-item
 	  abermud:item-adjective
@@ -156,6 +164,23 @@
   (define abermud:word-serialization 
     (gzochi:make-serialization abermud:write-word abermud:read-word))
 
+  ;; Serializer function for writing an immutable list of `abermud:word' records
+  ;; to the specified port.
+  
+  (define (abermud:write-words port words)
+    (gzochi:write-integer port (length words))
+    (for-each (lambda (word) (abermud:write-word port word)) words))
+
+  ;; Deserializer function for reading an immutable list of `abermud:word'
+  ;; records from the specified port.
+  
+  (define (abermud:read-words port)
+    (let ((count (gzochi:read-integer port)))
+      (let loop ((i 0) (words '()))
+	(if (< i count)
+	    (loop (+ i 1) (cons (abermud:read-word port) words))
+	    (reverse words)))))
+  
   ;; Custom hash and equality functions for the `abermud:word' type.
 
   (define (abermud:word-hash word) 
@@ -166,15 +191,7 @@
 	 (abermud:word? w2)
 	 (equal? (abermud:word-text w1) (abermud:word-text w2))
 	 (eq? (abermud:word-word-type w1) (abermud:word-word-type w2))))
-
-  ;; Convenience function to wrap a word with a `gzochi:managed-serializable'
-  ;; wrapper so that it can be treated as a managed record, attaching the
-  ;; required serialization helpers.
-
-  (define (wrap-word word)
-    (gzochi:make-managed-serializable
-     word (g:@ abermud:write-word) (g:@ abermud:read-word)))
-
+  
   ;; Returns the table of known words (keyed by word text) as a 
   ;; `gzochi:managed-hashtable' record, lazily initializing it if it is not
   ;; already bound.
@@ -201,38 +218,27 @@
 	   (word-list
 	    (if (gzochi:managed-hashtable-contains? words text)
 		(let ((lst (gzochi:managed-hashtable-ref words text #f)))
-		  (gzochi:cons (wrap-word word) lst))
-		(gzochi:managed-list (wrap-word word)))))
+		  (cons word lst))
+		(list word))))
 
       (gzochi:managed-hashtable-set!
        words text word-list
-       #:key-serializer (g:@ (gzochi io) gzochi:write-string) 
-       #:key-deserializer (g:@ (gzochi io) gzochi:read-string))))
-
-  (define (find-words words text)
-    (gzochi:managed-hashtable-ref words text #f))
+       #:key-serialization (g:<-> gzochi:write-string gzochi:read-string)
+       #:value-serialization (g:<-> abermud:write-words abermud:read-words))))
 
   ;; Finds all of the words associated with the specified word text. There may
   ;; be zero, one, or many, depending on how many word types share the 
   ;; specified text.
 
   (define (abermud:find-words text)
-    (let ((word-list (find-words (get-words) text)))
-      (and word-list 
-	   (map gzochi:managed-serializable-value 
-		(gzochi:managed-list->list word-list)))))
+    (gzochi:managed-hashtable-ref (get-words) text #f))
 
   ;; Returns the a word structure that matches the specified text and word 
   ;; type, or `#f' if one has not been registered.
 
   (define (abermud:find-word text type)
-    (let ((word-list (find-words (get-words) text)))
-      (let loop ((word-list word-list))
-	(and word-list
-	     (let ((word 
-		    (gzochi:managed-serializable-value (gzochi:car word-list))))
-	       (if (eq? (abermud:word-word-type word) type)
-		   word (loop (gzochi:cdr word-list))))))))
+    (find (lambda (word) (eq? (abermud:word-word-type word) type))
+	  (or (abermud:find-words text) '())))
 
   ;; Convenience function to find in the word table the adjective with the 
   ;; specified text.
@@ -372,9 +378,8 @@
       (if (gzochi:managed-hashtable-contains? items term)
 	  (raise (make-assertion-violation))
 	  (gzochi:managed-hashtable-set! 
-	   items term item 
-	   #:key-serializer (g:@ abermud:write-term)
-	   #:key-deserializer (g:@ abermud:read-term)))))
+	   items term item
+	   #:key-serialization (g:<-> abermud:write-term abermud:read-term)))))
 
   ;; Returns the item identified by the specified words. If `word2' is given,
   ;; it must be a noun and `word1' must be an adjective; otherwise, `word1' 
