@@ -1,5 +1,5 @@
 /* avatars.h: Prototypes and declarations for avatars.c
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * This is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,6 +24,6 @@
 /* The avatar sprite data, in the form of a single bitmap in a packed PBM-like 
    format. */
 
-unsigned char mazewar_avatar_bits[3072];
+extern unsigned char mazewar_avatar_bits[3072];
 
 #endif /* MAZEWAR_AVATARS_H */
