@@ -1,5 +1,5 @@
 /* font.h: Prototypes and declarations for font.h
- * Copyright (C) 2012 Andreas Schiffler
+ * Copyright (C) 2020 Andreas Schiffler
  *
  * This is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,6 +21,6 @@
 /* The 8x13 font, in the form of a single bitmap in a packed PBM-like 
    format. */
 
-unsigned char mazewar_font_8x13_bytes[2912];
+extern unsigned char mazewar_font_8x13_bytes[2912];
 
 #endif /* MAZEWAR_FONT_H */
