@@ -1,5 +1,5 @@
 /* view.h: Declarations and prototypes for views.c
- * Copyright (C) 2012 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * This is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@ typedef struct _mazewar_edge
   mazewar_vertex p2; /* The second point. */
 } mazewar_edge;
 
-mazewar_edge mazewar_view_edges[2880]; /* Pre-computed 3-D edge data table. */
+/* Pre-computed 3-D edge data table. */
+
+extern mazewar_edge mazewar_view_edges[2880]; 
 
 #endif /* MAZEWAR_DATA_VIEWS_H */
