@@ -1,5 +1,5 @@
 ;; gzochi/main-loop.scm: A main loop implementation
-;; Copyright (C) 2017 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 	  gzochi:main-loop-run
 	  gzochi:main-loop-stop)
 
-  (import (only (guile) hashv make-mutex select sort) 
+  (import (only (guile) hashv select sort) 
 	  (ice-9 optargs) 
 	  (ice-9 threads) 
 	  (rnrs)
