@@ -1,5 +1,5 @@
 ;; echo-chamber/client.scm --- Echo chamber benchmark, client side
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -178,7 +178,7 @@
 ;; specified server address.
 
 (define (connect hostname port num)
-  (define addr-info (car (getaddrinfo hostname)))
+  (define addr-info (car (getaddrinfo hostname #f 0 AF_INET SOCK_STREAM)))
   (define serv-addr
     (make-socket-address
      AF_INET (sockaddr:addr (addrinfo:addr addr-info)) port))
