@@ -7,4 +7,4 @@ NUM_CLIENTS=10
 MESSAGES_PER_CLIENT=10
 
 
-guile -e main -s client.scm localhost 8001 10 10 100
+guile -e main -s client.scm localhost 8001 $NUM_CLIENTS $MESSAGES_PER_CLIENT 100
