@@ -1,5 +1,5 @@
 /* admin.c: Primitive functions for gzochi admin API
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@ static GzochidGameServer *game_server;
 static SCM scm_application_context_name;
 static SCM scm_make_application_context;
 
-SCM_DEFINE (primitive_applications, "primitive-applications", 0, 0, 0, 
-	    (), "List the current active applications.")
+SCM
+primitive_applications (void)
 {
   SCM ret = SCM_EOL;
   GList *applications = gzochid_game_server_get_applications (game_server);
@@ -60,8 +60,8 @@ SCM_DEFINE (primitive_applications, "primitive-applications", 0, 0, 0,
   return ret;
 }
 
-SCM_DEFINE (primitive_current_application, "primitive-current-application", 
-	    0, 0, 0, (), "Returns the current application or #f if not set.")
+SCM
+primitive_current_application (void)
 {
   GzochidApplicationContext *context =
     gzochid_get_current_application_context ();
@@ -84,9 +84,8 @@ with_application_unwind_handler (void *data)
   gzochid_transaction_end ();
 }
 
-SCM_DEFINE (primitive_with_application, "primitive-with-application", 
-	    2, 0, 0, (SCM context, SCM thunk), 
-	    "Returns the current application or #f if not set.")
+SCM
+primitive_with_application (SCM context, SCM thunk)
 {
   char *name = scm_to_locale_string 
     (scm_call_1 (scm_application_context_name, context));
@@ -124,7 +123,6 @@ SCM_DEFINE (primitive_with_application, "primitive-with-application",
 void 
 gzochid_api_admin_init (GzochidGameServer *server)
 {
-  SCM current_module = scm_current_module ();
   SCM gzochi_admin = scm_c_resolve_module ("gzochi admin");
 
   SCM make_application_context_var = scm_c_module_lookup 
@@ -141,9 +139,21 @@ gzochid_api_admin_init (GzochidGameServer *server)
   scm_gc_protect_object (scm_application_context_name);
 
   game_server = g_object_ref (server);
-  scm_set_current_module (gzochi_admin);
 
-  #include "admin.x"
+  scm_call_1
+    (scm_c_public_ref ("gzochi admin", "gzochi:set-primitive-applications!"),
+     scm_c_make_gsubr ("primitive-applications", 0, 0, 0,
+		       primitive_applications));
 
-  scm_set_current_module (current_module);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi admin", "gzochi:set-primitive-current-application!"),
+     scm_c_make_gsubr ("primitive-current-application", 0, 0, 0,
+		       primitive_current_application));
+  
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi admin", "gzochi:set-primitive-with-application!"),
+     scm_c_make_gsubr ("primitive-with-application", 2, 0, 0,
+		       primitive_with_application));
 }
