@@ -1,5 +1,5 @@
 /* channel.c: Primitive functions for user-facing gzochid channel API
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -28,8 +28,8 @@
 #include "channel.h"
 #include "util.h"
 
-SCM_DEFINE (primitive_create_channel, "primitive-create-channel", 1, 0, 0,
-	    (SCM name), "Create a new channel with the specified name.")
+SCM
+primitive_create_channel (SCM name)
 {
   GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context ();
@@ -57,8 +57,8 @@ SCM_DEFINE (primitive_create_channel, "primitive-create-channel", 1, 0, 0,
   return ret;
 }
 
-SCM_DEFINE (primitive_get_channel, "primitive-get-channel", 1, 0, 0,
-	    (SCM name), "Retrieves the channel with the specified name.")
+SCM
+primitive_get_channel (SCM name)
 {
   GError *err = NULL;
   GzochidApplicationContext *context = 
@@ -87,8 +87,8 @@ SCM_DEFINE (primitive_get_channel, "primitive-get-channel", 1, 0, 0,
   return ret;
 }
 
-SCM_DEFINE (primitive_join_channel, "primitive-join-channel", 2, 0, 0,
-	    (SCM channel, SCM session), "Add client session to a channel.")
+SCM
+primitive_join_channel (SCM channel, SCM session)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -119,9 +119,8 @@ SCM_DEFINE (primitive_join_channel, "primitive-join-channel", 2, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_leave_channel, "primitive-leave-channel", 2, 0, 0,
-	    (SCM channel, SCM session), 
-	    "Remove a client session from a channel.")
+SCM
+primitive_leave_channel (SCM channel, SCM session)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -152,9 +151,8 @@ SCM_DEFINE (primitive_leave_channel, "primitive-leave-channel", 2, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_send_channel_message, "primitive-send-channel-message", 
-	    2, 0, 0, (SCM channel, SCM bv), 
-	    "Send a message to the sessions that belong to a channel.")
+SCM
+primitive_send_channel_message (SCM channel, SCM bv)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -179,8 +177,8 @@ SCM_DEFINE (primitive_send_channel_message, "primitive-send-channel-message",
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_close_channel, "primitive-close-channel", 1, 0, 0,
-	    (SCM channel, SCM msg), "Shut down a channel.")
+SCM
+primitive_close_channel (SCM channel, SCM msg)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -203,11 +201,45 @@ SCM_DEFINE (primitive_close_channel, "primitive-close-channel", 1, 0, 0,
 
 void gzochid_api_channel_init (void)
 {
-  SCM current_module = scm_current_module ();
-  SCM gzochi_private_channel = scm_c_resolve_module ("gzochi private channel");
-  scm_set_current_module (gzochi_private_channel);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-create-channel!"),
+     scm_c_make_gsubr ("primitive-create-channel", 1, 0, 0,
+		       primitive_create_channel));
 
-  #include "channel.x"
-
-  scm_set_current_module (current_module);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-get-channel!"),
+     scm_c_make_gsubr ("primitive-get-channel", 1, 0, 0,
+		       primitive_get_channel));
+  
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-join-channel!"),
+     scm_c_make_gsubr ("primitive-join-channel", 2, 0, 0,
+		       primitive_join_channel));
+  
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-leave-channel!"),
+     scm_c_make_gsubr ("primitive-leave-channel", 2, 0, 0,
+		       primitive_leave_channel));
+  
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-send-channel-message!"),
+     scm_c_make_gsubr ("primitive-send-channel-message", 2, 0, 0,
+		       primitive_send_channel_message));
+  
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-create-channel!"),
+     scm_c_make_gsubr ("primitive-create-channel", 1, 0, 0,
+		       primitive_create_channel));
+  
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private channel", "gzochi:set-primitive-close-channel!"),
+     scm_c_make_gsubr ("primitive-close-channel", 1, 0, 0,
+		       primitive_close_channel));
 }
