@@ -1,5 +1,5 @@
 /* data.c: Primitive functions for user-facing gzochi data management API
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@
 #include "data.h"
 #include "util.h"
 
-SCM_DEFINE (primitive_create_reference, "primitive-create-reference", 1, 0, 0, 
-	    (SCM obj), "Create a managed reference.")
+SCM
+primitive_create_reference (SCM obj)
 {
   GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context ();
@@ -56,8 +56,8 @@ SCM_DEFINE (primitive_create_reference, "primitive-create-reference", 1, 0, 0,
   return ret;
 }
 
-SCM_DEFINE (primitive_dereference, "primitive-dereference", 1, 0, 0, (SCM ref),
-	    "Dereference a managed reference.")
+SCM
+primitive_dereference (SCM ref)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -97,15 +97,15 @@ static char *prefix_name (char *name)
 {
   int name_len = strlen (name);
   char *prefixed_name = malloc (sizeof (char) * (name_len + 3));
-  
-  prefixed_name = strncpy (prefixed_name, "o.", 3);
-  prefixed_name = strncat (prefixed_name, name, name_len);
+
+  memcpy (prefixed_name, "o.", 3);
+  memcpy (prefixed_name + 2, name, name_len + 1);
 
   return prefixed_name;
 }
 
-SCM_DEFINE (primitive_get_binding, "primitive-get-binding", 1, 0, 0, (SCM name),
-	    "Retrieve a managed record bound to a name.")
+SCM
+primitive_get_binding (SCM name)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -152,8 +152,8 @@ SCM_DEFINE (primitive_get_binding, "primitive-get-binding", 1, 0, 0, (SCM name),
   return ret;
 }
 
-SCM_DEFINE (primitive_set_binding_x, "primitive-set-binding!", 2, 0, 0,
-	    (SCM name, SCM obj), "Bind a managed record to a name.")
+SCM
+primitive_set_binding_x (SCM name, SCM obj)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -189,8 +189,8 @@ SCM_DEFINE (primitive_set_binding_x, "primitive-set-binding!", 2, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_remove_binding_x, "primitive-remove-binding!", 1, 0, 0,
-	    (SCM name), "Remove the binding for a name.")
+SCM
+primitive_remove_binding_x (SCM name)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -222,8 +222,8 @@ SCM_DEFINE (primitive_remove_binding_x, "primitive-remove-binding!", 1, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_remove_object_x, "primitive-remove-object!", 1, 0, 0,
-	    (SCM obj), "Remove a managed record from the data store.")
+SCM
+primitive_remove_object_x (SCM obj)
 {
   GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context ();
@@ -251,8 +251,8 @@ SCM_DEFINE (primitive_remove_object_x, "primitive-remove-object!", 1, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_mark_for_write_x, "primitive-mark-for-write!", 1, 0, 0,
-	    (SCM obj), "Mark a managed record that has been modified.")
+SCM
+primitive_mark_for_write_x (SCM obj)
 {
   GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context ();
@@ -270,11 +270,39 @@ SCM_DEFINE (primitive_mark_for_write_x, "primitive-mark-for-write!", 1, 0, 0,
 
 void gzochid_api_data_init (void)
 {
-  SCM current_module = scm_current_module ();
-  SCM gzochi_private_data = scm_c_resolve_module ("gzochi private data");
-  scm_set_current_module (gzochi_private_data);
-
-  #include "data.x"
-
-  scm_set_current_module (current_module);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-create-reference!"),
+     scm_c_make_gsubr ("primitive-create-reference", 1, 0, 0,
+		       primitive_create_reference));
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-dereference!"),
+     scm_c_make_gsubr ("primitive-dereference", 1, 0, 0,
+		       primitive_dereference));
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-get-binding!"),
+     scm_c_make_gsubr ("primitive-get-binding", 1, 0, 0,
+		       primitive_get_binding));
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-set-binding!"),
+     scm_c_make_gsubr ("primitive-set-binding!", 2, 0, 0,
+		       primitive_set_binding_x));
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-remove-binding!"),
+     scm_c_make_gsubr ("primitive-remove-binding!", 1, 0, 0,
+		       primitive_remove_binding_x));
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-remove-object!"),
+     scm_c_make_gsubr ("primitive-remove-object!", 1, 0, 0,
+		       primitive_remove_object_x));
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data", "gzochi:set-primitive-mark-for-write!"),
+     scm_c_make_gsubr ("primitive-mark-for-write!", 1, 0, 0,
+		       primitive_mark_for_write_x));
 }
