@@ -1,5 +1,5 @@
 /* log.c: Primitive functions for user-facing gzochid transactional log API
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -25,9 +25,8 @@
 
 #include "util.h"
 
-SCM_DEFINE (primitive_log_internal, "primitive-log-internal", 2, 0, 0,
-	    (SCM priority, SCM msg), 
-	    "Non-transactionally log a message for the current application")
+SCM
+primitive_log_internal (SCM priority, SCM msg)
 {
   char *cpriority = scm_to_locale_string (scm_symbol_to_string (priority));
   char *cmsg = scm_to_locale_string (msg);
@@ -49,8 +48,8 @@ SCM_DEFINE (primitive_log_internal, "primitive-log-internal", 2, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_log, "primitive-log", 2, 0, 0, (SCM priority, SCM msg), 
-	    "Log a message for the current application")
+SCM
+primitive_log (SCM priority, SCM msg)
 {
   GzochidApplicationContext *context = 
     gzochid_api_ensure_current_application_context ();
@@ -78,11 +77,14 @@ SCM_DEFINE (primitive_log, "primitive-log", 2, 0, 0, (SCM priority, SCM msg),
 
 void gzochid_api_log_init (void)
 {
-  SCM current_module = scm_current_module ();
-  SCM gzochi_private_log = scm_c_resolve_module ("gzochi private log");
-  scm_set_current_module (gzochi_private_log);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private log", "gzochi:set-primitive-log!"),
+     scm_c_make_gsubr ("primitive-log", 2, 0, 0, primitive_log));
 
-  #include "log.x"
-
-  scm_set_current_module (current_module);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private log", "gzochi:set-primitive-log-internal!"),
+     scm_c_make_gsubr ("primitive-log-internal", 2, 0, 0,
+		       primitive_log_internal));
 }
