@@ -1,5 +1,5 @@
 /* session.c: Primitive functions for user-facing gzochid client session API
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@
 #include "session.h"
 #include "util.h"
 
-SCM_DEFINE (primitive_send_message, "primitive-send-message", 2, 0, 0,
-	    (SCM session, SCM msg), "Send a message to a client session.")
+SCM
+primitive_send_message (SCM session, SCM msg)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -54,8 +54,8 @@ SCM_DEFINE (primitive_send_message, "primitive-send-message", 2, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (primitive_disconnect, "primitive-disconnect", 1, 0, 0,
-	    (SCM session), "Disconnect a client session.")
+SCM
+primitive_disconnect (SCM session)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -78,13 +78,17 @@ SCM_DEFINE (primitive_disconnect, "primitive-disconnect", 1, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-void gzochid_api_session_init (void)
+void
+gzochid_api_session_init (void)
 {
-  SCM current_module = scm_current_module ();
-  SCM gzochi_private_session = scm_c_resolve_module ("gzochi private session");
-  scm_set_current_module (gzochi_private_session);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private session", "gzochi:set-primitive-send-message!"),
+     scm_c_make_gsubr ("primitive-send-message", 2, 0, 0,
+		       primitive_send_message));
 
-  #include "session.x"
-
-  scm_set_current_module (current_module);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private session", "gzochi:set-primitive-disconnect!"),
+     scm_c_make_gsubr ("primitive-disconnect", 1, 0, 0, primitive_disconnect));
 }
