@@ -1,5 +1,5 @@
 /* task.c: Primitive functions for user-facing gzochi task management API
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -32,9 +32,8 @@
 #include "task.h"
 #include "util.h"
 
-SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0, 
-	    (SCM callback, SCM delay, SCM period), 
-	    "Schedule a durable task to run.")
+SCM
+primitive_schedule_task (SCM callback, SCM delay, SCM period)
 {
   GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context (); 
@@ -121,8 +120,8 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
     }
 }
 
-SCM_DEFINE (primitive_cancel_task, "primitive-cancel-task", 1, 0, 0, 
-	    (SCM scm_handle), "Cancel a previously scheduled periodic task.")
+SCM
+primitive_cancel_task (SCM scm_handle)
 {
   GError *err = NULL;
   GzochidApplicationContext *context =
@@ -151,11 +150,16 @@ SCM_DEFINE (primitive_cancel_task, "primitive-cancel-task", 1, 0, 0,
 void 
 gzochid_api_task_init (void)
 {
-  SCM current_module = scm_current_module ();
-  SCM gzochi_private_task = scm_c_resolve_module ("gzochi private task");
-  scm_set_current_module (gzochi_private_task);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private task", "gzochi:set-primitive-schedule-task!"),
+     scm_c_make_gsubr ("primitive-schedule-task", 1, 2, 0,
+		       primitive_schedule_task));
 
-  #include "task.x"
-
-  scm_set_current_module (current_module);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private task", "gzochi:set-primitive-cancel-task!"),
+     scm_c_make_gsubr ("primitive-cancel-task", 1, 0, 0,
+		       primitive_cancel_task));
+  
 }
