@@ -1,5 +1,5 @@
 /* util.h: Prototypes and declarations for util.c
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,7 +22,11 @@
 
 #include "../app.h"
 
-gzochid_application_context *gzochid_api_ensure_current_application_context 
+/* Verifies that a thread-local application context has been registered (via
+   `gzochid_with_application_context') exiting non-locally via raising 
+   `gzochi:no-current-application-condition' if none was found. */
+
+GzochidApplicationContext *gzochid_api_ensure_current_application_context
 (void);
 
 void gzochid_api_check_not_found (GError *);
