@@ -1,5 +1,5 @@
 /* app-task.h: Prototypes and declarations for app-task.c
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -27,26 +27,81 @@
 #include "task.h"
 #include "tx.h"
 
-struct _gzochid_task_transaction_context
+/* Enumeration of application states during the bootstrap process. */
+
+enum _gzochid_application_bootstrap_stage
+  {
+    /* The application is still waiting to begin normal task processing. */
+
+    NOT_STARTED,
+
+    /* The bootstrap process has started (i.e., the transactional bootstrap 
+       work has been scheduled) but has not yet completed. */
+    
+    STARTING,
+    
+    STARTED, /* The application is fully bootstrapped and processing tasks. */
+    FAILED /* The transactional bootstrap work did not complete successfully. */
+  };
+
+typedef enum _gzochid_application_bootstrap_stage
+gzochid_application_bootstrap_stage;
+
+/* Encapsulates state related to whether an application is ready to submit tasks
+   to the local server's task processing queue. */
+
+struct _gzochid_application_state
 {
-  gzochid_application_context *context;
-  gzochid_auth_identity *identity;
-  GList *scheduled_tasks;
+  /* The bootstrap stage. */
+
+  gzochid_application_bootstrap_stage bootstrap_stage; 
+  
+  /* List of `guint64' pointers representing durable task oids buffered for 
+     queue submission. */
+
+  GList *queued_task_oids; 
+
+  GMutex mutex; /* Synchronizes access to the running flag and task buffers. */
+  GCond cond; /* Condition for broadcasting changes in running status. */
 };
 
-typedef struct _gzochid_task_transaction_context
-gzochid_task_transaction_context;
+typedef struct _gzochid_application_state gzochid_application_state;
+
+/* Construct and return a pointer to a new application state object. The memory
+   used by this object should be released via `gzochid_application_state_free'
+   when no longer needed. */
+
+gzochid_application_state *gzochid_application_state_new ();
+
+/* Frees the memory associated with the specified `gzochid_application_state'
+   object. */
+
+void gzochid_application_state_free (gzochid_application_state *);
+
+/* A function type to represent an application-specific task executing on behalf
+   of a particular identity. */
 
 typedef void (*gzochid_application_worker) 
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
+
+/* An application-specific task. Conceptually, this is a layer on top of 
+   `GzochidTask'. */
 
 struct _gzochid_application_task
 {
-  gzochid_application_worker worker;
-  gzochid_application_context *context;
-  gzochid_auth_identity *identity;
-  gpointer data;
+  /* The application to which the task belongs. */
 
+  GzochidApplicationContext *context; 
+
+  gzochid_auth_identity *identity; /* The identity that "owns" the task. */
+  gzochid_application_worker worker; /* The task worker function. */
+
+  /* The task destroy function, called when the task's reference count reaches 
+     zero. */
+
+  GDestroyNotify destroy_notify; 
+
+  gpointer data; /* The task data. */
   guint ref_count; /* The reference count. */
 };
 
@@ -81,14 +136,14 @@ typedef struct _gzochid_transactional_application_task_execution
 gzochid_transactional_application_task_execution;
 
 gzochid_transaction_result gzochid_application_transaction_execute 
-(gzochid_application_context *, void (*) (gpointer), gpointer);
+(GzochidApplicationContext *, void (*) (gpointer), gpointer);
 
 gzochid_transaction_result gzochid_application_transaction_execute_timed 
-(gzochid_application_context *, void (*) (gpointer), gpointer, struct timeval);
+(GzochidApplicationContext *, void (*) (gpointer), gpointer, struct timeval);
 
 gzochid_application_task *gzochid_application_task_new 
-(gzochid_application_context *, gzochid_auth_identity *, 
- gzochid_application_worker, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, 
+ gzochid_application_worker, GDestroyNotify, gpointer);
 
 /* Increases the reference count of the specified application task and returns 
    the application task. */
@@ -127,7 +182,7 @@ void gzochid_transactional_application_task_execution_free
 (gzochid_transactional_application_task_execution *);
 
 void gzochid_application_transactional_task_worker 
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 
 /* A `gzochid_application_worker' implementation that synchronously executes the
    transactional application task wrapped by the specified 
@@ -136,7 +191,7 @@ void gzochid_application_transactional_task_worker
    catch and cleanup handler tasks as appropriate. */
 
 void gzochid_application_reexecuting_transactional_task_worker 
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 
 /* A `gzochid_application_worker' implementation that asynchronously executes
    the transactional application task wrapped by the specified
@@ -145,7 +200,7 @@ void gzochid_application_reexecuting_transactional_task_worker
    catch and cleanup handler tasks as appropriate. */
 
 void gzochid_application_resubmitting_transactional_task_worker 
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 
 void gzochid_application_task_thread_worker (gpointer, gpointer);
 
