@@ -1,5 +1,5 @@
 /* app.c: Application context routines for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,10 +16,13 @@
  */
 
 #include <assert.h>
+#include <errno.h>
 #include <glib.h>
 #include <glib-object.h>
+#include <glib/gstdio.h>
 #include <libguile.h>
 #include <stddef.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/time.h>
 
@@ -31,6 +34,7 @@
 #include "dataclient.h"
 #include "descriptor.h"
 #include "event.h"
+#include "filelog.h"
 #include "guile.h"
 #include "gzochid-auth.h"
 #include "metaclient.h"
@@ -67,6 +71,11 @@ struct _GzochidApplicationContext
   char *deployment_root;
   
   GList *load_paths; /* Complete set of module load paths. */
+
+  /* The log domain, off the form "app/[app name]" for convenience use by 
+     app-specific logging facilities. */
+
+  char *log_domain; 
   
   /* The global authentication plugin registry. */
 
@@ -86,8 +95,11 @@ struct _GzochidApplicationContext
   
   GzochidStorageEngineContainer *storage_engine_container;
 
-  gzochid_application_store *store; /* The application store. */
+  /* The oid strategy to use with the application store. */
+  
+  gzochid_oid_allocation_strategy *oid_strategy; 
 
+  gzochid_application_store *store; /* The application store. */
   gzochid_application_state *state; /* The application state struct. */
   GzochidTaskQueue *task_queue; /* The global task scheduling queue. */
   guint64 tx_timeout_ms; /* The default transaction timeout. */
@@ -167,7 +179,6 @@ initialize_data (GzochidApplicationContext *app_context)
   char *data_dir = NULL, *work_dir = NULL;
   GHashTable *config = gzochid_configuration_extract_group
     (app_context->configuration, "game");
-  gzochid_oid_allocation_strategy *oid_strategy = NULL;  
   gzochid_storage_engine *storage_engine = NULL;
   gzochid_storage_engine_interface *iface = NULL;
   
@@ -222,16 +233,17 @@ initialize_data (GzochidApplicationContext *app_context)
       
       g_object_get (metaclient, "data-client", &dataclient, NULL);
 
-      oid_strategy = gzochid_dataclient_oid_strategy_new
+      app_context->oid_strategy = gzochid_dataclient_oid_strategy_new
 	(dataclient, app_context->descriptor->name);
 
       g_object_unref (dataclient);
       g_object_unref (metaclient);
     }
-  else oid_strategy = gzochid_storage_oid_strategy_new (iface, data_dir);
+  else app_context->oid_strategy =
+	 gzochid_storage_oid_strategy_new (iface, data_dir);
 
   app_context->store = gzochid_application_store_open
-    (iface, data_dir, oid_strategy);
+    (iface, data_dir, app_context->oid_strategy);
   
   if (iface == &gzochid_storage_engine_interface_dataclient)
     {
@@ -282,7 +294,8 @@ initialize_load_paths (GzochidApplicationContext *app_context)
   assert (app_context->descriptor->deployment_root != NULL);
   
   app_context->load_paths = g_list_prepend 
-    (g_list_copy (app_context->descriptor->load_paths),
+    (g_list_copy_deep
+     (app_context->descriptor->load_paths, (GCopyFunc) strdup, NULL),
      strdup (app_context->descriptor->deployment_root));
   
   G_LOCK (load_path);
@@ -291,6 +304,33 @@ initialize_load_paths (GzochidApplicationContext *app_context)
   G_UNLOCK (load_path);
 }
 
+static void
+initialize_logging (GzochidApplicationContext *app_context)
+{
+  app_context->log_domain = g_strdup_printf
+    ("app/%s", app_context->descriptor->name);
+
+  if (app_context->descriptor->log_filename != NULL)
+    {
+      FILE *log_file = g_fopen (app_context->descriptor->log_filename, "a");
+
+      if (log_file == NULL)
+	{
+	  g_critical
+	    ("Failed to open log file '%s' for writing for application %s: %s",
+	     app_context->descriptor->log_filename,
+	     app_context->descriptor->name, g_strerror (errno));
+	  exit (EXIT_FAILURE);
+	}
+      
+      g_log_set_handler_full
+	(app_context->log_domain,
+	 G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
+	 gzochid_file_log_handler, gzochid_file_log_context_new (log_file),
+	 gzochid_file_log_context_free);
+    }
+}
+
 /*
   Completes the initialization of an application running on a server in a 
   single-node configuration.
@@ -432,6 +472,7 @@ gzochid_application_context_constructed (GObject *object)
   initialize_auth (app_context);
   initialize_data (app_context);
   initialize_load_paths (app_context);
+  initialize_logging (app_context);
 
   gzochid_event_attach
     (app_context->event_source, update_stats, app_context->stats);
@@ -560,6 +601,7 @@ gzochid_application_context_dispose (GObject *gobject)
     GZOCHID_APPLICATION_CONTEXT (gobject);
 
   g_object_unref (app_context->configuration);
+  g_object_unref (app_context->descriptor);
   g_object_unref (app_context->metaclient_container);
   g_object_unref (app_context->auth_plugin_registry);
   g_object_unref (app_context->storage_engine_container);
@@ -574,8 +616,13 @@ gzochid_application_context_finalize (GObject *gobject)
   GzochidApplicationContext *app_context =
     GZOCHID_APPLICATION_CONTEXT (gobject);
 
+  gzochid_auth_identity_cache_destroy (app_context->identity_cache);
+  g_list_free_full (app_context->load_paths, free);
+  g_free (app_context->log_domain);
+  gzochid_oid_allocation_strategy_free (app_context->oid_strategy);
   gzochid_application_session_map_free (app_context->session_map);
   gzochid_application_state_free (app_context->state);
+  gzochid_application_store_close (app_context->store);
   
   g_source_destroy ((GSource *) app_context->event_source);
   g_source_unref ((GSource *) app_context->event_source);
@@ -715,6 +762,13 @@ gzochid_application_context_get_name (GzochidApplicationContext *app_context)
   return app_context->descriptor->name;
 }
 
+const char *
+gzochid_application_context_get_log_domain
+(GzochidApplicationContext *app_context)
+{
+  return app_context->log_domain;
+}
+
 const char *
 gzochid_application_context_get_description
 (GzochidApplicationContext *app_context)
