@@ -127,6 +127,18 @@ gzochid_auth_identity *gzochid_application_context_authenticate
 
 const char *gzochid_application_context_get_name (GzochidApplicationContext *);
 
+/* 
+   Returns the GLib log domain for the specified application, as constructed by
+   prepending "app/" to the application name given by the appication 
+   descriptor. 
+
+   The returned string is owned by the application and should not be freed or
+   otherwise modified.
+*/
+
+const char *gzochid_application_context_get_log_domain
+(GzochidApplicationContext *);
+
 /* 
    Returns the descriptive name of the specified application, as given by its
    descriptor. 
