@@ -311,7 +311,7 @@ gzochid_auth_plugin_registry_lookup (GzochidAuthPluginRegistry *registry,
 
 void 
 gzochid_auth_identity_serializer 
-(gzochid_application_context *context, void *ptr, GByteArray *out, GError **err)
+(GzochidApplicationContext *context, void *ptr, GByteArray *out, GError **err)
 {
   gzochid_auth_identity *identity = ptr;
 
@@ -321,11 +321,11 @@ gzochid_auth_identity_serializer
 
 void *
 gzochid_auth_identity_deserializer
-(gzochid_application_context *context, GByteArray *in, GError **err)
+(GzochidApplicationContext *context, GByteArray *in, GError **err)
 {
   char *name = strndup ((char *) in->data, in->len);
   gzochid_auth_identity *identity =
-    gzochid_auth_identity_from_name (context->identity_cache, name);
+    gzochid_application_context_resolve_identity (context, name);
 
   g_byte_array_remove_range (in, 0, strlen (name) + 1);
 
@@ -335,8 +335,7 @@ gzochid_auth_identity_deserializer
 }
 
 void
-gzochid_auth_identity_finalizer (gzochid_application_context *context,
-				 void *ptr)
+gzochid_auth_identity_finalizer (GzochidApplicationContext *context, void *ptr)
 {
   gzochid_auth_identity_unref (ptr);
 }
