@@ -1,5 +1,5 @@
 /* channel.c: Channel management routines for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -1177,8 +1177,8 @@ make_channel_binding (char *name)
   int name_len = strlen (name) + 1;
   char *binding = malloc (sizeof (char) * (prefix_len + name_len));
 
-  strncpy (binding, CHANNEL_PREFIX, prefix_len);
-  strncpy (binding + prefix_len, name, name_len);
+  memcpy (binding, CHANNEL_PREFIX, prefix_len);
+  memcpy (binding + prefix_len, name, name_len);
 
   return binding;
 }
