@@ -1,5 +1,5 @@
 /* channel.h: Prototypes and declarations for channel.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -41,8 +41,8 @@ gzochid_channel_operation_task_serialization;
   transaction is marked for rollback.
 */
 
-gzochid_channel *gzochid_channel_create (gzochid_application_context *, char *);
-gzochid_channel *gzochid_channel_get (gzochid_application_context *, char *);
+gzochid_channel *gzochid_channel_create (GzochidApplicationContext *, char *);
+gzochid_channel *gzochid_channel_get (GzochidApplicationContext *, char *);
 
 /*
   Initialize and return a new channel with the specified name. 
@@ -63,12 +63,12 @@ gzochid_channel *gzochid_channel_new (char *);
 void gzochid_channel_free (gzochid_channel *);
 
 void gzochid_channel_join 
-(gzochid_application_context *, gzochid_channel *, gzochid_client_session *);
+(GzochidApplicationContext *, gzochid_channel *, gzochid_client_session *);
 void gzochid_channel_leave 
-(gzochid_application_context *, gzochid_channel *, gzochid_client_session *);
+(GzochidApplicationContext *, gzochid_channel *, gzochid_client_session *);
 void gzochid_channel_send 
-(gzochid_application_context *, gzochid_channel *,  unsigned char *, short);
-void gzochid_channel_close (gzochid_application_context *, gzochid_channel *);
+(GzochidApplicationContext *, gzochid_channel *,  unsigned char *, short);
+void gzochid_channel_close (GzochidApplicationContext *, gzochid_channel *);
 
 enum
   {
@@ -99,7 +99,7 @@ GQuark gzochid_channel_error_quark (void);
    error if the session is not locally connected, or if it is already joined to
    the channel. */
 
-void gzochid_channel_join_direct (gzochid_application_context *, guint64,
+void gzochid_channel_join_direct (GzochidApplicationContext *, guint64,
 				  guint64, GError **);
 
 /* Non-transactionally removes the session with the specified oid from the
@@ -107,19 +107,19 @@ void gzochid_channel_join_direct (gzochid_application_context *, guint64,
    Signals an error if the session is not locally connected, or if it is not
    currently a member of the channel. */
 
-void gzochid_channel_leave_direct (gzochid_application_context *, guint64,
+void gzochid_channel_leave_direct (GzochidApplicationContext *, guint64,
 				   guint64, GError **);
 
 /* Non-transactionally closes the channel with the specified oid, qualified by
    the specified application. */
 
-void gzochid_channel_close_direct (gzochid_application_context *, guint64);
+void gzochid_channel_close_direct (GzochidApplicationContext *, guint64);
 
 /* Non-transactionally sends the specified message to all locally-connected
    sessions that are members of the session with the specified oid, qualified by
    the specified application. */
 
-void gzochid_channel_message_direct (gzochid_application_context *, guint64,
+void gzochid_channel_message_direct (GzochidApplicationContext *, guint64,
 				     GBytes *);
 
 
