@@ -171,9 +171,10 @@ gzochid_config_keyfile_extract_config (GKeyFile *key_file, const char *group)
 
   for (; i < num_keys; i++) 
     g_hash_table_insert 
-      (config, keys[i], g_key_file_get_value (key_file, group, keys[i], NULL));
+      (config, strdup (keys[i]),
+       g_key_file_get_value (key_file, group, keys[i], NULL));
   
-  free (keys);
+  g_strfreev (keys);
   return config;
 }
 
