@@ -1,5 +1,5 @@
 /* config.h: Prototypes and declarations for config.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -25,18 +25,6 @@
 
 #define GZOCHID_TYPE_CONFIGURATION gzochid_configuration_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_configuration_get_type (void);
-
-typedef struct _GzochidConfiguration GzochidConfiguration;
-
-struct _GzochidConfigurationClass
-{
-  GObjectClass parent_class;
-};
-
 /*
   `GzochidConfiguration' is the global configuration object for gzochid. New
   instances may be constructed by `g_object_new', with the following 
@@ -52,15 +40,8 @@ struct _GzochidConfigurationClass
   resolution context via `gzochid_resolver_provide'. 
 */
 
-typedef struct _GzochidConfigurationClass GzochidConfigurationClass;
-
-static inline GzochidConfiguration *
-GZOCHID_CONFIGURATION (gconstpointer ptr) {
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_configuration_get_type (), GzochidConfiguration);
-}
-
-/* End boilerplate. */
+G_DECLARE_FINAL_TYPE (GzochidConfiguration, gzochid_configuration, GZOCHID,
+		      CONFIGURATION, GObject);
 
 gboolean gzochid_config_to_boolean (char *, gboolean);
 int gzochid_config_to_int (char *, int);
