@@ -1,5 +1,5 @@
 /* data-protocol.c: Data structures, formats used by the gzochi meta dataserver
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -314,13 +314,15 @@ gzochid_data_protocol_response_read (GBytes *data)
 }
 
 gzochid_data_changeset *
-gzochid_data_changeset_new_with_free_func (const char *app, GArray *changes,
+gzochid_data_changeset_new_with_free_func (const char *app, guint64 id,
+					   GArray *changes,
 					   GDestroyNotify free_func)
 {
   gzochid_data_changeset *changeset =
     g_slice_alloc (sizeof (gzochid_data_changeset));
 
   changeset->app = strdup (app);
+  changeset->id = id;
   changeset->changes = g_array_ref (changes);
   changeset->free_func = free_func;
   
@@ -328,9 +330,9 @@ gzochid_data_changeset_new_with_free_func (const char *app, GArray *changes,
 }
 
 gzochid_data_changeset *
-gzochid_data_changeset_new (const char *app, GArray *changes)
+gzochid_data_changeset_new (const char *app, guint64 id, GArray *changes)
 {
-  return gzochid_data_changeset_new_with_free_func (app, changes, NULL);
+  return gzochid_data_changeset_new_with_free_func (app, id, changes, NULL);
 }
 
 /* Frees the memory held by the internal pointers for a change record. The 
@@ -444,6 +446,17 @@ change_read (const unsigned char *bytes, size_t len,
     }
 }
 
+/* Calculate and return the serialized size in bytes of the specified 
+   `gzochid_data_change' struct. */
+
+static size_t
+change_size (const gzochid_data_change *change)
+{
+  return strlen (change->store) + 1
+    + g_bytes_get_size (change->key)
+    + (change->delete ? 2 : g_bytes_get_size (change->data));
+}
+
 /* Serializes the specified change record to the specified byte array. */
 
 static void
@@ -461,11 +474,11 @@ change_write (gzochid_data_change *change, GByteArray *arr)
   else write_bytes (change->data, arr);
 }
 
-void
+size_t
 gzochid_data_protocol_changeset_write (gzochid_data_changeset *changeset,
 				       GByteArray *arr)
 {
-  int i = 0;
+  unsigned short i = 0;
   size_t len = 0;
   
   g_byte_array_append
@@ -473,20 +486,37 @@ gzochid_data_protocol_changeset_write (gzochid_data_changeset *changeset,
 
   len = arr->len;
 
+  /* Write the changeset id to the buffer. */
+  
+  g_byte_array_set_size (arr, len + 8);
+  gzochi_common_io_write_long (changeset->id, arr->data, len);
+  
+  len = arr->len;
+
   /* Grow the array by two bytes. */
   
   g_byte_array_set_size (arr, len + 2);
 
-  /* Write the object change count directly to the buffer. */
-  
-  gzochi_common_io_write_short (changeset->changes->len, arr->data, len);
+  /* Defer writing the number of changes in the message payload for a moment,
+     since we don't know for sure how many are going to fit. */
   
   for (; i < changeset->changes->len; i++)
     {
       gzochid_data_change *change = &g_array_index
 	(changeset->changes, gzochid_data_change, i);
+      size_t csize = change_size (change);
+
+      if (arr->len + csize > G_MAXUSHORT)
+	break;
+
       change_write (change, arr);
     }
+
+  /* Now go back and write the object change count directly to the buffer. */
+  
+  gzochi_common_io_write_short (i, arr->data, len);
+
+  return i;
 }
 
 gzochid_data_changeset *
@@ -498,7 +528,8 @@ gzochid_data_protocol_changeset_read (GBytes *data)
   const unsigned char *bytes = g_bytes_get_data (data, &len);
   GArray *changes = NULL;
   gzochid_data_changeset *changeset = NULL;
-
+  guint64 id = 0;
+  
   const char *app = gzochid_protocol_read_str (bytes, len, &offset);
   
   if (app == NULL || offset == 0)
@@ -506,9 +537,14 @@ gzochid_data_protocol_changeset_read (GBytes *data)
 
   len -= offset;
 
-  if (len <= 2)
+  if (len <= 10)
     return NULL;
 
+  id = gzochi_common_io_read_long (bytes, offset);
+
+  offset += 8;
+  len -= 8;
+  
   num_changes = gzochi_common_io_read_short (bytes, offset);
 
   offset += 2;
@@ -544,8 +580,7 @@ gzochid_data_protocol_changeset_read (GBytes *data)
     }
   else changes = g_array_new (FALSE, FALSE, sizeof (gzochid_data_change));
   
-  changeset = gzochid_data_changeset_new_with_free_func
-    (app, changes, (GDestroyNotify) change_clear);
+  changeset = gzochid_data_changeset_new (app, id, changes);
 
   /* Transfer ownership to the changeset record. */
   
