@@ -1,5 +1,5 @@
 /* data.c: Application data management routines for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -168,7 +168,7 @@ flush_reference (gzochid_data_managed_reference *reference,
 
       store->iface->transaction_put
 	(context->transaction, store->oids, (char *) &encoded_oid,
-	 sizeof (guint64), (char *) out->data, out->len);
+	 sizeof (guint64), out->data, out->len);
 
       g_object_get (context->app_context, "event-source", &event_source, NULL);
       
@@ -300,7 +300,7 @@ get_binding (gzochid_data_transaction_context *context, char *name,
 {
   size_t oid_bytes_len = 0;
   gzochid_application_store *store = get_store (context->app_context);
-  char *oid_bytes = store->iface->transaction_get 
+  unsigned char *oid_bytes = store->iface->transaction_get 
     (context->transaction, store->names, name, strlen (name) + 1,
      &oid_bytes_len);
 
@@ -344,8 +344,8 @@ set_binding (gzochid_data_transaction_context *context, char *name, guint64 oid,
   gzochid_application_store *store = get_store (context->app_context);
 
   store->iface->transaction_put 
-    (context->transaction, store->names, name, strlen (name) + 1, 
-     (char *) &encoded_oid, sizeof (guint64));
+    (context->transaction, store->names, name, strlen (name) + 1,
+     (unsigned char *) &encoded_oid, sizeof (guint64));
 
   if (context->transaction->rollback)
     {
@@ -524,7 +524,7 @@ dereference (gzochid_data_transaction_context *context,
 	     GError **err)
 {
   size_t data_len = 0;
-  char *data = NULL; 
+  unsigned char *data = NULL; 
   GByteArray *in = NULL;
   guint64 encoded_oid;
   GzochidApplicationContext *app_context = context->app_context;
@@ -964,7 +964,7 @@ gzochid_data_mark (GzochidApplicationContext *context,
       && reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW
       && reference->state != GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED)
     {
-      char *data = NULL;
+      unsigned char *data = NULL;
       guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
       gzochid_application_store *store = get_store (context);
       
