@@ -1,5 +1,5 @@
 /* dataclient-protocol.c: Implementation of dataclient protocol.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -198,11 +198,12 @@ client_dispatch (const GByteArray *buffer, gpointer user_data)
   return total;
 }
 
-/* The client error handler. This is currently a no-op. */
+/* The client error handler. */
 
 static void
 client_error (gpointer user_data)
 {
+  gzochid_dataclient_disconnected (user_data);
 }
 
 /*
