@@ -1,5 +1,5 @@
 /* dataclient-protocol.h: Prototypes and declarations for dataclient-protocol.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@
 
 /* A `gzochid_client_protocol' implementation for the dataclient protocol. */
 
-gzochid_client_protocol gzochid_dataclient_client_protocol;
+extern gzochid_client_protocol gzochid_dataclient_client_protocol;
 
 #endif /* GZOCHID_DATACLIENT_PROTOCOL_H */
