@@ -1,5 +1,5 @@
 /* dataclient.c: Dataserver client for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -47,6 +47,11 @@
 
 struct _dataclient_callback_registration
 {
+  /* Reference to the enclosing client, for access to the release callback 
+     queue. */
+
+  struct _GzochidDataClient *client; 
+  
   /* The expected opcode of the response. Used as a kind of check bit to ensure
      that the correct response is being processed on behalf of a request. */
 
@@ -101,7 +106,7 @@ struct _dataclient_callback_queue
 {  
   GMutex mutex; /* Mutex to protect the list of registrations */
 
-  /* List of `dataclient_oids_callback_resitration' objects. */
+  /* List of `dataclient_oids_callback_registration' objects. */
   
   GList *oids_callback_registrations;
   
@@ -137,6 +142,14 @@ struct _GzochidDataClient
   GHashTable *application_callback_queues; 
 
   GMutex queue_mutex; /* Protects the callback queue table. */
+
+  /* A `GList' of `dataclient_callback_registration' instances corresponding to
+     outstanding lock release callbacks, for the purpose of explicit release 
+     during disconnect handling. */
+  
+  GList *lock_release_callbacks; 
+
+  GMutex lock_release_mutex; /* Protects the release callback list. */
   
   /* A `GMainContext' for scheduling tasks (such as timed lock releases) to be 
      run by the metaclient while connected to the server. This field is 
@@ -150,6 +163,56 @@ struct _GzochidDataClient
   gzochid_reconnectable_socket *socket;
 };
 
+/* Create and return a new callback registration object with the specified
+   expected opcode and success, failure, and release callbacks (with associated
+   user data pointers. This object should freed via `free_callback' when no 
+   longer needed. */
+
+static dataclient_callback_registration *
+create_callback
+(GzochidDataClient *client, unsigned char expected_opcode,
+ gzochid_dataclient_success_callback success_callback, gpointer success_data,
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
+{
+  dataclient_callback_registration *registration =
+    malloc (sizeof (dataclient_callback_registration));
+
+  registration->client = g_object_ref (client);
+  registration->expected_opcode = expected_opcode;
+  
+  registration->success_callback = success_callback;
+  registration->success_data = success_data;
+
+  registration->failure_callback = failure_callback;
+  registration->failure_data = failure_data;
+
+  /* This initial release timeout is a sentinel value of sorts indicating that
+     the lock has not yet be acquired. */
+  
+  registration->timeout = G_MAXINT64;
+  
+  registration->release_callback = release_callback;
+  registration->release_data = release_data;
+  
+  return registration;
+}
+
+/*
+  Clean up the resources associated with the specified 
+  `dataclient_callback_registration' instance, which must have been created via
+  `create_callback'.
+*/
+
+static void
+free_callback (gpointer data)
+{
+  dataclient_callback_registration *registration = data;
+  
+  g_object_unref (registration->client);
+  free (registration);
+}
+
 /* Boilerplate setup for the data client object. */
 
 G_DEFINE_TYPE (GzochidDataClient, gzochid_data_client, G_TYPE_OBJECT);
@@ -215,8 +278,10 @@ gzochid_data_client_finalize (GObject *gobject)
   GzochidDataClient *client = GZOCHID_DATA_CLIENT (gobject);
   
   g_hash_table_destroy (client->application_callback_queues);
-
+  g_list_free_full (client->lock_release_callbacks, free_callback);
+  
   g_mutex_clear (&client->queue_mutex);
+  g_mutex_clear (&client->lock_release_mutex);
 
   G_OBJECT_CLASS (gzochid_data_client_parent_class)->finalize (gobject);  
 }
@@ -258,40 +323,6 @@ gzochid_data_client_class_init (GzochidDataClientClass *klass)
     (object_class, N_PROPERTIES, obj_properties);
 }
 
-/* Create and return a new callback registration object with the specified
-   expected opcode and success, failure, and release callbacks (with associated
-   user data pointers. This object should freed via `free' when no longer 
-   needed. */
-
-static dataclient_callback_registration *
-create_callback
-(unsigned char expected_opcode,
- gzochid_dataclient_success_callback success_callback, gpointer success_data,
- gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
- gzochid_dataclient_release_callback release_callback, gpointer release_data)
-{
-  dataclient_callback_registration *registration =
-    malloc (sizeof (dataclient_callback_registration));
-
-  registration->expected_opcode = expected_opcode;
-  
-  registration->success_callback = success_callback;
-  registration->success_data = success_data;
-
-  registration->failure_callback = failure_callback;
-  registration->failure_data = failure_data;
-
-  /* This initial release timeout is a sentinel value of sorts indicating that
-     the lock has not yet be acquired. */
-  
-  registration->timeout = G_MAXINT64;
-  
-  registration->release_callback = release_callback;
-  registration->release_data = release_data;
-  
-  return registration;
-}
-
 /* Create and return a new oid callback registration object with the specified
    callback function and associated user data pointer. This object should freed
    via `free' when no longer needed. */
@@ -316,7 +347,7 @@ free_callback_queue (dataclient_callback_queue *queue)
 {
   g_mutex_clear (&queue->mutex);
   g_list_free_full (queue->oids_callback_registrations, free);
-  g_list_free_full (queue->callback_registrations, free);
+  g_list_free_full (queue->callback_registrations, free_callback);
   free (queue);
 }
 
@@ -370,9 +401,10 @@ static void
 gzochid_data_client_init (GzochidDataClient *self)
 {
   g_mutex_init (&self->queue_mutex);
+  g_mutex_init (&self->lock_release_mutex);
+  
   self->application_callback_queues = g_hash_table_new_full
     (g_str_hash, g_str_equal, free, (GDestroyNotify) free_callback_queue);
-  
 }
 
 /* End boilerplate. */
@@ -483,7 +515,14 @@ static gboolean
 invoke_release_callback (gpointer user_data)
 {
   dataclient_callback_registration *registration = user_data;
-  registration->release_callback (registration->release_data);  
+  GzochidDataClient *client = registration->client;
+  
+  g_mutex_lock (&client->lock_release_mutex);
+  registration->release_callback (registration->release_data);
+  client->lock_release_callbacks = g_list_remove
+    (client->lock_release_callbacks, registration);
+  g_mutex_unlock (&client->lock_release_mutex);
+
   return FALSE;
 }
 
@@ -512,7 +551,7 @@ process_queued_callback (GzochidDataClient *client,
       g_warning
 	("Received response %d for %s/%s; expected response %d.", opcode,
 	 response->app, response->store, callbacks->expected_opcode);
-      free (callbacks);
+      free_callback (callbacks);
     }
   else if (response->success)
     {
@@ -540,7 +579,8 @@ process_queued_callback (GzochidDataClient *client,
       if (release_callback != NULL)
 	{
 	  g_source_set_callback
-	    (release_callback, invoke_release_callback, callbacks, free);
+	    (release_callback, invoke_release_callback, callbacks,
+	     free_callback);
 	  g_source_attach (release_callback, client->main_context);
 	}
     }
@@ -549,7 +589,7 @@ process_queued_callback (GzochidDataClient *client,
       g_debug
 	("Lock request against %s/%s failed.", response->app, response->store);
       callbacks->failure_callback (response->timeout, callbacks->failure_data);
-      free (callbacks);
+      free_callback (callbacks);
     }
 }
 
@@ -618,7 +658,7 @@ gzochid_dataclient_request_value
   
   queue->callback_registrations = g_list_append
     (queue->callback_registrations,
-     create_callback (GZOCHID_DATA_PROTOCOL_VALUE_RESPONSE,
+     create_callback (client, GZOCHID_DATA_PROTOCOL_VALUE_RESPONSE,
 		      success_callback, success_data,
 		      failure_callback, failure_data,
 		      release_callback, release_data));
@@ -643,6 +683,93 @@ gzochid_dataclient_received_next_key (GzochidDataClient *client,
   release_callback_queue (queue);
 }
 
+/*
+  A `GFunc' implementation for invoking the failure callback for a outstanding
+  request without waiting for a response from the data server. This function
+  is called as part of the cleanup process for handling disconnects; the queue
+  mutex for the relevant callback queue should be held by the caller.
+*/
+
+static void
+invoke_failure_callback_premature (gpointer data, gpointer user_data)
+{
+  struct timeval timeout = { 0, 0 };
+  dataclient_callback_registration *registration = data;
+
+  registration->failure_callback (timeout, registration->failure_data);
+}
+
+/*
+  A `GFunc' implementation for invoking the lock release timeout callback for a
+  previously-obtained lock without waiting for the callback to expire naturally.
+  This function is called as part of the cleanup process for handling 
+  disconnects; the queue mutex for the relevant callback queue should be held by
+  the caller.
+*/
+
+static void
+invoke_release_callback_premature (gpointer data, gpointer user_data)
+{
+  dataclient_callback_registration *registration = data;
+  GzochidDataClient *client = user_data;
+  GSource *source = NULL;
+
+  registration->release_callback (registration->release_data);
+
+  source = g_main_context_find_source_by_user_data
+    (client->main_context, registration);
+  
+  if (source != NULL)
+    g_source_destroy (source);
+}
+
+/*
+  A `GHFunc' implementation for clearing out each application callback queue
+  known to the client, as part of the cleanup process for handling disconnects;
+  the queue mutex for the relevant callback queue should be held by the caller.
+*/
+
+static void
+clear_callback_queue (gpointer key, gpointer value, gpointer user_data)
+{
+  dataclient_callback_queue *queue = value;
+  
+  g_mutex_lock (&queue->mutex);
+
+  g_list_free_full (queue->oids_callback_registrations, free);
+  queue->oids_callback_registrations = NULL;
+
+  g_list_foreach
+    (queue->callback_registrations, invoke_failure_callback_premature, NULL);
+  g_list_free_full (queue->callback_registrations, free_callback);
+  queue->callback_registrations = NULL;
+
+  g_mutex_unlock (&queue->mutex);
+}
+
+void
+gzochid_dataclient_disconnected (GzochidDataClient *client)
+{
+  g_warning
+    ("Disconnected from the data server; clearing callbacks and locks.");
+
+  /* Clear out the application-level callback queues. */
+  
+  g_mutex_lock (&client->queue_mutex);
+  g_hash_table_foreach
+    (client->application_callback_queues, clear_callback_queue, client);
+  g_mutex_unlock (&client->queue_mutex);
+
+  /* Force all of the active locks to release. */
+
+  g_mutex_lock (&client->lock_release_mutex);
+  g_list_foreach
+    (client->lock_release_callbacks, invoke_release_callback_premature, client);
+  g_list_free_full (client->lock_release_callbacks, free_callback);
+  client->lock_release_callbacks = NULL;
+  g_mutex_unlock (&client->lock_release_mutex);
+}
+
 /* Convenience function to append a length-prefixed array of bytes to the
    specified `GByteArray', or a pair of `NULL' bytes if the specified byte
    buffer is `NULL'. */
@@ -707,7 +834,7 @@ gzochid_dataclient_request_next_key
 
   queue->callback_registrations = g_list_append
     (queue->callback_registrations,
-     create_callback (GZOCHID_DATA_PROTOCOL_NEXT_KEY_RESPONSE,
+     create_callback (client, GZOCHID_DATA_PROTOCOL_NEXT_KEY_RESPONSE,
 		      success_callback, success_data,
 		      failure_callback, failure_data,
 		      release_callback, release_data));
