@@ -1,5 +1,5 @@
 /* dataserver-protocol.c: Implementation of dataserver protocol.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -532,6 +532,11 @@ dispatch_end_changeset (gzochi_metad_dataserver_client *client,
 	    ("Failed to complete changeset submission: %s", err->message);
 	  g_error_free (err);
 
+	  /* If the client submitted an un-committable transaction, we need to
+	     disconnect them because they're going to be in an inconsistent
+	     state from this point forward. */
+	  
+	  gzochid_client_socket_close (client->sock);
 	  return FALSE;
 	}
       else return TRUE;
