@@ -1,5 +1,5 @@
 /* dataserver.c: Data server for gzochi-metad
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -158,14 +158,7 @@ gzochi_metad_data_server_finalize (GObject *gobject)
   g_hash_table_destroy (server->application_stores);
 
   if (server->storage_engine != NULL)
-    {
-      if (server->storage_engine->handle == NULL)
-
-	/* A `NULL' handle indicates a "synthetic" storage engine (i.e., the
-	   memory engine). */
-	
-	free (server->storage_engine);
-    }
+    free (server->storage_engine);
 
   G_OBJECT_CLASS (gzochi_metad_data_server_parent_class)->finalize (gobject);
 }
@@ -194,18 +187,16 @@ gzochi_metad_data_server_class_init (GzochiMetadDataServerClass *klass)
 
   obj_properties[PROP_CONFIGURATION] = g_param_spec_object
     ("configuration", "config", "The global configuration object",
-     GZOCHID_TYPE_CONFIGURATION,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+     GZOCHID_TYPE_CONFIGURATION, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
 
   obj_properties[PROP_RESOLUTION_CONTEXT] = g_param_spec_object
     ("resolution-context", "resolution-context",
      "The global resolution context", GZOCHID_TYPE_RESOLUTION_CONTEXT,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
   
   obj_properties[PROP_SOCKET_SERVER] = g_param_spec_object
     ("socket-server", "socket-server", "The global socket server",
-     GZOCHID_TYPE_SOCKET_SERVER,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+     GZOCHID_TYPE_SOCKET_SERVER, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
 
   g_object_class_install_properties
     (object_class, N_PROPERTIES, obj_properties);
@@ -386,7 +377,7 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
       size_t data_len = 0;
       gzochid_storage_transaction *transaction = app_store->store->iface
 	->transaction_begin (app_store->store->storage_context);
-      char *data = app_store->store->iface->transaction_get
+      unsigned char *data = app_store->store->iface->transaction_get
 	(transaction, store, (char *) g_bytes_get_data (key, NULL),
 	 g_bytes_get_size (key), &data_len);
 
@@ -663,7 +654,7 @@ gzochi_metad_dataserver_end_changeset (GzochiMetadDataServer *server,
 	       (transaction, store,
 		(char *) g_bytes_get_data (change.key, NULL),
 		g_bytes_get_size (change.key),
-		(char *) g_bytes_get_data (change.data, NULL),
+		(unsigned char *) g_bytes_get_data (change.data, NULL),
 		g_bytes_get_size (change.data));
 
 	if (transaction->rollback)
