@@ -134,6 +134,45 @@ descriptor_start_element (GMarkupParseContext *context,
 	  else descriptor->auth_type = strdup (type);
 	}
     }
+  else if (strcmp (element_name, "logging") == 0)
+    {
+      if (parent == NULL || strcmp (parent, "game") != 0)
+	*error = g_error_new 
+	  (G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT, 
+	   "Invalid position for 'logging' element.");
+      else 
+	{
+	  const gchar *log_level = find_attribute_value 
+	    ("threshold", attribute_names, attribute_values);
+
+	  if (log_level != NULL)
+	    {
+	      if (strcmp (log_level, "EMERG") == 0 ||
+		  strcmp (log_level, "ALERT") == 0 ||
+		  strcmp (log_level, "CRIT") == 0)
+		descriptor->log_level = G_LOG_LEVEL_ERROR;	      
+	      else if (strcmp (log_level, "ERR") == 0)
+		descriptor->log_level = G_LOG_LEVEL_CRITICAL;
+	      else if (strcmp (log_level, "WARNING") == 0)
+		descriptor->log_level = G_LOG_LEVEL_WARNING;
+	      else if (strcmp (log_level, "NOTICE") == 0)
+		descriptor->log_level = G_LOG_LEVEL_MESSAGE;
+	      else if (strcmp (log_level, "INFO") == 0)
+	        descriptor->log_level = G_LOG_LEVEL_INFO;
+	      else if (strcmp (log_level, "DEBUG") == 0)
+		descriptor->log_level = G_LOG_LEVEL_DEBUG;
+	      else *error = g_error_new
+		     (G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+		      "Invalid value '%s' for log_level.", log_level);
+	    }
+
+	  const gchar *filename = find_attribute_value 
+	    ("file", attribute_names, attribute_values);
+
+	  if (filename != NULL && strlen (filename) > 0)
+	    descriptor->log_filename = strdup (filename);
+	}
+    }
   else if (strcmp (element_name, "initialized") == 0
 	   || strcmp (element_name, "logged-in") == 0
 	   || strcmp (element_name, "ready") == 0)
@@ -282,6 +321,8 @@ application_descriptor_finalize (GObject *object)
 
   if (self->initialized != NULL)
     gzochid_application_callback_free (self->initialized);
+  if (self->log_filename != NULL)
+    free (self->log_filename);
   if (self->logged_in != NULL)
     gzochid_application_callback_free (self->logged_in);
   if (self->ready != NULL)
@@ -309,6 +350,8 @@ gzochid_application_descriptor_class_init
 static void
 gzochid_application_descriptor_init (GzochidApplicationDescriptor *self)
 {
+  self->log_level = G_LOG_LEVEL_MESSAGE;
+  
   self->properties = g_hash_table_new_full
     (g_str_hash, g_str_equal, (GDestroyNotify) free, (GDestroyNotify) free);
   self->auth_properties = g_hash_table_new_full
