@@ -1,5 +1,5 @@
 /* descriptor.h: Prototypes and declarations for descriptor.c
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -49,7 +49,16 @@ struct _GzochidApplicationDescriptor
   char *deployment_root;
   
   GList *load_paths; /* Descriptor-specified module load paths. */
- 
+
+  /* The log message priority threshold for the application. Defaults to 
+     `G_LOG_LEVEL_MESSAGE'. */
+
+  GLogLevelFlags log_level;
+  
+  /* The relative or absolute full path to the log file for the application. */
+
+  char *log_filename; 
+  
   gzochid_application_callback *initialized;
   gzochid_application_callback *logged_in;
   gzochid_application_callback *ready;
