@@ -1,5 +1,5 @@
 /* game-protocol.h: Prototypes and declarations for game-protocol.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -43,11 +43,11 @@ extern gzochid_client_protocol gzochid_game_client_protocol;
 typedef struct _gzochid_game_protocol_closure gzochid_game_protocol_closure;
 
 /* Construct and return a new `gzochid_game_protocol_closure' around the 
-   specified `GzochidGameServer', `gzochid_task_queue', and task execution 
+   specified `GzochidGameServer', `GzochidTaskQueue', and task execution 
    timeout value. */   
 
 gzochid_game_protocol_closure *gzochid_game_protocol_create_closure
-(GzochidGameServer *, gzochid_task_queue *, struct timeval);
+(GzochidGameServer *, GzochidTaskQueue *, struct timeval);
 
 /* A struct representing a connected gzochi game application client. 
    `gzochid_game_client' instances are created and managed by the protocol. */
