@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -164,6 +164,8 @@ game_server_dispose (GObject *obj)
   g_object_unref (self->auth_plugin_registry);
   g_object_unref (self->configuration);
   g_object_unref (self->event_loop);
+  g_object_unref (self->metaclient_container);
+  g_object_unref (self->resolution_context);
   g_object_unref (self->socket_server);
   g_object_unref (self->storage_engine_container);
   g_object_unref (self->task_queue);
@@ -177,6 +179,8 @@ game_server_finalize (GObject *obj)
   GzochidGameServer *self = GZOCHID_GAME_SERVER (obj);
 
   g_hash_table_destroy (self->applications);
+  free (self->apps_dir);
+  free (self->work_dir);
 
   G_OBJECT_CLASS (gzochid_game_server_parent_class)->finalize (obj);
 }
