@@ -1,5 +1,5 @@
 /* game.h: Prototypes and declarations for game.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -27,28 +27,8 @@
 
 #define GZOCHID_TYPE_GAME_SERVER gzochid_game_server_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_game_server_get_type (void);
-
-typedef struct _GzochidGameServer GzochidGameServer;
-
-struct _GzochidGameServerClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochidGameServerClass GzochidGameServerClass;
-
-static inline GzochidGameServer *
-GZOCHID_GAME_SERVER (gconstpointer ptr)
-{
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_game_server_get_type (), GzochidGameServer);
-}
-
-/* End boilerplate. */
+G_DECLARE_FINAL_TYPE (GzochidGameServer, gzochid_game_server, GZOCHID,
+                      GAME_SERVER, GObject);
 
 /*
   Starts the specified game server, preparing it to begin servicing requests 
@@ -65,7 +45,7 @@ GZOCHID_GAME_SERVER (gconstpointer ptr)
 
 void gzochid_game_server_start (GzochidGameServer *, GError **);
 
-gzochid_application_context *gzochid_game_server_lookup_application
+GzochidApplicationContext *gzochid_game_server_lookup_application
 (GzochidGameServer *, const char *);
 GList *gzochid_game_server_get_applications (GzochidGameServer *);
 
