@@ -1,5 +1,5 @@
 /* gzochi-load.c: Utility for importing data from game application databases
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ struct load_context
 
 struct datum
 {
-  char *data;
+  unsigned char *data;
   size_t data_len;
 };
 
@@ -79,15 +79,15 @@ setup_load_context (gzochid_storage_engine *engine, char *data_dir, char *db,
 }
 
 static void
-load_data_header (char *line, size_t line_len, int line_num)
+load_data_header (unsigned char *line, size_t line_len, int line_num)
 {
 }
 
-static char *
+static unsigned char *
 pack_line (char *line, size_t len, int line_num)
 {
   size_t i = 0;
-  char *packed_line = calloc (len / 2, sizeof (char));
+  unsigned char *packed_line = calloc (len / 2, sizeof (unsigned char));
 
   for (; i < len; i += 2)
     if (sscanf (line + i, "%2hhx", &packed_line[i / 2]) != 1)
@@ -137,8 +137,9 @@ load_data (gzochid_storage_engine *engine, char *data_dir, char *db,
   struct load_context *context = 
     setup_load_context (engine, data_dir, db, force);
 
-  while ((line.data_len = getline (&line.data, &line.data_len, stdin)) != -1 
-	 && strcmp (line.data, "HEADER=END\n") != 0)
+  while ((line.data_len =
+	  getline ((char **) &line.data, &line.data_len, stdin)) != -1 
+	 && strcmp ((char *) line.data, "HEADER=END\n") != 0)
     {
       load_data_header (line.data, line.data_len, line_num++);
       
@@ -155,12 +156,13 @@ load_data (gzochid_storage_engine *engine, char *data_dir, char *db,
 
   line_num++;
 
-  while ((line.data_len = getline (&line.data, &line.data_len, stdin)) != -1 
-	 && strcmp (line.data, "DATA=END\n") != 0)
+  while ((line.data_len =
+	  getline ((char **) &line.data, &line.data_len, stdin)) != -1 
+	 && strcmp ((char *) line.data, "DATA=END\n") != 0)
     {
       if (reading_key)
 	{
-	  key = read_line (line.data, line.data_len, line_num++);
+	  key = read_line ((char *) line.data, line.data_len, line_num++);
 	  reading_key = FALSE;
 	}
       else 
@@ -168,10 +170,10 @@ load_data (gzochid_storage_engine *engine, char *data_dir, char *db,
 	  gzochid_storage_transaction *tx = context->engine->interface
 	    ->transaction_begin (context->storage_context);
 
-	  value = read_line (line.data, line.data_len, line_num++);
+	  value = read_line ((char *) line.data, line.data_len, line_num++);
 	  
 	  context->engine->interface->transaction_put 
-	    (tx, context->store, key.data, key.data_len, value.data, 
+	    (tx, context->store, (char *) key.data, key.data_len, value.data, 
 	     value.data_len);
 
 	  context->engine->interface->transaction_prepare (tx);
