@@ -1,5 +1,5 @@
 /* gzochi-metad.c: Main server bootstrapping routines for gzochi-metad
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 #include <getopt.h>
 #include <glib.h>
 #include <glib-object.h>
+#ifndef WIN32
+#include <glib-unix.h>
+#endif /* WIN32 */
 #include <libintl.h>
 #include <locale.h>
 #include <stdio.h>
@@ -228,8 +231,15 @@ root_context_dispose (GObject *object)
 {
   GzochiMetadRootContext *root_context = GZOCHI_METAD_ROOT_CONTEXT (object);
 
+  g_object_unref (root_context->channel_server);
   g_object_unref (root_context->configuration);
-
+  g_object_unref (root_context->data_server);
+  g_object_unref (root_context->event_loop);
+  g_object_unref (root_context->http_server);
+  g_object_unref (root_context->session_server);
+  g_object_unref (root_context->socket_server);
+  g_object_unref (root_context->task_server);
+  
   G_OBJECT_CLASS (gzochi_metad_root_context_parent_class)->dispose (object);
 }
 
@@ -331,7 +341,7 @@ Copyright (C) %s Julian Graham\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"),
-	  "2018");
+	  "2019");
 }
 
 /* Prints help and configuration options to standard output. */
@@ -398,13 +408,34 @@ initialize_logging (GKeyFile *key_file)
   g_hash_table_destroy (log_config);
 }
 
-/* Configure and start the admin web console if it's enabled. */
+static gboolean
+sigterm_handler (gpointer user_data)
+{
+  GzochiMetadRootContext *root_context = user_data;
+
+  g_main_loop_quit (root_context->socket_server->main_loop);  
+  return TRUE;
+}
 
 static void
+initialize_signal_handlers (GzochiMetadRootContext *root_context)
+{
+#ifndef WIN32
+  GSource *sigterm_source = g_unix_signal_source_new (SIGTERM);
+
+  g_source_set_callback (sigterm_source, sigterm_handler, root_context, NULL);
+  g_source_attach (sigterm_source, root_context->socket_server->main_context);
+#endif /* WIN32 */
+}
+
+/* Configure and start the admin web console if it's enabled. */
+
+static gpointer
 initialize_httpd (GzochidHttpServer *http_server,
 		  gzochid_event_source *root_event_source,
 		  GzochidResolutionContext *res_context, GKeyFile *key_file)
 {
+  gpointer httpd_state = NULL;
   GHashTable *admin_config =
     gzochid_config_keyfile_extract_config (key_file, "admin");
 
@@ -415,7 +446,7 @@ initialize_httpd (GzochidHttpServer *http_server,
       int port = gzochid_config_to_int
 	(g_hash_table_lookup (admin_config, "module.httpd.port"), 8800);
 
-      gzochid_httpd_meta_register_handlers
+      httpd_state = gzochid_httpd_meta_register_handlers
 	(http_server, root_event_source, res_context);
       gzochid_http_server_start (http_server, port, &err);
 
@@ -428,6 +459,7 @@ initialize_httpd (GzochidHttpServer *http_server,
     }
   
   g_hash_table_destroy (admin_config);
+  return httpd_state;
 }
 
 int 
@@ -443,6 +475,7 @@ main (int argc, char *argv[])
   GzochidConfiguration *configuration = NULL;
   GzochidResolutionContext *resolution_context = NULL;
   GzochiMetadRootContext *root_context = NULL;
+  gpointer httpd_state = NULL;
   
   setlocale (LC_ALL, "");
 
@@ -506,11 +539,16 @@ main (int argc, char *argv[])
       g_error ("Failed to bootstrap gzochi-metad: %s", err->message);
       exit (1);
     }
-
+  
   gzochid_event_loop_start (root_context->event_loop);
-  initialize_httpd (root_context->http_server, root_context->event_source,
-		    resolution_context, key_file);
+  httpd_state = initialize_httpd
+    (root_context->http_server, root_context->event_source, resolution_context,
+     key_file);
 
+  g_object_unref (resolution_context);
+  g_object_unref (configuration);
+  g_key_file_unref (key_file);
+  
   if (root_context->http_server != NULL)
     root_context->admin_server_base_url =
       strdup (gzochid_http_server_get_base_url (root_context->http_server));
@@ -521,8 +559,21 @@ main (int argc, char *argv[])
   gzochid_server_socket_listen
     (root_context->socket_server, root_context->server_socket,
      root_context->port);
+
+  initialize_signal_handlers (root_context);
   
   g_main_loop_run (root_context->socket_server->main_loop);
+
+  g_info ("Shutting down.");  
+
+  gzochid_http_server_stop (root_context->http_server);
+  free (root_context->admin_server_base_url);
+
+  if (httpd_state != NULL)
+    gzochid_httpd_meta_unregister_handlers (httpd_state);
   
+  gzochid_event_loop_stop (root_context->event_loop);
+  g_object_unref (root_context);
+
   return 0;
 }
