@@ -1,5 +1,5 @@
 /* gzochid-migrate.c: Utility for migrating game data schema
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -330,8 +330,6 @@ initialize_scheme_bindings ()
 {
   SCM gpd = scm_c_resolve_module ("gzochi private data");
   SCM gpdm = scm_c_resolve_module ("gzochi private data migration");
-  SCM scm_enqueue_oids = scm_c_make_gsubr 
-    ("enqueue-oids!", 2, 0, 0, enqueue_oids);
 
   gzochid_guile_init ();
   gzochid_scheme_initialize_bindings ();
@@ -343,13 +341,14 @@ initialize_scheme_bindings ()
   scm_gzochi_visit_object = 
     scm_variable_ref (scm_c_module_lookup (gpdm, "gzochi:visit-object"));
 
-  scm_variable_set_x 
-    (scm_c_module_lookup (gpdm, "enqueue-oids!"), scm_enqueue_oids);
+  scm_call_1
+    (scm_c_public_ref
+     ("gzochi private data migration", "gzochi:set-primitive-enqueue-oids!"),
+     scm_c_make_gsubr ("enqueue-oids!", 2, 0, 0, enqueue_oids));
 
   scm_gc_protect_object (scm_push_type_registry_x);
   scm_gc_protect_object (scm_pop_type_registry_x);
   scm_gc_protect_object (scm_gzochi_visit_object);
-  scm_gc_protect_object (scm_enqueue_oids);
 }
 
 static gzochid_storage_store *
@@ -743,7 +742,7 @@ migrate_object (struct migration *m, guint64 oid)
 
   gzochid_storage_transaction *tx = NULL;
   guint64 encoded_oid = gzochid_util_encode_oid (oid);
-  char *val = NULL;
+  unsigned char *val = NULL;
 
   args[0] = m;
   args[1] = &oid;
@@ -757,7 +756,8 @@ migrate_object (struct migration *m, guint64 oid)
   if (val == NULL)
     {
       app_store->iface->transaction_put
-	(tx, m->visited_oids, (char *) &encoded_oid, sizeof (guint64), "", 1);
+	(tx, m->visited_oids, (char *) &encoded_oid, sizeof (guint64),
+	 (unsigned char *) "", 1);
       if (gzochid_transaction_execute (migrate_object_tx, args) 
 	  != GZOCHID_TRANSACTION_SUCCESS && !m->explicit_rollback)
 	{
@@ -834,7 +834,7 @@ run_migration (struct migration *m)
 		->transaction_begin (app_store->storage_context);
 
 	      size_t oid_len = 0;
-	      char *oid_bytes = NULL;
+	      unsigned char *oid_bytes = NULL;
 	      guint64 encoded_oid = 0;
 	      
 	      oid_bytes = app_store->iface->transaction_get 
