@@ -1,5 +1,5 @@
 /* gzochid-storage.h: Public SPI for gzochid storage engine system
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -85,15 +85,15 @@ struct _gzochid_storage_engine_interface
   void (*transaction_commit) (gzochid_storage_transaction *);
   void (*transaction_rollback) (gzochid_storage_transaction *);
   void (*transaction_prepare) (gzochid_storage_transaction *);
-  char *(*transaction_get) 
+  unsigned char *(*transaction_get) 
     (gzochid_storage_transaction *, gzochid_storage_store *, char *, size_t, 
      size_t *);
-  char *(*transaction_get_for_update)
+  unsigned char *(*transaction_get_for_update)
     (gzochid_storage_transaction *, gzochid_storage_store *, char *, size_t, 
      size_t *);
   void (*transaction_put)
     (gzochid_storage_transaction *, gzochid_storage_store *, char *, size_t, 
-     char *, size_t);
+     unsigned char *, size_t);
   int (*transaction_delete)
     (gzochid_storage_transaction *, gzochid_storage_store *, char *, size_t);
   char *(*transaction_first_key)
