@@ -1,5 +1,5 @@
 /* gzochid.c: Main server bootstrapping routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,21 +19,28 @@
 #include <getopt.h>
 #include <glib.h>
 #include <glib-object.h>
+#ifndef WIN32
+#include <glib-unix.h>
+#endif /* WIN32 */
 #include <locale.h>
 #include <libintl.h>
+#include <libguile.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
 
-#include "admin.h"
+#include "api/admin.h"
 #include "config.h"
+#include "debug.h"
+#include "httpd-app.h"
 #include "metaclient.h"
 #include "event.h"
 #include "game.h"
 #include "guile.h"
 #include "gzochid.h"
 #include "log.h"
+#include "socket.h"
 #include "threads.h"
 
 #define _(String) gettext (String)
@@ -50,8 +57,9 @@ G_DEFINE_TYPE (GzochidRootContext, gzochid_root_context, G_TYPE_OBJECT);
 enum gzochid_root_context_properties
   {
     PROP_CONFIGURATION = 1,
-    PROP_RESOLUTION_CONTEXT,
+    PROP_DEBUG_SERVER,
     PROP_EVENT_LOOP,
+    PROP_HTTP_SERVER,
     PROP_SOCKET_SERVER,
     PROP_META_CLIENT_CONTAINER,
     PROP_GAME_SERVER,
@@ -60,6 +68,22 @@ enum gzochid_root_context_properties
 
 static GParamSpec *obj_properties[N_PROPERTIES] = { NULL };
 
+static void
+root_context_dispose (GObject *object)
+{
+  GzochidRootContext *root_context = GZOCHID_ROOT_CONTEXT (object);
+
+  g_object_unref (root_context->configuration);
+  g_object_unref (root_context->debug_server);
+  g_object_unref (root_context->event_loop);
+  g_object_unref (root_context->http_server);
+  g_object_unref (root_context->socket_server);
+  g_object_unref (root_context->metaclient_container);
+  g_object_unref (root_context->game_server);
+  
+  G_OBJECT_CLASS (gzochid_root_context_parent_class)->dispose (object);
+}
+
 static void
 root_context_set_property (GObject *object, guint property_id,
 			   const GValue *value, GParamSpec *pspec)
@@ -72,8 +96,12 @@ root_context_set_property (GObject *object, guint property_id,
       self->configuration = g_object_ref (g_value_get_object (value));
       break;
 
-    case PROP_RESOLUTION_CONTEXT:
-      self->resolution_context = g_object_ref_sink (g_value_get_object (value));
+    case PROP_DEBUG_SERVER:
+      self->debug_server = g_object_ref (g_value_get_object (value));
+      break;
+
+    case PROP_HTTP_SERVER:
+      self->http_server = g_object_ref (g_value_get_object (value));
       break;
 
     case PROP_EVENT_LOOP:
@@ -103,21 +131,25 @@ gzochid_root_context_class_init (GzochidRootContextClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->dispose = root_context_dispose;
   object_class->set_property = root_context_set_property;
 
   obj_properties[PROP_CONFIGURATION] = g_param_spec_object
     ("configuration", "configuration", "The server configuration",
      GZOCHID_TYPE_CONFIGURATION, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
-  obj_properties[PROP_RESOLUTION_CONTEXT] = g_param_spec_object
-    ("resolution-context", "resolutuon-context", "The root resolution context",
-     GZOCHID_TYPE_RESOLUTION_CONTEXT,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  obj_properties[PROP_DEBUG_SERVER] = g_param_spec_object
+    ("debug-server", "debugger", "The admin debugging server",
+     GZOCHID_TYPE_DEBUG_SERVER, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
   obj_properties[PROP_EVENT_LOOP] = g_param_spec_object
     ("event-loop", "event-loop", "The global event loop",
      GZOCHID_TYPE_EVENT_LOOP, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
   
+  obj_properties[PROP_HTTP_SERVER] = g_param_spec_object
+    ("http-server", "httpd", "The admin HTTP server", GZOCHID_TYPE_HTTP_SERVER,
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
   obj_properties[PROP_SOCKET_SERVER] = g_param_spec_object
     ("socket-server", "socket-server", "The global socket server",
      GZOCHID_TYPE_SOCKET_SERVER, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
@@ -138,7 +170,6 @@ gzochid_root_context_class_init (GzochidRootContextClass *klass)
 static void
 gzochid_root_context_init (GzochidRootContext *self)
 {
-  self->admin_context = (gzochid_context *) gzochid_admin_context_new ();
 }
 
 static const struct option longopts[] =
@@ -204,6 +235,26 @@ There is NO WARRANTY, to the extent permitted by law.\n"),
 	  "2017");
 }
 
+static gboolean
+sigterm_handler (gpointer user_data)
+{
+  GzochidRootContext *root_context = user_data;
+
+  g_main_loop_quit (root_context->socket_server->main_loop);  
+  return TRUE;
+}
+
+static void
+initialize_signal_handlers (GzochidRootContext *root_context)
+{
+#ifndef WIN32
+  GSource *sigterm_source = g_unix_signal_source_new (SIGTERM);
+
+  g_source_set_callback (sigterm_source, sigterm_handler, root_context, NULL);
+  g_source_attach (sigterm_source, root_context->socket_server->main_context);
+#endif /* WIN32 */
+}
+
 /* Set the logging threshold. */
 
 static void
@@ -238,33 +289,90 @@ initialize_logging (GKeyFile *key_file)
   g_hash_table_destroy (log_config);
 }
 
-static void
-root_context_start (GzochidRootContext *root_context)
+/* Configure and start the admin web console if it's enabled. */
+
+static gpointer
+initialize_httpd (GzochidHttpServer *http_server,
+		  GzochidGameServer *game_server,
+		  GzochidResolutionContext *res_context, GKeyFile *key_file)
 {
-  GError *err = NULL;
-  GzochidMetaClient *metaclient = NULL;
+  gpointer httpd_state = NULL;
+  GHashTable *admin_config =
+    gzochid_config_keyfile_extract_config (key_file, "admin");
 
-  gzochid_event_loop_start (root_context->event_loop);
+  if (gzochid_config_to_boolean
+      (g_hash_table_lookup (admin_config, "module.httpd.enabled"), FALSE))
+    {
+      GError *err = NULL;
+      int port = gzochid_config_to_int
+	(g_hash_table_lookup (admin_config, "module.httpd.port"), 8800);
 
-  if (root_context->admin_context != NULL)
-    {  
-      gzochid_admin_context_init 
-	((gzochid_admin_context *) root_context->admin_context, root_context);
-      gzochid_context_until
-	(root_context->admin_context, GZOCHID_ADMIN_STATE_RUNNING);
+      httpd_state = gzochid_httpd_app_register_handlers
+	(http_server, game_server, res_context);
+      gzochid_http_server_start (http_server, port, &err);
+
+      if (err != NULL)
+	{
+	  g_critical
+	    ("Failed to start admin HTTP server: %s; exiting...", err->message);
+	  exit (EXIT_FAILURE);
+	}
+    }
+  
+  g_hash_table_destroy (admin_config);
+  return httpd_state;
+}
+
+static void
+initialize_debugging (GzochidDebugServer *debug_server, GKeyFile *key_file)
+{
+  GHashTable *admin_config =
+    gzochid_config_keyfile_extract_config (key_file, "admin");
+
+  if (gzochid_config_to_boolean 
+      (g_hash_table_lookup (admin_config, "module.debug.enabled"), FALSE))
+    {
+      GError *err = NULL;
+      int port = gzochid_config_to_int 
+	(g_hash_table_lookup (admin_config, "module.debug.port"), 
+	 37146);
+
+      gzochid_debug_server_start (debug_server, port, &err);
+
+      if (err != NULL)
+	{
+	  g_critical
+	    ("Failed to start admin debugging server: %s; exiting...",
+	     err->message);
+	  exit (EXIT_FAILURE);
+	}
     }
 
-  gzochid_game_server_start (root_context->game_server, &err);
+  g_hash_table_destroy (admin_config);
+}
+
+static void
+initialize_game_server (GzochidGameServer *game_server)
+{
+  GError *err = NULL;
+  gzochid_game_server_start (game_server, &err);
   
   if (err != NULL)
     {
       g_critical ("Failed to start game server: %s; exiting...", err->message);
       exit (EXIT_FAILURE);
     }
-  
-  g_object_get (root_context->metaclient_container,
-		"metaclient", &metaclient,
-		NULL);
+
+  scm_with_guile ((void *(*)(void *)) gzochid_api_admin_init, game_server);
+}
+
+static void
+initialize_metaclient (GzochidMetaClientContainer *metaclient_container)
+{
+  GError *err = NULL;
+  GzochidMetaClient *metaclient = NULL;
+
+  g_object_get (metaclient_container, "metaclient", &metaclient, NULL);
   
   if (metaclient != NULL)
     {
@@ -294,7 +402,8 @@ main (int argc, char *argv[])
   GzochidConfiguration *configuration = NULL;
   GzochidResolutionContext *resolution_context = NULL;
   GzochidRootContext *root_context = NULL;
-
+  gpointer httpd_state = NULL;
+  
   setlocale (LC_ALL, "");
 
   while ((optc = getopt_long (argc, argv, "c:hv", longopts, NULL)) != -1)
@@ -349,7 +458,6 @@ main (int argc, char *argv[])
     (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, "path", conf_path, NULL);
   gzochid_resolver_provide (resolution_context, G_OBJECT (configuration), NULL);
   
-  initialize_guile ();
   root_context = gzochid_resolver_require_full
     (resolution_context, GZOCHID_TYPE_ROOT_CONTEXT, &err);
 
@@ -359,10 +467,35 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  root_context->gzochid_conf_path = conf_path;  
-  root_context_start (root_context);
+  root_context->gzochid_conf_path = conf_path;
+  gzochid_event_loop_start (root_context->event_loop);
+
+  initialize_guile ();
+  httpd_state = initialize_httpd
+    (root_context->http_server, root_context->game_server, resolution_context,
+     key_file);
+  
+  initialize_debugging (root_context->debug_server, key_file);
+  initialize_game_server (root_context->game_server);
+  initialize_metaclient (root_context->metaclient_container);
+  initialize_signal_handlers (root_context);
+
+  g_object_unref (resolution_context);
+  g_object_unref (configuration);
+  g_key_file_unref (key_file);
   
   g_main_loop_run (root_context->socket_server->main_loop);
   
+  g_info ("Shutting down.");  
+
+  gzochid_debug_server_stop (root_context->debug_server);
+  gzochid_http_server_stop (root_context->http_server);
+
+  if (httpd_state != NULL)
+    gzochid_httpd_app_unregister_handlers (httpd_state);
+
+  gzochid_event_loop_stop (root_context->event_loop);
+  g_object_unref (root_context);
+
   return 0;
 }
