@@ -1,5 +1,5 @@
 /* httpd-app.c: Handler implementations for the gzochid admin web console
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -127,11 +127,22 @@ gzochid_server_state_new ()
   return state;
 }
 
+/* Free the resources associated with the specified `gzochid_server_state'
+   object. */
+
+static void
+gzochid_server_state_free (gzochid_server_state *state)
+{
+  g_mutex_clear (&state->mutex);
+  
+  free (state);
+}
+
 /* Captures serialization state of an object. */
 
 struct data_state 
 {
-  char *data; /* The raw serialized object bytes. */
+  unsigned char *data; /* The raw serialized object bytes. */
   int data_offset; /* The current offset within the object bytes. */
   int data_length; /* the object length. */
 
@@ -602,7 +613,7 @@ render_oid (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   gzochid_application_store *store = NULL;
   gzochid_storage_transaction *tx = NULL;
   guint64 oid = gzochid_util_encode_oid (g_ascii_strtoull (oid_str, NULL, 10)); 
-  char *data = NULL;
+  unsigned char *data = NULL;
 
   g_object_get (app_context, "app-store", &store, NULL);
 
@@ -781,7 +792,7 @@ attach_data_client_handler (GzochidResolutionContext *res_context,
   g_object_unref (configuration);
 }
 
-void
+gpointer
 gzochid_httpd_app_register_handlers (GzochidHttpServer *http_server,
 				     GzochidGameServer *game_server,
 				     GzochidResolutionContext *res_context)
@@ -815,4 +826,12 @@ gzochid_httpd_app_register_handlers (GzochidHttpServer *http_server,
 
       gzochid_httpd_append_terminal (apps_root, "/names/", list_names, NULL);
     }
+
+  return state;
+}
+
+void
+gzochid_httpd_app_unregister_handlers (gpointer state)
+{
+  gzochid_server_state_free (state);
 }
