@@ -1,5 +1,5 @@
 /* httpd-meta.h: Prototypes and declarations for httpd-meta.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 #ifndef GZOCHID_HTTPD_META_H
 #define GZOCHID_HTTPD_META_H
 
+#include <glib.h>
+
 #include "event.h"
 #include "httpd.h"
 #include "resolver.h"
@@ -35,8 +37,12 @@
   the meta server's infrastructure. 
 */
 
-void gzochid_httpd_meta_register_handlers (GzochidHttpServer *,
-					   gzochid_event_source *,
-					   GzochidResolutionContext *);
+gpointer gzochid_httpd_meta_register_handlers (GzochidHttpServer *,
+					       gzochid_event_source *,
+					       GzochidResolutionContext *);
+
+/* Clean up resources associated with the specified state pointer. */
+
+void gzochid_httpd_meta_unregister_handlers (gpointer);
 
 #endif /* GZOCHID_HTTPD_META_H */
