@@ -1,5 +1,5 @@
 /* httpd.c: Embedded httpd and minimal request mapping framework for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -451,7 +451,7 @@ gzochid_httpd_append_continuation (gzochid_httpd_partial *partial,
   return (gzochid_httpd_partial *) target;
 }
 
-static int 
+static mhd_result 
 not_found404 (struct MHD_Connection *connection, const char *page, 
 	      enum MHD_ResponseMemoryMode memory_mode)
 {
@@ -464,7 +464,7 @@ not_found404 (struct MHD_Connection *connection, const char *page,
   return ret;
 }
 
-static int 
+static mhd_result
 not_found404_default (struct MHD_Connection *connection)
 {
   return not_found404 
@@ -486,7 +486,7 @@ gzochid_http_write_response
 
 /* The `MHD_AccessHandlerCallback' for GNU microhttpd. */
 
-static int 
+static mhd_result
 dispatch (void *cls, struct MHD_Connection *connection, 
 	  const char *path, const char *method, const char *version, 
 	  const char *upload_data, size_t *upload_data_size, void **con_cls)
@@ -500,7 +500,7 @@ dispatch (void *cls, struct MHD_Connection *connection,
   path_match *match = NULL;
   
   if (strcmp (method, "GET") != 0)
-    return 0;
+    return MHD_NO;
 
   sink.response_written = FALSE;
   sink.connection = connection;
