@@ -1,5 +1,5 @@
 /* io.h: Prototypes and declarations for io.c
- * Copyright (C) 2014 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -30,15 +30,15 @@ typedef enum
   }
   GzochidIoError;
 
-struct _gzochid_application_context;
+struct _GzochidApplicationContext;
 
 typedef struct _gzochid_io_serialization
 {
   void (*serializer) 
-    (struct _gzochid_application_context *, void *, GByteArray *, GError **);
+    (struct _GzochidApplicationContext *, void *, GByteArray *, GError **);
   void *(*deserializer) 
-    (struct _gzochid_application_context *, GByteArray *, GError **);
-  void (*finalizer) (struct _gzochid_application_context *, void *);
+    (struct _GzochidApplicationContext *, GByteArray *, GError **);
+  void (*finalizer) (struct _GzochidApplicationContext *, void *);
 } gzochid_io_serialization;
 
 #endif /* GZOCHID_IO_H */
