@@ -1,5 +1,5 @@
 /* log.c: Log configuration routines for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <assert.h>
 #include <glib.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -25,6 +26,12 @@
 
 #include "log.h"
 
+#ifndef G_LOG_DOMAIN
+#define G_LOG_DOMAIN "gzochid"
+#endif /* G_LOG_DOMAIN */
+
+#define GZOCHID_LOG_LEVEL_MASK (G_LOG_LEVEL_MASK | GZOCHID_LOG_LEVEL_TRACE)
+
 struct _priority_mask_holder
 {
   GLogLevelFlags priority_mask;
@@ -32,25 +39,18 @@ struct _priority_mask_holder
 
 typedef struct _priority_mask_holder priority_mask_holder;
 
+static priority_mask_holder *default_holder;
+
 static void
 log_handler (const gchar *log_domain, GLogLevelFlags log_level,
 	     const gchar *message, gpointer user_data)
 {
+  const char *domain = log_domain == NULL ? G_LOG_DOMAIN : log_domain;
   const char *severity = "";
   struct timeval tv;
   struct tm ltm;
 
-  priority_mask_holder *holder = user_data;
-
-  /* Replicate behavior of default logger with respect to printing debug 
-     messages. */
-  
-  const gchar *domains = g_getenv ("G_MESSAGES_DEBUG");
-  
-  if (!(log_level & G_LOG_LEVEL_MASK & holder->priority_mask)
-      && (domains == NULL
-	  || ((!log_domain || !strstr (domains, log_domain))
-	      && strcmp (domains, "all") != 0)))
+  if (!gzochid_log_level_visible (log_domain, log_level))
     return;
 
   gettimeofday (&tv, NULL);
@@ -58,30 +58,37 @@ log_handler (const gchar *log_domain, GLogLevelFlags log_level,
 
   if (log_level & G_LOG_LEVEL_ERROR ||
       log_level & G_LOG_LEVEL_CRITICAL)
-    severity = "ERR";
+    severity = "ERR    ";
   else if (log_level & G_LOG_LEVEL_WARNING)
     severity = "WARNING";
   else if (log_level & G_LOG_LEVEL_MESSAGE)
-    severity = "NOTICE";
+    severity = "NOTICE ";
   else if (log_level & G_LOG_LEVEL_INFO)
-    severity = "INFO";
+    severity = "INFO   ";
   else if (log_level & G_LOG_LEVEL_DEBUG)
-    severity = "DEBUG";
-
-  fprintf (stderr, "%d-%.2d-%.2dT%.2d:%.2d,%dZ %s %s\n", 1900 + ltm.tm_year, 
-	   ltm.tm_mon + 1, ltm.tm_mday, ltm.tm_hour, ltm.tm_min, 
-	   (int) (tv.tv_usec / 1000), severity, message);
+    severity = "DEBUG  ";
+  else if (log_level & GZOCHID_LOG_LEVEL_TRACE)
+    severity = "TRACE  ";
+  
+  fprintf (stderr, "%d-%.2d-%.2dT%.2d:%.2d,%dZ %s [%s] %s\n",
+	   1900 + ltm.tm_year, ltm.tm_mon + 1, ltm.tm_mday, ltm.tm_hour,
+	   ltm.tm_min, (int) (tv.tv_usec / 1000), severity, domain, message);
 }
 
 void gzochid_install_log_handler (GLogLevelFlags log_level)
 {
-  priority_mask_holder *holder = NULL;
-  GLogLevelFlags priority_mask = G_LOG_LEVEL_MASK;
+  GLogLevelFlags priority_mask = GZOCHID_LOG_LEVEL_MASK;
   
+  assert (default_holder == NULL);
+  
+  if (log_level & G_LOG_LEVEL_DEBUG)
+    priority_mask = priority_mask & ~GZOCHID_LOG_LEVEL_TRACE;
   if (log_level & G_LOG_LEVEL_INFO)
-    priority_mask = priority_mask & ~G_LOG_LEVEL_DEBUG;
+    priority_mask = priority_mask &
+      ~(G_LOG_LEVEL_DEBUG | GZOCHID_LOG_LEVEL_TRACE);
   else if (log_level & G_LOG_LEVEL_MESSAGE)
-    priority_mask = priority_mask & ~(G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG);
+    priority_mask = priority_mask &
+      ~(G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | GZOCHID_LOG_LEVEL_TRACE);
   else if (log_level & G_LOG_LEVEL_WARNING)
     priority_mask =
       G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING;
@@ -90,10 +97,26 @@ void gzochid_install_log_handler (GLogLevelFlags log_level)
   else if (log_level & G_LOG_LEVEL_ERROR)
     priority_mask = G_LOG_LEVEL_ERROR;
 
-  holder = malloc (sizeof (priority_mask_holder));
-  holder->priority_mask = priority_mask;
+  default_holder = malloc (sizeof (priority_mask_holder));
+  default_holder->priority_mask = priority_mask;
   
-  g_log_set_handler
-    (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-     log_handler, holder);
+  g_log_set_default_handler (log_handler, NULL);
+}
+
+gboolean
+gzochid_log_level_visible (const gchar *log_domain, GLogLevelFlags log_level)
+{
+  const gchar *domains = NULL;
+
+  if (default_holder == NULL)
+    return FALSE;
+
+  /* Replicate behavior of default logger with respect to printing debug 
+     messages. */
+  
+  domains = g_getenv ("G_MESSAGES_DEBUG");
+
+  return log_level & GZOCHID_LOG_LEVEL_MASK & default_holder->priority_mask
+    || (domains != NULL && ((log_domain != NULL && strstr (domains, log_domain))
+			    || strcmp (domains, "all") == 0));
 }
