@@ -1,5 +1,5 @@
 /* metaclient-protocol.h: Prototypes and declarations for metaclient-protocol.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,6 +26,6 @@
 
 /* A `gzochid_client_protocol' implementation for the metaclient protocol. */
 
-gzochid_client_protocol gzochid_metaclient_client_protocol;
+extern gzochid_client_protocol gzochid_metaclient_client_protocol;
 
 #endif /* GZOCHID_METACLIENT_PROTOCOL_H */
