@@ -1,5 +1,5 @@
 /* metaclient.c: Metaserver client for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -38,8 +38,10 @@
 #include "metaclient.h"
 #include "metaclient-protocol.h"
 #include "resolver.h"
+#include "schedule.h"
 #include "sessionclient.h"
 #include "socket.h"
+#include "taskclient.h"
 
 struct _GzochidMetaClient
 {
@@ -72,7 +74,7 @@ struct _GzochidMetaClient
     The dataclient instance to which the metaclient delegates handling of 
     `GZOCHID_DATA_PROTOCOL_*' opcodes. 
 
-    The sessionclient has a lifecycle similar to that of the channelclient.
+    The dataclient has a lifecycle similar to that of the channelclient.
   */
   
   GzochidDataClient *dataclient;
@@ -85,7 +87,16 @@ struct _GzochidMetaClient
   */
 
   GzochidSessionClient *sessionclient;
-  
+
+  /*
+    The taskclient instance to which the metaclient delegates handling of 
+    `GZOCHID_TASK_PROTOCOL_*' opcodes. 
+
+    The taskclient has a lifecycle similar to that of the channelclient.
+  */
+
+  GzochidTaskClient *taskclient;
+
   /* The socket server for the meta client. */
 
   GzochidSocketServer *socket_server;
@@ -145,6 +156,7 @@ enum gzochid_meta_client_properties
     PROP_CHANNEL_CLIENT,
     PROP_DATA_CLIENT,
     PROP_SESSION_CLIENT,
+    PROP_TASK_CLIENT,
     N_PROPERTIES
   };
 
@@ -177,7 +189,11 @@ gzochid_meta_client_get_property (GObject *object, guint property_id,
     case PROP_SESSION_CLIENT:
       g_value_set_object (value, client->sessionclient);
       break;
-      
+
+    case PROP_TASK_CLIENT:
+      g_value_set_object (value, client->taskclient);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -229,6 +245,7 @@ gzochid_meta_client_dispose (GObject *object)
   g_object_unref (client->dataclient);
   g_object_unref (client->resolution_context);
   g_object_unref (client->sessionclient);
+  g_object_unref (client->taskclient);
   g_object_unref (client->socket_server);
 
   g_main_context_unref (client->main_context);
@@ -265,6 +282,8 @@ gzochid_meta_client_constructed (GObject *gobject)
   GzochidMetaClient *client = GZOCHID_META_CLIENT (gobject);  
   GzochidGameServer *game_server = gzochid_resolver_require_full
     (client->resolution_context, GZOCHID_TYPE_GAME_SERVER, NULL);
+  GzochidTaskQueue *task_queue = gzochid_resolver_require_full
+    (client->resolution_context, GZOCHID_TYPE_TASK_QUEUE, NULL);
 
   /* Extract and save the "metaserver" configuration group to use to look up
      connection details. */
@@ -300,7 +319,18 @@ gzochid_meta_client_constructed (GObject *gobject)
      "reconnectable-socket", client->socket,
      NULL);
 
+  /* Explicit construction of the taskclient with game server and 
+     reconnectable socket pointer. */
+
+  client->taskclient = g_object_new
+    (GZOCHID_TYPE_TASK_CLIENT,
+     "game-server", game_server,
+     "reconnectable-socket", client->socket,
+     "task-queue", task_queue,
+     NULL);
+
   g_object_unref (game_server);
+  g_object_unref (task_queue);
 }
 
 static void
@@ -350,7 +380,11 @@ gzochid_meta_client_class_init (GzochidMetaClientClass *klass)
   obj_properties[PROP_SESSION_CLIENT] = g_param_spec_object
     ("session-client", "sessionclient", "The session client",
      GZOCHID_TYPE_SESSION_CLIENT, G_PARAM_READABLE);
-  
+
+  obj_properties[PROP_TASK_CLIENT] = g_param_spec_object
+    ("task-client", "taskclient", "The task client", GZOCHID_TYPE_TASK_CLIENT,
+     G_PARAM_READABLE);
+
   g_object_class_install_properties
     (object_class, N_PROPERTIES, obj_properties);
 }
