@@ -1,5 +1,5 @@
 /* metaserver-protocol.h: Prototypes and declarations for metaserver-protocol.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,10 +24,10 @@ typedef struct _gzochi_metad_metaserver_client gzochi_metad_metaserver_client;
 
 /* A `gzochid_server_protocol' implementation for the metaserver protocol. */
 
-gzochid_server_protocol gzochi_metad_metaserver_server_protocol;
+extern gzochid_server_protocol gzochi_metad_metaserver_server_protocol;
 
 /* A `gzochid_client_protocol' implementation for the metaserver protocol. */
 
-gzochid_client_protocol gzochi_metad_metaserver_client_protocol;
+extern gzochid_client_protocol gzochi_metad_metaserver_client_protocol;
 
 #endif /* GZOCHI_METAD_METASERVER_PROTOCOL_H */
