@@ -1,5 +1,5 @@
 /* oids-storage.c: A storage-based oid block allocation strategy for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ allocate (gpointer user_data, gzochid_data_oids_block *ret, GError **err)
   gzochid_oid_storage_context *context = user_data;
   
   char next_oid_key[] = { NEXT_OID_KEY };
-  char *next_oid_value = NULL;
+  unsigned char *next_oid_value = NULL;
   size_t next_oid_value_len = 0;
   guint64 encoded_oid = 0, block_start = 0, next_block = 0;
   
@@ -91,8 +91,8 @@ allocate (gpointer user_data, gzochid_data_oids_block *ret, GError **err)
   encoded_oid = gzochid_util_encode_oid (next_block);
   
   context->engine->transaction_put
-    (transaction, context->store, next_oid_key, 1, (char *) &encoded_oid,
-     sizeof (guint64));
+    (transaction, context->store, next_oid_key, 1,
+     (unsigned char *) &encoded_oid, sizeof (guint64));
   
   if (!transaction->rollback)
     context->engine->transaction_prepare (transaction);
