@@ -1,5 +1,5 @@
 /* resolver.c: Minimal dependency injection framework for gzochid components
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -31,42 +31,38 @@ gzochid_resolution_error_quark (void)
 
 struct _GzochidResolutionContext
 {
-  GInitiallyUnowned parent_instance;
+  GObject parent_instance;
 
-  GHashTable *instances; /* The instance cache. */
+  /* The instance cache, as a map of `GType *' to `GWeakRef *'.  */
+
+  GHashTable *instances; 
 
   /*
-    Forsaken instances are those instances that hold a reference to the
-    resolution context that created them. They shouldn't be unref'd during the
-    disposal of the context, because the results are unpredictable (usually a
-    double `free'). 
+    List of strong references to instances provided to the resolution context
+    via `gzochid_resolver_provide'.
   */
-  
-  GList *forsaken_instances;
+
+  GList *provided_instances;
+
+  /* 
+     Temporary storage of instances created or referenced during a resolution
+     cycle, for unref at the end.
+  */
+
+  GList *referenced_instances;
 };
 
 G_DEFINE_TYPE (GzochidResolutionContext, gzochid_resolution_context,
-	       G_TYPE_INITIALLY_UNOWNED);
+	       G_TYPE_OBJECT);
 
 static void
-unref_value (gpointer key, gpointer value, gpointer userdata)
-{
-  GList *forsaken_instances = userdata;
-
-  if (!g_list_find (forsaken_instances, value))
-    g_object_unref (value);
-}
-
-void
 gzochid_resolution_context_dispose (GObject *object)
 {
   GzochidResolutionContext *self = GZOCHID_RESOLUTION_CONTEXT (object);
 
-  GType context_type = GZOCHID_TYPE_RESOLUTION_CONTEXT;
-
-  g_hash_table_remove (self->instances, &context_type);
-  g_hash_table_foreach (self->instances, unref_value, self->forsaken_instances);
-
+  g_list_foreach (self->provided_instances, (GFunc) g_object_unref, NULL);
+  g_list_foreach (self->referenced_instances, (GFunc) g_object_unref, NULL);
+  
   G_OBJECT_CLASS (gzochid_resolution_context_parent_class)->dispose (object);
 }
 
@@ -76,7 +72,8 @@ gzochid_resolution_context_finalize (GObject *object)
   GzochidResolutionContext *self = GZOCHID_RESOLUTION_CONTEXT (object);
   
   g_hash_table_destroy (self->instances);
-  g_list_free (self->forsaken_instances);
+  g_list_free (self->provided_instances);
+  g_list_free (self->referenced_instances);
 
   G_OBJECT_CLASS (gzochid_resolution_context_parent_class)->finalize (object);
 }
@@ -90,19 +87,20 @@ gzochid_resolution_context_class_init (GzochidResolutionContextClass *klass)
   object_class->finalize = gzochid_resolution_context_finalize;
 }
 
+/* `GDestroyNotify' callback for the `GWeakRef' values in the instance table. */
+
+static void
+free_weak_ref (gpointer data)
+{
+  g_weak_ref_clear (data);
+  free (data);
+}
+
 static void
 gzochid_resolution_context_init (GzochidResolutionContext *self)
 {
-  GType *resolution_context_type = malloc (sizeof (GType));
-
-  *resolution_context_type = GZOCHID_TYPE_RESOLUTION_CONTEXT;
-
-  self->instances = g_hash_table_new_full (g_int_hash, g_int_equal, free, NULL);
-  self->forsaken_instances = NULL;
-  
-  /* Add the resolution context to its own instance cache. */
-  
-  g_hash_table_insert (self->instances, resolution_context_type, self);
+  self->instances = g_hash_table_new_full
+    (g_int_hash, g_int_equal, free, free_weak_ref);
 }
 
 /* Represents a maximally-injectable constructor for a GObject-derived type: The
@@ -285,6 +283,8 @@ build_dependency_tree_inner (GzochidResolutionContext *context, GList *stack,
   GError *local_err = NULL;
   tree_state *state = NULL;
   GType *type = NULL;
+  GWeakRef *weak_ref = NULL;
+  GObject *inst = NULL;
   
   assert (stack != NULL);
 
@@ -297,9 +297,14 @@ build_dependency_tree_inner (GzochidResolutionContext *context, GList *stack,
   
   type = state->types->data;
   state->types = g_list_delete_link (state->types, state->types);  
-
-  if (g_hash_table_contains (context->instances, type))
+  weak_ref = g_hash_table_lookup (context->instances, type);
+  inst = weak_ref != NULL ? g_weak_ref_get (weak_ref) : NULL;
+  
+  if (*type == GZOCHID_TYPE_RESOLUTION_CONTEXT || inst != NULL)
     {
+      if (inst != NULL)
+	g_object_unref (inst);
+	
       free (type);
       return NULL;
     }
@@ -418,19 +423,16 @@ construct_instance (GNode *node, gpointer data)
   constructor *ctor = node->data;
   GzochidResolutionContext *context = data;
   GType type = G_TYPE_FROM_CLASS (ctor->klass);
+  GWeakRef *weak_ref = g_hash_table_lookup (context->instances, &type);
+  GObject *inst = weak_ref != NULL ? g_weak_ref_get (weak_ref) : NULL;
   
-  if (!g_hash_table_contains (context->instances, &type))
+  if (inst == NULL)
     {
-      GObject *inst = NULL;
-      GType *type_key = malloc (sizeof (GType));
-      
-      *type_key = type;
-
       if (ctor->n_parameters > 0)
 	{
 	  guint i = 0;
-	  GParameter *params = malloc
-	    (sizeof (GParameter) * ctor->n_parameters);
+	  const char *names[ctor->n_parameters];
+	  GValue values[ctor->n_parameters];
 
 	  for (; i < ctor->n_parameters; i++)
 	    {
@@ -438,43 +440,55 @@ construct_instance (GNode *node, gpointer data)
 	      GType ptype = pspec.value_type;
 	      GValue value = G_VALUE_INIT;
 	      GObject *obj = NULL;
-	      
-	      params[i].name = pspec.name;
-	      params[i].value = value;
 
-	      g_value_init (&params[i].value, ptype);
+	      names[i] = pspec.name;
+	      values[i] = value;
+
+	      g_value_init (&values[i], ptype);
+
+	      if (ptype == GZOCHID_TYPE_RESOLUTION_CONTEXT)
+		obj = G_OBJECT (context);
+	      else
+		{		  
+		  assert (g_hash_table_contains (context->instances, &ptype));
+		  obj = g_weak_ref_get
+		    (g_hash_table_lookup (context->instances, &ptype));
+		  assert (obj != NULL);
+
+		  /* 
+		     Calling `g_weak_ref_get' increments the reference count,
+		     so we need to make sure we unref it later.
+		  */
+		  
+		  context->referenced_instances = g_list_append
+		    (context->referenced_instances, obj);
+		}
 	      
-	      assert (g_hash_table_contains (context->instances, &ptype));
-	      obj = g_hash_table_lookup (context->instances, &ptype);
-	      
-	      g_value_take_object (&params[i].value, obj);
+	      g_value_take_object (&values[i], obj);
 	    }
 	 
-	  inst = g_object_newv (type, ctor->n_parameters, params);
-
-	  if (!g_object_is_floating (context))
-	    {
-	      /* 
-		 We passed the resolution context to this type's constructor and
-		 it sunk its floating reference. That's fine, but it'll create a
-		 reference cycle unless we mark this instance as "unreachable"
-		 from the context during disposal. 
-	      */
-	      
-	      g_object_ref (context);
-	      g_object_force_floating (G_OBJECT (context));
-	      g_object_force_floating (inst);
-
-	      context->forsaken_instances = g_list_prepend
-		(context->forsaken_instances, inst);
-	    }
-
-	  free (params);
+	  inst = g_object_new_with_properties
+	    (type, ctor->n_parameters, names, values);	  
 	}
       else inst = g_object_new (type, NULL);
 
-      g_hash_table_insert (context->instances, type_key, inst);
+      if (weak_ref == NULL)
+	{
+	  GType *type_key = malloc (sizeof (GType));
+
+	  *type_key = type;
+
+	  weak_ref = malloc (sizeof (GWeakRef));
+	  g_weak_ref_init (weak_ref, inst);
+	  g_hash_table_insert (context->instances, type_key, weak_ref);
+	}
+      else g_weak_ref_set (weak_ref, inst);
     }
+
+  if (inst != NULL)
+    context->referenced_instances =
+      g_list_append (context->referenced_instances, inst);
+
   free_constructor (ctor);
 
   return FALSE;
@@ -485,10 +499,14 @@ gzochid_resolver_require_full (GzochidResolutionContext *context,
 			       const GType type, GError **err)
 {
   GError *local_err = NULL;
+  GWeakRef *weak_ref = g_hash_table_lookup (context->instances, &type);
+  GObject *inst = weak_ref != NULL ? g_weak_ref_get (weak_ref) : NULL;
   
   assert (context != NULL);
 
-  if (!g_hash_table_contains (context->instances, &type))
+  if (type == GZOCHID_TYPE_RESOLUTION_CONTEXT)
+    return context;
+  else if (inst == NULL)
     {
       GNode *dependency_tree = build_dependency_tree
 	(context, type, &local_err);
@@ -503,9 +521,18 @@ gzochid_resolver_require_full (GzochidResolutionContext *context,
 	(dependency_tree, G_POST_ORDER, G_TRAVERSE_ALL, -1, construct_instance,
 	 context);
       g_node_destroy (dependency_tree);
+
+      weak_ref = g_hash_table_lookup (context->instances, &type);
+      inst = g_weak_ref_get (weak_ref);
     }
 
-  return g_object_ref_sink (g_hash_table_lookup (context->instances, &type));
+  /* Unref all the instances referenced during this type resolution cycle. */
+  
+  g_list_free_full (context->referenced_instances, g_object_unref);
+  
+  context->referenced_instances = NULL;
+
+  return inst;
 }
 
 gpointer
@@ -515,9 +542,7 @@ gzochid_resolver_require (GType type, GError **err)
     (GZOCHID_TYPE_RESOLUTION_CONTEXT, NULL);
   GObject *obj = gzochid_resolver_require_full (context, type, err);
 
-  if (g_object_is_floating (context))
-    g_object_unref (context);
-
+  g_object_unref (context);
   return obj;
 }
 
@@ -526,14 +551,20 @@ gzochid_resolver_provide (GzochidResolutionContext *context, GObject *obj,
 			  GError **err)
 {
   GType type = G_OBJECT_TYPE (obj);
-
-  if (!g_hash_table_contains (context->instances, &type))
+  GWeakRef *weak_ref = g_hash_table_lookup (context->instances, &type);
+  
+  if (type != GZOCHID_TYPE_RESOLUTION_CONTEXT && weak_ref == NULL)
     {
       GType *type_key = malloc (sizeof (GType));
+
       *type_key = type;
 
-      g_hash_table_insert
-	(context->instances, type_key, g_object_ref_sink (obj));
+      weak_ref = malloc (sizeof (GWeakRef));
+      g_weak_ref_init (weak_ref, g_object_ref_sink (obj));
+      context->provided_instances = g_list_append
+	(context->provided_instances, obj);
+
+      g_hash_table_insert (context->instances, type_key, weak_ref);
     }
   else g_set_error
 	 (err, GZOCHID_RESOLUTION_ERROR,
@@ -551,15 +582,20 @@ static void
 copy_ref (gpointer key, gpointer value, gpointer user_data)
 {
   GType *src_type = key;
-
-  if (*src_type != GZOCHID_TYPE_RESOLUTION_CONTEXT)
+  GObject *value_obj = g_weak_ref_get (value);
+  
+  if (*src_type != GZOCHID_TYPE_RESOLUTION_CONTEXT && value_obj != NULL)
     {
       GType *dst_type = malloc (sizeof (GType));
-      GzochidResolutionContext *dst = user_data;
+      GWeakRef *weak_ref = malloc (sizeof (GWeakRef));
+
+      GzochidResolutionContext *dst = user_data;      
 
       *dst_type = *src_type;
+      g_weak_ref_init (weak_ref, value_obj);
+      g_object_unref (value_obj);
       
-      g_hash_table_insert (dst->instances, dst_type, g_object_ref (value));
+      g_hash_table_insert (dst->instances, dst_type, weak_ref);
     }
 }
 
