@@ -244,7 +244,7 @@ gzochid_task_queue_constructed (GObject *obj)
   int max_threads = gzochid_config_to_int 
     (g_hash_table_lookup (config, "thread_pool.max_threads"), 4);
 
-  self->pool = gzochid_thread_pool_new (self, max_threads, TRUE, NULL);
+  self->pool = gzochid_thread_pool_new (self, max_threads, FALSE, NULL);
 
   g_hash_table_destroy (config);
 }
@@ -299,8 +299,9 @@ gzochid_task_queue_finalize (GObject *gobject)
 	(gzochid_seq_hash_table_lookup (queue->queue, task_key));
       gzochid_seq_hash_table_remove (queue->queue, task_key);
     }
-  
+
   gzochid_seq_hash_table_free (queue->queue);
+  gzochid_thread_pool_free (queue->pool, TRUE, TRUE);
   
   G_OBJECT_CLASS (gzochid_task_queue_parent_class)->finalize (gobject);  
 }
