@@ -1,5 +1,5 @@
 /* scheme-task.h: Prototypes and declarations for scheme-task.c
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -29,17 +29,17 @@
 #include "io.h"
 
 void gzochid_scheme_application_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 void gzochid_scheme_application_task_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 void gzochid_scheme_application_initialized_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 void gzochid_scheme_application_logged_in_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 void gzochid_scheme_application_received_message_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 void gzochid_scheme_application_disconnected_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 
 /* Transactional application worker to be used as a cleanup mechanism if 
    `gzochid_scheme_application_disconnected_worker' cannot commit its 
@@ -47,18 +47,18 @@ void gzochid_scheme_application_disconnected_worker
    the session key. */
 
 void gzochid_scheme_application_disconnected_cleanup_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 
 /* Called to invoke an application's optional "ready" lifecycle handler. */
 
-void gzochid_scheme_application_ready (gzochid_application_context *, 
+void gzochid_scheme_application_ready (GzochidApplicationContext *, 
 				       gzochid_auth_identity *, GError **);
 
 extern gzochid_application_task_serialization
 gzochid_scheme_task_serialization;
 
 gzochid_application_task *gzochid_scheme_task_new
-(gzochid_application_context *, gzochid_auth_identity *, char *, GList *, SCM);
+(GzochidApplicationContext *, gzochid_auth_identity *, char *, GList *, SCM);
 
 void gzochid_scheme_task_initialize_bindings (void);
 
