@@ -1,5 +1,5 @@
 /* scheme.c: Supplementary Scheme routines for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -146,12 +146,11 @@ scheme_managed_record_serializer (GzochidApplicationContext *context, SCM arg,
     (g_list_append (g_list_append (NULL, "gzochi"), "private"), "data");
   SCM vec = SCM_EOL, port = SCM_EOL, proc = SCM_EOL;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
-  SCM port_and_proc = 
-    scm_struct_ref (scm_open_bytevector_output_port (SCM_BOOL_F), SCM_INUM0);
+  SCM port_and_proc = scm_open_bytevector_output_port (SCM_BOOL_F);
   int vec_len = 0;
 
-  port = SCM_CAR (port_and_proc);
-  proc = SCM_CADR (port_and_proc);
+  port = scm_c_value_ref (port_and_proc, 0);
+  proc = scm_c_value_ref (port_and_proc, 1);
 
   gzochid_scheme_invoke_callback
     (context,
@@ -527,13 +526,7 @@ gzochid_scheme_create_managed_reference
 (gzochid_data_managed_reference *reference)
 {
   SCM scm_oid = scm_from_unsigned_integer (reference->oid);
-  SCM data = SCM_BOOL_F;
-  SCM ret = SCM_BOOL_F;
-
-  if (reference->obj != NULL)
-    data = gzochid_scm_location_resolve (reference->context, reference->obj);
-  
-  ret = scm_call_2 (scm_make_managed_reference, scm_oid, data);
+  SCM ret = scm_call_1 (scm_make_managed_reference, scm_oid);
 
   scm_gc_protect_object (ret);
 
