@@ -1,5 +1,5 @@
 /* scheme.h: Prototypes and declarations for scheme.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -76,9 +76,9 @@ guint64 gzochid_scheme_channel_oid (SCM);
 guint64 gzochid_scheme_task_handle_oid (SCM);
 
 SCM gzochid_scheme_invoke 
-(gzochid_application_context *, gzochid_auth_identity *, SCM, SCM, SCM);
+(GzochidApplicationContext *, gzochid_auth_identity *, SCM, SCM, SCM);
 SCM gzochid_scheme_invoke_callback
-(gzochid_application_context *, gzochid_auth_identity *, char *, GList *, SCM,
+(GzochidApplicationContext *, gzochid_auth_identity *, char *, GList *, SCM,
  SCM);
 
 extern gzochid_io_serialization gzochid_scheme_data_serialization;
