@@ -1,5 +1,5 @@
 ;; gzochi/admin.scm: Administrative interface to gzochid
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -22,8 +22,14 @@
 
           gzochi:applications
 	  gzochi:current-application
-	  gzochi:with-application)
-  (import (only (guile) thunk?)
+	  gzochi:with-application
+
+	  gzochi:set-primitive-applications!
+	  gzochi:set-primitive-current-application!
+	  gzochi:set-primitive-with-application!
+	  )
+  (import (only (guile) make-undefined-variable thunk? variable-ref
+		        variable-set!)
 	  (rnrs base)
 	  (rnrs conditions)
 	  (rnrs exceptions)
@@ -36,15 +42,21 @@
     (opaque #t)
     (sealed #t))
 
-  (define primitive-applications #f)
-  (define primitive-current-application #f)
-  (define primitive-with-application #f)
+  (define primitive-applications (make-undefined-variable))
+  (define primitive-current-application (make-undefined-variable))
+  (define primitive-with-application (make-undefined-variable))
 
-  (define (gzochi:applications)
-    (primitive-applications))
+  (define (gzochi:set-primitive-applications! proc)
+    (variable-set! primitive-applications proc))
+  (define (gzochi:set-primitive-current-application! proc)
+    (variable-set! primitive-current-application proc))
+  (define (gzochi:set-primitive-with-application! proc)
+    (variable-set! primitive-with-application proc))
+  
+  (define (gzochi:applications) ((variable-ref primitive-applications)))
 
   (define (gzochi:current-application)
-    (primitive-current-application))
+    ((variable-ref primitive-current-application)))
 
   (define (gzochi:with-application context thunk)
     (or (gzochi:application-context? context)
@@ -53,5 +65,5 @@
     (or (thunk? thunk)
 	(assertion-violation 'gzochi:with-application "Expected thunk." thunk))
 
-    (primitive-with-application context thunk))
+    ((variable-ref primitive-with-application) context thunk))
 )
