@@ -1,5 +1,5 @@
 ;; gzochi/private/channel.scm: Private infrastructure for gzochi channel API 
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -28,9 +28,18 @@
 	  gzochi:join-channel
 	  gzochi:leave-channel
 	  gzochi:close-channel
-	  gzochi:send-channel-message)
+	  gzochi:send-channel-message
 
-  (import (gzochi session)
+	  gzochi:set-primitive-create-channel!
+	  gzochi:set-primitive-get-channel!
+	  gzochi:set-primitive-join-channel!
+	  gzochi:set-primitive-leave-channel!
+	  gzochi:set-primitive-send-channel-message!
+	  gzochi:set-primitive-close-channel!
+	  )
+
+  (import (only (guile) make-undefined-variable variable-ref variable-set!)
+	  (gzochi session)
 	  (gzochi conditions)
 	  (gzochi private data)
 	  (gzochi io)
@@ -47,16 +56,16 @@
 	(assertion-violation
 	 'gzochi:create-channel "Channel name must be a string." name))
 
-    (and (primitive-get-channel name)
+    (and ((variable-ref primitive-get-channel) name)
 	 (raise (gzochi:make-name-exists-condition name)))
-    (primitive-create-channel name))
+    ((variable-ref primitive-create-channel) name))
 
   (define (gzochi:get-channel name)
     (or (string? name)
 	(assertion-violation
 	 'gzochi:get-channel "Channel name must be a string." name))
 
-    (or (primitive-get-channel name) 
+    (or ((variable-ref primitive-get-channel) name) 
 	(raise (gzochi:make-name-not-bound-condition name))))
 
   (define (gzochi:join-channel channel session)
@@ -67,7 +76,7 @@
 	(assertion-violation
 	 'gzochi:join-channel "Expeted gzochi:client-session." session))
 
-    (primitive-join-channel channel session))
+    ((variable-ref primitive-join-channel) channel session))
 
   (define (gzochi:leave-channel channel session)
     (or (gzochi:channel? channel)
@@ -77,7 +86,7 @@
 	(assertion-violation
 	 'gzochi:leave-channel "Expeted gzochi:client-session." session))
 
-    (primitive-leave-channel channel session))
+    ((variable-ref primitive-leave-channel) channel session))
 
   (define (gzochi:send-channel-message channel msg)
     (or (gzochi:channel? channel)
@@ -87,19 +96,32 @@
 	(assertion-violation
 	 'gzochi:send-channel-message "Expected bytevector." msg))
     
-    (primitive-send-channel-message channel msg))
+    ((variable-ref primitive-send-channel-message) channel msg))
 
   (define (gzochi:close-channel channel)
     (or (gzochi:channel? channel)
 	(assertion-violation
 	 'gzochi:close-channel "Expected gzochi:channel." channel))
 
-    (primitive-close-channel channel))
+    ((variable-ref primitive-close-channel) channel))
   
-  (define primitive-create-channel #f)
-  (define primitive-get-channel #f)
-  (define primitive-join-channel #f)
-  (define primitive-leave-channel #f)
-  (define primitive-send-channel-message #f)
-  (define primitive-close-channel #f)
+  (define primitive-create-channel (make-undefined-variable))
+  (define primitive-get-channel (make-undefined-variable))
+  (define primitive-join-channel (make-undefined-variable))
+  (define primitive-leave-channel (make-undefined-variable))
+  (define primitive-send-channel-message (make-undefined-variable))
+  (define primitive-close-channel (make-undefined-variable))
+
+  (define (gzochi:set-primitive-create-channel! proc)
+    (variable-set! primitive-create-channel proc))
+  (define (gzochi:set-primitive-get-channel! proc)
+    (variable-set! primitive-get-channel proc))
+  (define (gzochi:set-primitive-join-channel! proc)
+    (variable-set! primitive-join-channel proc))
+  (define (gzochi:set-primitive-leave-channel! proc)
+    (variable-set! primitive-leave-channel proc))
+  (define (gzochi:set-primitive-send-channel-message! proc)
+    (variable-set! primitive-send-channel-message proc))
+  (define (gzochi:set-primitive-close-channel! proc)
+    (variable-set! primitive-close-channel proc))
 )
