@@ -1,5 +1,5 @@
 ;; gzochi/private/data.scm: Private infrastructure for gzochi data API
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -62,12 +62,21 @@
 	  gzochi:deserialize-managed-record
 
 	  gzochi:mark-for-write!
-	  gzochi:mark-for-read!)
+	  gzochi:mark-for-read!
 
+	  gzochi:set-primitive-mark-for-write!
+	  gzochi:set-primitive-create-reference!
+	  gzochi:set-primitive-dereference!
+	  gzochi:set-primitive-get-binding!
+	  gzochi:set-primitive-set-binding!
+	  gzochi:set-primitive-remove-binding!
+	  gzochi:set-primitive-remove-object!)
+  
   (import (gzochi io)
 	  (gzochi log)
-	  (only (guile) and=> fluid-ref fluid-set! gensym macroexpand 
-		        make-fluid simple-format) 
+	  (only (guile) and=> cond-expand fluid-ref fluid-set! gensym
+	 	        macroexpand make-fluid make-undefined-variable
+			simple-format variable-ref variable-set!) 
 	  (ice-9 optargs)
 	  (rnrs)
 	  (only (srfi :1) last split-at take)
@@ -86,7 +95,7 @@
 
   (define (gzochi:deserialize-managed-reference port)
     (and (gzochi:read-boolean port)
-	 (gzochi:make-managed-reference (gzochi:read-integer port) #f)))
+	 (gzochi:make-managed-reference (gzochi:read-integer port))))
 
   (define gzochi:managed-reference-serialization 
     (gzochi:make-serialization gzochi:serialize-managed-reference 
@@ -572,7 +581,7 @@
 		(loop (+ i 1) (cons (deserializer port) vals)))
 	      (reverse vals)))))
 	
-    (define (constructor-descriptor rtd serialization)
+    (define (constructor-descriptor rtd args)
       (let* 
 	  ((parent-rtd (record-type-parent rtd))
 	   (parent-registration 
@@ -585,19 +594,25 @@
 		       #f "Attempting to deserialize unknown type ~A." 
 		       (record-type-name parent-rtd))))))
 	   
-	   (parent-rctd 
-	    (and parent-registration
-		 (constructor-descriptor 
-		  parent-rtd 
-		  (gzochi:managed-record-type-registration-field-serializations
-		   parent-registration)))))
-		  
+	   (parent-rctd (and parent-registration
+			     (constructor-descriptor parent-rtd (cdr args)))))
+
 	(make-record-constructor-descriptor 
 	 rtd parent-rctd (lambda (n)
 			   (lambda ()
-			     (let ((p (n)))
-			       (apply p (deserialize rtd serialization))))))))
-	
+			     (cond-expand
+
+			      ;; Guile 3.x "unified" the R6RS records system
+			      ;; with Guile's native records. Among other
+			      ;; things, this incomaptibly changes the
+			      ;; constructor delegation protocol such that the
+			      ;; protocol argument may sometimes be a
+			      ;; constructor and not a superconstructor.
+
+			      (guile-3 (let ((p (if parent-rctd (n) n)))
+					 (apply p (car args))))
+
+			      (else (apply (n) (car args)))))))))
     
     (let* ((serial-uid (gzochi:read-symbol port))
 	   (registration (gzochi:serial-uid->type-registration serial-uid)))
@@ -606,16 +621,34 @@
 	   'gzochi:deserialize-managed-record
 	   (simple-format #f "Attempting to deserialize unknown type ~A."
 			  serial-uid)))
-      
-      (let* 
-	  ((rctd 
-	    (constructor-descriptor 
-	     (gzochi:managed-record-type-registration-rtd registration)
-	     (gzochi:managed-record-type-registration-field-serializations
-	      registration)))
-	   (ctor (record-constructor rctd)))
 
-	(ctor))))
+      (let loop ((r registration) (rtd-serializations '()))
+	(let*
+	    ((rtd (gzochi:managed-record-type-registration-rtd r))
+	     (serializations
+	      (gzochi:managed-record-type-registration-field-serializations r)))
+
+	  ;; Walk up the chain of inheritance until we reach the "root,"
+	  ;; building up a list as we go.
+	  
+	  (let ((rtd-serializations-new
+		 (cons (cons rtd serializations) rtd-serializations))
+		(parent-rtd (record-type-parent rtd)))
+	    (if (and parent-rtd (gzochi:rtd->type-registration parent-rtd))
+		(loop (gzochi:rtd->type-registration parent-rtd)
+		      rtd-serializations-new)
+
+		;; Once we hit the root, deserialize all of the fields in
+		;; parent-to-child order.
+		
+		(let* ((args (map (lambda (rs) (deserialize (car rs) (cdr rs)))
+				  rtd-serializations-new))
+		       (rctd (constructor-descriptor 
+			      (gzochi:managed-record-type-registration-rtd
+			       registration)
+			      (reverse args)))
+		       (ctor (record-constructor rctd)))
+		  (ctor))))))))
   
   (define-record-type (gzochi:managed-record-type-registration
 		       gzochi:make-managed-record-type-registration
@@ -694,7 +727,7 @@
 	 'gzochi:mark-for-write!
 	 "Only managed records may be marked." managed-record))
     
-    (primitive-mark-for-write! managed-record))
+    ((variable-ref primitive-mark-for-write!) managed-record))
     
   (define (gzochi:mark-for-read! managed-record) (if #f #f))
 
@@ -711,8 +744,8 @@
 	(assertion-violation
 	 'gzochi:create-reference
 	 "References can only be created to managed records." obj))
-    
-    (primitive-create-reference obj))
+
+    ((variable-ref primitive-create-reference) obj))
 
   (define (gzochi:dereference reference)
     (or (gzochi:managed-reference? reference)
@@ -720,13 +753,13 @@
 	 'gzochi:dereference "Only managed references can be dereferenced."
 	 reference))
     
-    (primitive-dereference reference))
+    ((variable-ref primitive-dereference) reference))
 
   (define (gzochi:get-binding name)
     (or (string? name)
 	(assertion-violation 'gzochi:get-binding "Expecting string." name))
     
-    (primitive-get-binding name))
+    ((variable-ref primitive-get-binding) name))
   
   (define (gzochi:set-binding! name obj)
     (or (string? name)
@@ -737,28 +770,45 @@
 	 'gzochi:set-binding!
 	 "Bindings may only be created for managed records." obj))
     
-    (primitive-set-binding! name obj))
+    ((variable-ref primitive-set-binding!) name obj))
 
   (define (gzochi:remove-binding! name)
     (or (string? name)
 	(assertion-violation 'gzochi:remove-binding! "Expecting string." name))
     
-    (primitive-remove-binding! name))
+    ((variable-ref primitive-remove-binding!) name))
 
   (define (gzochi:remove-object! obj)
     (or (gzochi:managed-record? obj)
 	(assertion-violation
 	 'gzochi:remove-object! "Not a managed record." obj))
     
-    (primitive-remove-object! obj))
+    ((variable-ref primitive-remove-object!) obj))
 
-  (define primitive-mark-for-write! #f)
+  (define primitive-mark-for-write! (make-undefined-variable))
 
-  (define primitive-create-reference #f)
-  (define primitive-dereference #f)
+  (define primitive-create-reference (make-undefined-variable))
+  (define primitive-dereference (make-undefined-variable))
 
-  (define primitive-get-binding #f)
-  (define primitive-set-binding! #f)
-  (define primitive-remove-binding! #f)
-  (define primitive-remove-object! #f)
+  (define primitive-get-binding (make-undefined-variable))
+  (define primitive-set-binding! (make-undefined-variable))
+  (define primitive-remove-binding! (make-undefined-variable))
+  (define primitive-remove-object! (make-undefined-variable))
+
+  (define (gzochi:set-primitive-mark-for-write! proc)
+    (variable-set! primitive-mark-for-write! proc))
+
+  (define (gzochi:set-primitive-create-reference! proc)
+    (variable-set! primitive-create-reference proc))
+  (define (gzochi:set-primitive-dereference! proc)
+    (variable-set! primitive-dereference proc))
+
+  (define (gzochi:set-primitive-get-binding! proc)
+    (variable-set! primitive-get-binding proc))
+  (define (gzochi:set-primitive-set-binding! proc)
+    (variable-set! primitive-set-binding! proc))
+  (define (gzochi:set-primitive-remove-binding! proc)
+    (variable-set! primitive-remove-binding! proc))
+  (define (gzochi:set-primitive-remove-object! proc)
+    (variable-set! primitive-remove-object! proc))
 )
