@@ -1,5 +1,5 @@
 ;; gzochi/private/data/migration.scm: Private infrastructure for migration tools
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -17,12 +17,19 @@
 #!r6rs
 
 (library (gzochi private data migration)
-  (export gzochi:visit-object)
-  (import (gzochi data) (gzochi private data) (rnrs))
-
-  (define (enqueue-oids! migration oids) 
-    (assertion-violation 'gzochi:visit-object "enqueue-oids! not initialized."))
+  (export gzochi:visit-object
+	  
+	  gzochi:set-primitive-enqueue-oids!)
+  
+  (import (only (guile) make-undefined-variable variable-ref variable-set!)
+	  (gzochi data)
+	  (gzochi private data)
+	  (rnrs))
 
+  (define primitive-enqueue-oids! (make-undefined-variable))
+  (define (gzochi:set-primitive-enqueue-oids! proc)
+    (variable-set! primitive-enqueue-oids! proc))
+  
   (define (gzochi:visit-object migration obj visitor)
     (define (reference-fields obj)
       (define (reference-fields-inner rtd oids)
@@ -40,6 +47,6 @@
 		      oids
 		      (reference-fields-inner parent-rtd oids)))))))
       (reference-fields-inner (gzochi:managed-record-rtd obj) '()))
-    (enqueue-oids! migration (reference-fields obj))
+    ((variable-ref primitive-enqueue-oids!) migration (reference-fields obj))
     (visitor obj))
 )
