@@ -1,5 +1,5 @@
 ;; gzochi/private/session.scm: Private infrastructure for gzochi client API 
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -29,18 +29,28 @@
 	  gzochi:client-session-listener-disconnected
 
 	  gzochi:send-message
-	  gzochi:disconnect)
+	  gzochi:disconnect
 
-  (import (gzochi io)
+	  gzochi:set-primitive-send-message!
+	  gzochi:set-primitive-disconnect!
+	  )
+
+  (import (only (guile) make-undefined-variable variable-ref variable-set!)
+	  (gzochi io)
 	  (gzochi private data)
 	  (rnrs base)
 	  (rnrs bytevectors)
 	  (rnrs conditions)
 	  (rnrs exceptions))
 
-  (define primitive-send-message #f)
-  (define primitive-disconnect #f)
+  (define primitive-send-message (make-undefined-variable))
+  (define primitive-disconnect (make-undefined-variable))
 
+  (define (gzochi:set-primitive-send-message! proc)
+    (variable-set! primitive-send-message proc))
+  (define (gzochi:set-primitive-disconnect! proc)
+    (variable-set! primitive-disconnect proc))
+  
   (gzochi:define-managed-record-type 
    (gzochi:client-session gzochi:make-client-session gzochi:client-session?)
 
@@ -63,12 +73,12 @@
     (or (bytevector? msg)
 	(assertion-violation 'gzochi:send-message "Expected bytevector." msg))
     
-    (primitive-send-message session msg))
+    ((variable-ref primitive-send-message) session msg))
 
   (define (gzochi:disconnect session)
     (or (gzochi:client-session? session)
 	(assertion-violation
 	 'gzochi:disconnect "Expected gzochi:client-session." session))
 
-    (primitive-disconnect session))
+    ((variable-ref primitive-disconnect) session))
 )
