@@ -1,5 +1,5 @@
 ;; gzochi/tx.scm: Public exports for gzochi external transaction API
-;; Copyright (C) 2016 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -29,9 +29,13 @@
 	  gzochi:transaction-participant?
 	  gzochi:transaction-participant-prepare
 	  gzochi:transaction-participant-commit
-	  gzochi:transaction-participant-rollback)
+	  gzochi:transaction-participant-rollback
 
-  (import (only (guile) define* thunk?)
+	  gzochi:set-primitive-join-transaction!
+	  gzochi:set-primitive-abort-transaction!)
+
+  (import (only (guile) define* make-undefined-variable thunk? variable-ref
+		        variable-set!)
 	  (rnrs base)
 	  (rnrs conditions)
 	  (rnrs exceptions)
@@ -64,9 +68,14 @@
   ;; `primitive-join-transaction' and `primitive-abort-transaction' are replaced
   ;; at boot time with concrete implementations in api/tx.c.
 	      
-  (define primitive-join-transaction #f)
-  (define primitive-abort-transaction #f)
+  (define primitive-join-transaction (make-undefined-variable))
+  (define primitive-abort-transaction (make-undefined-variable))
 
+  (define (gzochi:set-primitive-join-transaction! proc)
+    (variable-set! primitive-join-transaction proc))
+  (define (gzochi:set-primitive-abort-transaction! proc)
+    (variable-set! primitive-abort-transaction proc))
+  
   ;; Wrapper around `primitive-join-transaction' that asserts the participant 
   ;; is valid as per `gzochi:transaction-participant?'.
 
@@ -76,7 +85,7 @@
 	 'gzochi:join-transaction "Expected transaction participant."
 	 participant))
     
-    (primitive-join-transaction participant))
+    ((variable-ref primitive-join-transaction) participant))
 
   ;; Wrapper around `primitive-abort-transaction' that asserts the participant 
   ;; is valid as per `gzochi:transaction-participant?'.
@@ -87,5 +96,5 @@
 	 'gzochi:join-transaction "Expected transaction participant."
 	 participant))
 
-    (primitive-abort-transaction participant retryable?))
+    ((variable-ref primitive-abort-transaction) participant retryable?))
 )
