@@ -1,5 +1,5 @@
 /* session.c: Client session management routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <string.h>
 
 #include "app.h"
+#include "app-session.h"
 #include "auth_int.h"
 #include "data.h"
 #include "event.h"
@@ -91,7 +92,7 @@ gzochid_client_session_pending_message_operation;
 
 struct _gzochid_client_session_transaction_context
 {
-  gzochid_application_context *context;
+  GzochidApplicationContext *app_context;
   
   gboolean login_failed;
   gzochid_client_session_pending_operation *login_operation;
@@ -109,12 +110,12 @@ gzochid_session_error_quark ()
 }
 
 static gzochid_client_session_transaction_context *
-create_transaction_context (gzochid_application_context *context)
+create_transaction_context (GzochidApplicationContext *context)
 {
   gzochid_client_session_transaction_context *tx_context = 
     calloc (1, sizeof (gzochid_client_session_transaction_context));
 
-  tx_context->context = context;
+  tx_context->app_context = g_object_ref (context);
 
   return tx_context;
 }
@@ -181,6 +182,7 @@ session_prepare (gpointer data)
 static void 
 cleanup_transaction (gzochid_client_session_transaction_context *tx_context)
 {
+  g_object_unref (tx_context->app_context);  
   g_list_free_full (tx_context->operations, free);
 
   if (tx_context->login_operation != NULL)
@@ -193,7 +195,7 @@ cleanup_transaction (gzochid_client_session_transaction_context *tx_context)
    may live on a different gzochid application server node. */
 
 static void
-forward_disconnect (GzochidMetaClient *metaclient, char *app,
+forward_disconnect (GzochidMetaClient *metaclient, const char *app,
  		    guint64 target_session_oid)
 {
   GzochidSessionClient *session_client = NULL;      
@@ -212,7 +214,7 @@ forward_disconnect (GzochidMetaClient *metaclient, char *app,
    may live on a different gzochid application server node. */
 
 static void
-forward_message (GzochidMetaClient *metaclient, char *app,
+forward_message (GzochidMetaClient *metaclient, const char *app,
  		 guint64 target_session_oid, unsigned char *msg, short len)
 {
   GBytes *msg_bytes = g_bytes_new (msg, len);
@@ -235,23 +237,30 @@ session_commit_operation
 (gzochid_client_session_transaction_context *tx_context, 
  gzochid_client_session_pending_operation *op)
 {
-  gzochid_application_context *context = tx_context->context;
+  GzochidApplicationContext *context = tx_context->app_context;
+  GzochidMetaClient *metaclient =
+    gzochid_application_context_get_metaclient (context);
+  gzochid_application_session_map *session_map = NULL;
+
   gzochid_client_session_pending_message_operation *msg_op = NULL;
   gzochid_game_client *client = NULL;
-  
-  g_mutex_lock (&context->client_mapping_lock);
-  client = g_hash_table_lookup (context->oids_to_clients, &op->target_session);
+
+  g_object_get (context, "session-map", &session_map, NULL);
+  g_mutex_lock (&session_map->client_mapping_lock);
+
+  client = g_hash_table_lookup
+    (session_map->oids_to_clients, &op->target_session);
 
   /* If the client isn't connected locally, and this server's not part of a
      cluster, then there's nothing to be done. */
-  
-  if (client == NULL && context->metaclient == NULL)
+
+  if (client == NULL && metaclient == NULL)
     {
       g_warning
 	("Client not found for session '%" G_GUINT64_FORMAT
 	 "'; skipping operation.", op->target_session);
 
-      g_mutex_unlock (&context->client_mapping_lock);
+      g_mutex_unlock (&session_map->client_mapping_lock);
       return;
     }
 
@@ -261,9 +270,10 @@ session_commit_operation
 
       if (client != NULL)
 	gzochid_game_client_disconnect (client); 
-      else if (context->metaclient != NULL)
+      else if (metaclient != NULL)
 	forward_disconnect
-	  (context->metaclient, context->descriptor->name, op->target_session);
+	  (metaclient, gzochid_application_context_get_name (context),
+	   op->target_session);
 
       break;
       
@@ -276,16 +286,22 @@ session_commit_operation
       if (!tx_context->login_failed 
 	  || op->target_session != tx_context->login_operation->target_session) 
 	{
+	  gzochid_event_source *event_source = NULL;
+	  g_object_get (context, "event-source", &event_source, NULL);
+	  
 	  msg_op = (gzochid_client_session_pending_message_operation *) op;
 
 	  gzochid_event_dispatch
-	    (context->event_source,
+	    (event_source,
 	     g_object_new (GZOCHID_TYPE_EVENT, "type", MESSAGE_SENT, NULL));
 
+	  g_source_unref ((GSource *) event_source);
+	  
 	  if (client != NULL)
 	    gzochid_game_client_send (client, msg_op->message, msg_op->len);
-	  else if (context->metaclient != NULL)
-	    forward_message (context->metaclient, context->descriptor->name,
+	  else if (metaclient != NULL)
+	    forward_message (metaclient,
+			     gzochid_application_context_get_name (context),
 			     op->target_session, msg_op->message, msg_op->len);
 	}
 
@@ -295,7 +311,10 @@ session_commit_operation
       break;
     }
 
-  g_mutex_unlock (&context->client_mapping_lock);
+  if (metaclient != NULL)
+    g_object_unref (metaclient);
+  
+  g_mutex_unlock (&session_map->client_mapping_lock);
 }
 
 static void 
@@ -330,7 +349,7 @@ session_rollback (gpointer data)
 }
 
 static gzochid_client_session_handler *
-deserialize_handler (gzochid_application_context *context, GByteArray *in, 
+deserialize_handler (GzochidApplicationContext *context, GByteArray *in, 
 		     GError **err)
 {
   gzochid_client_session_handler *handler = malloc 
@@ -345,8 +364,8 @@ deserialize_handler (gzochid_application_context *context, GByteArray *in,
 }
 
 static gpointer 
-deserialize_client_session (gzochid_application_context *context,
-			    GByteArray *in, GError **err)
+deserialize_client_session (GzochidApplicationContext *context, GByteArray *in,
+			    GError **err)
 {
   gzochid_auth_identity *identity = 
     gzochid_auth_identity_deserializer (context, in, NULL);
@@ -366,7 +385,7 @@ deserialize_client_session (gzochid_application_context *context,
 }
 
 static void 
-serialize_handler (gzochid_application_context *context, 
+serialize_handler (GzochidApplicationContext *context, 
 		   gzochid_client_session_handler *handler, GByteArray *out, 
 		   GError **err)
 {
@@ -377,7 +396,7 @@ serialize_handler (gzochid_application_context *context,
 }
 
 static void 
-serialize_client_session (gzochid_application_context *context, gpointer obj, 
+serialize_client_session (GzochidApplicationContext *context, gpointer obj, 
 			  GByteArray *out, GError **err)
 {
   gzochid_client_session *session = obj;
@@ -395,7 +414,7 @@ serialize_client_session (gzochid_application_context *context, gpointer obj,
 }
 
 static void 
-finalize_handler (gzochid_application_context *context, 
+finalize_handler (GzochidApplicationContext *context, 
 		  gzochid_client_session_handler *handler)
 {
   gzochid_application_callback_serialization.finalizer 
@@ -406,7 +425,7 @@ finalize_handler (gzochid_application_context *context,
 }
 
 static void 
-finalize_client_session (gzochid_application_context *context, gpointer obj)
+finalize_client_session (GzochidApplicationContext *context, gpointer obj)
 {
   gzochid_client_session *session = obj;
 
@@ -429,7 +448,7 @@ gzochid_io_serialization gzochid_client_session_serialization =
   };
 
 static void 
-remove_session (gzochid_application_context *context, guint64 session_oid, 
+remove_session (GzochidApplicationContext *context, guint64 session_oid, 
 		GError **err)
 {
   GError *local_err = NULL;
@@ -507,7 +526,7 @@ remove_session (gzochid_application_context *context, guint64 session_oid,
 
 void 
 gzochid_client_session_disconnected_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity,
+(GzochidApplicationContext *context, gzochid_auth_identity *identity,
  gpointer data)
 {
   GError *err = NULL;
@@ -542,7 +561,7 @@ gzochid_client_session_free (gzochid_client_session *session)
 }
 
 static gzochid_client_session_transaction_context *
-join_transaction (gzochid_application_context *context)
+join_transaction (GzochidApplicationContext *context)
 {
   gzochid_client_session_transaction_context *tx_context = NULL;
 
@@ -558,7 +577,7 @@ join_transaction (gzochid_application_context *context)
 }
 
 void 
-gzochid_client_session_disconnect (gzochid_application_context *context, 
+gzochid_client_session_disconnect (GzochidApplicationContext *context, 
 				   gzochid_client_session *session)
 {
   GError *err = NULL;
@@ -626,7 +645,7 @@ gzochid_client_session_set_handler (gzochid_client_session *session,
 }
 
 void 
-gzochid_client_session_send_login_success (gzochid_application_context *context,
+gzochid_client_session_send_login_success (GzochidApplicationContext *context,
 					   gzochid_client_session *session)
 {
   gzochid_client_session_transaction_context *tx_context =
@@ -645,7 +664,7 @@ gzochid_client_session_send_login_success (gzochid_application_context *context,
 }
 
 void 
-gzochid_client_session_send_login_failure (gzochid_application_context *context,
+gzochid_client_session_send_login_failure (GzochidApplicationContext *context,
 					   gzochid_client_session *session)
 {
   gzochid_client_session_transaction_context *tx_context =
@@ -664,7 +683,7 @@ gzochid_client_session_send_login_failure (gzochid_application_context *context,
 }
 
 void 
-gzochid_client_session_send_message (gzochid_application_context *context, 
+gzochid_client_session_send_message (GzochidApplicationContext *context, 
 				     gzochid_client_session *session, 
 				     unsigned char *msg, short len)
 {
@@ -710,7 +729,7 @@ gzochid_persistence_task_data_new (gzochid_io_serialization *serialization,
 }
 
 static void 
-persistence_task_worker (gzochid_application_context *context,
+persistence_task_worker (GzochidApplicationContext *context,
 			 gzochid_auth_identity *identity, gpointer data)
 {
   GError *local_err = NULL;
@@ -743,34 +762,23 @@ persistence_task_worker (gzochid_application_context *context,
   g_string_free (binding, TRUE);
 }
 
-static void
-persistence_task_cleanup_worker (gzochid_application_context *context,
-				 gzochid_auth_identity *identity, gpointer data)
-{
-  free (data);
-}
-
-gzochid_task *
+GzochidTask *
 gzochid_data_persistence_task_new
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *context, gzochid_auth_identity *identity, 
  gzochid_io_serialization *serialization, gpointer data, 
  gzochid_oid_holder *holder, char *prefix)
 {
   gzochid_persistence_task_data *task_data = gzochid_persistence_task_data_new 
     (serialization, data, holder, prefix);
   gzochid_application_task *task = gzochid_application_task_new
-    (context, identity, persistence_task_worker, task_data);
-  gzochid_application_task *cleanup_task = gzochid_application_task_new 
-    (context, identity, persistence_task_cleanup_worker, task_data);
+    (context, identity, persistence_task_worker, free, task_data);
   gzochid_transactional_application_task_execution *execution = 
-    gzochid_transactional_application_task_execution_new
-    (task, NULL, cleanup_task);
+    gzochid_transactional_application_task_execution_new (task, NULL, NULL);
 
-  /* Not necessary to hold a ref to these, as we've transferred them to the
+  /* Not necessary to hold a ref to this, as we've transferred it to the
      execution. */
   
   gzochid_application_task_unref (task);
-  gzochid_application_task_unref (cleanup_task);
   
   /* This task uses the re-executing task worker (in concert with 
      `gzochid_schedule_run_task' below) to ensure that the full transactional
@@ -779,25 +787,32 @@ gzochid_data_persistence_task_new
 
   gzochid_application_task *application_task = gzochid_application_task_new 
     (context, identity,
-     gzochid_application_reexecuting_transactional_task_worker, execution);
+     gzochid_application_reexecuting_transactional_task_worker,
+     (GDestroyNotify) gzochid_transactional_application_task_execution_free,
+     execution);
 
   return gzochid_task_immediate_new 
-    (gzochid_application_task_thread_worker, application_task);
+    (gzochid_application_task_thread_worker,
+     (GDestroyNotify) gzochid_application_task_unref, application_task);
 }
 
 void 
-gzochid_client_session_persist (gzochid_application_context *context, 
+gzochid_client_session_persist (GzochidApplicationContext *context, 
 				gzochid_client_session *session, guint64 *oid,
 				GError **err)
 {
+  GzochidTaskQueue *task_queue = NULL;
   gzochid_oid_holder *holder = gzochid_oid_holder_new ();
-  gzochid_task *task = gzochid_data_persistence_task_new
+  GzochidTask *task = gzochid_data_persistence_task_new
     (context, session->identity, &gzochid_client_session_serialization, session,
      holder, SESSION_PREFIX);
 
-  gzochid_schedule_run_task (context->task_queue, task);  
-  gzochid_task_free (task);
+  g_object_get (context, "task-queue", &task_queue, NULL);  
 
+  gzochid_schedule_run_task (task_queue, task);
+
+  g_object_unref (task_queue);
+  
   if (holder->err != NULL)
     {
       g_propagate_error (err, holder->err);
@@ -809,8 +824,7 @@ gzochid_client_session_persist (gzochid_application_context *context,
 }
 
 void 
-gzochid_sweep_client_sessions (gzochid_application_context *context,
-			       GError **err)
+gzochid_sweep_client_sessions (GzochidApplicationContext *context, GError **err)
 {
   GError *local_err = NULL;
   char *next_binding = NULL;
