@@ -1,5 +1,5 @@
 /* sessionclient-protocol.h: Prototypes and decls for sessionclient-protocol.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@
 
 /* A `gzochid_client_protocol' implementation for the sessionclient protocol. */
 
-gzochid_client_protocol gzochid_sessionclient_client_protocol;
+extern gzochid_client_protocol gzochid_sessionclient_client_protocol;
 
 #endif /* GZOCHID_SESSIONCLIENT_PROTOCOL_H */
