@@ -473,6 +473,17 @@ gzochid_client_socket_write (gzochid_client_socket *sock,
   g_mutex_unlock (&sock->sock_mutex);
 }
 
+void
+gzochid_client_socket_close (gzochid_client_socket *sock)
+{
+  /* Closing the channel doesn't actually "unplug" the client socket from the
+     server's main context. Rather, it'll cause the "error" flag to be set on 
+     the next poll, which will in turn cause the socket to not be re-added to
+     the main context and to then be subsequently fully unreffed. */
+  
+  g_io_channel_shutdown (sock->channel, TRUE, NULL);
+}
+
 gzochid_client_socket *
 gzochid_client_socket_ref (gzochid_client_socket *sock)
 {
@@ -501,6 +512,7 @@ gzochid_client_socket_unref (gzochid_client_socket *sock)
 	  g_source_unref (sock->write_source);
 	}
       
+      g_byte_array_unref (sock->send_buffer);
       g_mutex_unlock (&sock->sock_mutex);
       
       if (sock->read_source != NULL)
@@ -516,7 +528,6 @@ gzochid_client_socket_unref (gzochid_client_socket *sock)
       g_mutex_clear (&sock->sock_mutex);
 
       g_byte_array_unref (sock->recv_buffer);
-      g_byte_array_unref (sock->send_buffer);
 
       g_free (sock->connection_description);
 
