@@ -1,5 +1,5 @@
 /* storage-mem.c: Database storage routines for gzochid (in-memory)
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -2929,7 +2929,7 @@ mark_for_rollback (gzochid_storage_transaction *tx, gboolean should_retry)
    Returns NULL if no such leaf node exists.
 */
 
-static char *
+static unsigned char *
 get_internal (gzochid_storage_transaction *tx, gzochid_storage_store *store, 
 	      char *key, size_t key_len, size_t *value_len, gboolean update)
 {
@@ -2968,7 +2968,7 @@ get_internal (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 
       if (value != NULL)
 	{
-	  char *ret = malloc (sizeof (char) * tmp_value_len);
+	  unsigned char *ret = malloc (sizeof (char) * tmp_value_len);
 	  memcpy (ret, value, tmp_value_len);
 	  
 	  if (value_len != NULL)
@@ -2982,7 +2982,7 @@ get_internal (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 
 /* Returns the value for the specified key or NULL if none exists. */
 
-static char *
+static unsigned char *
 transaction_get (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 		 char *key, size_t key_len, size_t *value_len)
 {
@@ -3194,7 +3194,8 @@ put_internal (btree_transaction *btx, btree *database, btree_node *node,
 
 static void
 transaction_put (gzochid_storage_transaction *tx, gzochid_storage_store *store,
-		 char *key, size_t key_len, char *value, size_t value_len)
+		 char *key, size_t key_len, unsigned char *value,
+		 size_t value_len)
 {
   GError *err = NULL;
   btree_node *node = NULL;
@@ -3213,8 +3214,8 @@ transaction_put (gzochid_storage_transaction *tx, gzochid_storage_store *store,
       if (page != NULL)
 	{
 	  put_internal
-	    (btx, store->database, node, (unsigned char *) key, key_len,
-	     (unsigned char *) value, value_len, &err);
+	    (btx, store->database, node, (unsigned char *) key, key_len, value,
+	     value_len, &err);
 
 	  if (err != NULL)
 	    rollback = TRUE;
@@ -3223,7 +3224,7 @@ transaction_put (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 	{
 	  btree_node *new_leaf = insert_leaf_with_record
 	    (btx, store->database, node, NULL, (unsigned char *) key, key_len,
-	     (unsigned char *) value, value_len);
+	     value, value_len);
 
 	  if (new_leaf == NULL)
 	    rollback = TRUE;
@@ -3490,7 +3491,7 @@ transaction_next_key (gzochid_storage_transaction *tx,
    is found, a write lock is established on it before this function returns. 
 */
 
-static char *
+static unsigned char *
 transaction_get_for_update (gzochid_storage_transaction *tx, 
 			    gzochid_storage_store *store, char *key, 
 			    size_t key_len, size_t *value_len)
