@@ -1,5 +1,5 @@
 /* bdb.c: Database storage routines for gzochid (Berkeley DB)
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -158,8 +158,8 @@ open (gzochid_storage_context *context, char *path, unsigned int flags)
 
   char *filename = malloc (sizeof (char) * (pathlen + 4));
 
-  filename = strncpy (filename, path, pathlen + 1);
-  filename = strncat (filename, ".db", 3);
+  memcpy (filename, path, pathlen);
+  memcpy (filename + pathlen, ".db", 4);
   
   assert (db_create (&db, db_env, 0) == 0);
   db_flags = DB_THREAD;
@@ -178,6 +178,10 @@ open (gzochid_storage_context *context, char *path, unsigned int flags)
       g_warning
 	("Unable to open BDB database in %s: %s", filename, db_strerror (ret));
       dbopen_tx->abort (dbopen_tx);
+
+      free (filename);
+      free (store);
+
       return NULL;
     }
   dbopen_tx->commit (dbopen_tx, 0);
@@ -264,7 +268,7 @@ transaction_prepare (gzochid_storage_transaction *tx)
 {
 }
 
-static char *
+static unsigned char *
 transaction_get_internal (gzochid_storage_transaction *tx, 
 			  gzochid_storage_store *store, char *key, 
 			  size_t key_len, size_t *len, gboolean write_lock)
@@ -304,14 +308,14 @@ transaction_get_internal (gzochid_storage_transaction *tx,
     }
 }
 
-static char *
+static unsigned char *
 transaction_get (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 		 char *key, size_t key_len, size_t *len)
 {
   return transaction_get_internal (tx, store, key, key_len, len, FALSE);
 }
 
-static char *
+static unsigned char *
 transaction_get_for_update (gzochid_storage_transaction *tx, 
 			    gzochid_storage_store *store, char *key, 
 			    size_t key_len, size_t *len)
@@ -321,7 +325,8 @@ transaction_get_for_update (gzochid_storage_transaction *tx,
 
 static void 
 transaction_put (gzochid_storage_transaction *tx, gzochid_storage_store *store,
-		 char *key, size_t key_len, char *data, size_t data_len)
+		 char *key, size_t key_len, unsigned char *data,
+		 size_t data_len)
 {
   DB *db = store->database;
   DB_TXN *txn = tx->txn;
