@@ -1,5 +1,5 @@
 /* task.c: Application task management routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  */
 
 #include <glib.h>
+#include <glib-object.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <sys/time.h>
@@ -23,30 +24,52 @@
 #include "task.h"
 #include "threads.h"
 
-gzochid_task *
-gzochid_task_new (gzochid_thread_worker worker, gpointer data, 
-		  struct timeval target_execution_time)
+G_DEFINE_TYPE (GzochidTask, gzochid_task, G_TYPE_INITIALLY_UNOWNED);
+
+static void
+task_finalize (GObject *object)
 {
-  gzochid_task *task = malloc (sizeof (gzochid_task));
+  GzochidTask *task = GZOCHID_TASK (object);
+
+  if (task->destroy_notify != NULL)
+    task->destroy_notify (task->data);
+  
+  G_OBJECT_CLASS (gzochid_task_parent_class)->finalize (object);
+}
+
+static void
+gzochid_task_class_init (GzochidTaskClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->finalize = task_finalize;
+}
+
+static void
+gzochid_task_init (GzochidTask *task)
+{
+}
+
+GzochidTask *
+gzochid_task_new (gzochid_thread_worker worker, GDestroyNotify destroy_notify,
+		  gpointer data, struct timeval target_execution_time)
+{
+  GzochidTask *task = g_object_new (GZOCHID_TYPE_TASK, NULL);
   
   task->worker = worker;
+  task->destroy_notify = destroy_notify;
   task->data = data;
   task->target_execution_time = target_execution_time;
   
   return task;
 }
 
-gzochid_task *
-gzochid_task_immediate_new (gzochid_thread_worker worker, gpointer data)
+GzochidTask *
+gzochid_task_immediate_new (gzochid_thread_worker worker,
+			    GDestroyNotify destroy_notify, gpointer data)
 {
   struct timeval now;
   gettimeofday (&now, NULL);
 
-  return gzochid_task_new (worker, data, now);
-}
-
-void 
-gzochid_task_free (gzochid_task *task)
-{
-  free (task);
+  return gzochid_task_new (worker, destroy_notify, data, now);
 }
