@@ -1,5 +1,5 @@
 /* taskclient-protocol.h: Prototypes and declarations for taskclient-protocol.c
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@
 
 /* A `gzochid_client_protocol' implementation for the taskclient protocol. */
 
-gzochid_client_protocol gzochid_taskclient_client_protocol;
+extern gzochid_client_protocol gzochid_taskclient_client_protocol;
 
 #endif /* GZOCHID_TASKCLIENT_PROTOCOL_H */
