@@ -1,5 +1,5 @@
 /* threads.c: Threadpool implementation and thread support routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2019 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,23 +22,48 @@
 #include "threads.h"
 #include "tx.h"
 
-static void dispatcher (gpointer data, gpointer user_data)
+/* Wrapper struct for thread pool tasks. */
+
+struct _gzochid_thread_work
 {
-  gzochid_thread_work *work = (gzochid_thread_work *) data;
+  gzochid_thread_worker worker; /* The worker function. */
+  gpointer data; /* The work data pointer. */
+};
+
+typedef struct _gzochid_thread_work gzochid_thread_work;
+
+/*
+  Dispatcher function for thread pool tasks. Pass the work pointer to the 
+  function along with the configured closure pointer.
+*/
+
+static void
+dispatcher (gpointer data, gpointer user_data)
+{
+  gzochid_thread_work *work = data;
+
   work->worker (work->data, user_data);
   assert (!gzochid_transaction_active ());
   free (work);
 }
 
-GThreadPool *gzochid_thread_pool_new 
-(gpointer user_data, gint max_threads, gboolean exclusive, GError **error)
+GThreadPool *
+gzochid_thread_pool_new (gpointer user_data, gint max_threads,
+			 gboolean exclusive, GError **error)
 {
   return g_thread_pool_new 
     (dispatcher, user_data, max_threads, exclusive, error);
 }
 
-void gzochid_thread_pool_push 
-(GThreadPool *pool, gzochid_thread_worker worker, gpointer data, GError **error)
+void
+gzochid_thread_pool_free (GThreadPool *pool, gboolean immediate, gboolean wait)
+{
+  g_thread_pool_free (pool, immediate, wait);
+}
+
+void
+gzochid_thread_pool_push (GThreadPool *pool, gzochid_thread_worker worker,
+			  gpointer data, GError **error)
 {
   gzochid_thread_work *work = calloc (1, sizeof (gzochid_thread_work));
   
