@@ -1,5 +1,5 @@
 /* txlog.c: Transactional log routines for gzochid
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <syslog.h>
 
 #include "app.h"
+#include "descriptor.h"
 #include "tx.h"
 #include "txlog.h"
 
@@ -87,7 +88,11 @@ static void
 commit_message (gpointer data, gpointer user_data)
 {
   gzochid_log_message *message = data;
-  g_log (G_LOG_DOMAIN, message->priority, "%s", message->msg);
+  GzochidApplicationContext *context = user_data;
+  
+  g_log
+    (gzochid_application_context_get_log_domain (context), message->priority,
+     "%s", message->msg);
 }
 
 static void
@@ -97,7 +102,7 @@ log_commit (gpointer data)
   
   g_list_foreach 
     (tx_context->messages, (void (*) (gpointer, gpointer)) commit_message, 
-     NULL);
+     tx_context->context);
   cleanup_transaction (tx_context);
 }
 
@@ -128,12 +133,19 @@ gzochid_tx_vlog (GzochidApplicationContext *context, int priority, char *msg,
 {
   va_list app;
   va_copy (app, ap);
+  
+  GzochidApplicationDescriptor *descriptor = NULL;
   gzochid_log_transaction_context *tx_context = join_transaction (context);
 
-  tx_context->messages = g_list_append 
-    (tx_context->messages, 
-     gzochid_log_message_new (priority, g_strdup_vprintf (msg, app)));
+  g_object_get (context, "descriptor", &descriptor, NULL);
+  
+  if (priority <= descriptor->log_level)  
+    tx_context->messages = g_list_append 
+      (tx_context->messages, 
+       gzochid_log_message_new (priority, g_strdup_vprintf (msg, app)));
 
+  g_object_unref (descriptor);
+  
   va_end (app);
 }
 
