@@ -1,5 +1,5 @@
 /* txlog.h: Prototypes and declarations for txlog.c
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,11 +20,11 @@
 
 #include "app.h"
 
-void gzochid_tx_log (gzochid_application_context *, int, char *, ...);
-void gzochid_tx_err (gzochid_application_context *, char *, ...);
-void gzochid_tx_warning (gzochid_application_context *, char *, ...);
-void gzochid_tx_notice (gzochid_application_context *, char *, ...);
-void gzochid_tx_info (gzochid_application_context *, char *, ...);
-void gzochid_tx_debug (gzochid_application_context *, char *, ...);
+void gzochid_tx_log (GzochidApplicationContext *, int, char *, ...);
+void gzochid_tx_err (GzochidApplicationContext *, char *, ...);
+void gzochid_tx_warning (GzochidApplicationContext *, char *, ...);
+void gzochid_tx_notice (GzochidApplicationContext *, char *, ...);
+void gzochid_tx_info (GzochidApplicationContext *, char *, ...);
+void gzochid_tx_debug (GzochidApplicationContext *, char *, ...);
 
 #endif /* GZOCHID_TX_LOG_H */
