@@ -1,5 +1,5 @@
 /* util.c: Assorted utility routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -268,8 +268,7 @@ gzochid_util_string_data_compare (gconstpointer a, gconstpointer b,
 }
 
 gint
-gzochid_util_guint64_data_compare (gconstpointer a, gconstpointer b,
-				   gpointer user_data)
+gzochid_util_guint64_compare (gconstpointer a, gconstpointer b)
 {
   const guint64 an = *(const guint64 *) a;
   const guint64 bn = *(const guint64 *) b;
@@ -277,6 +276,13 @@ gzochid_util_guint64_data_compare (gconstpointer a, gconstpointer b,
   return an < bn ? -1 : an > bn ? 1 : 0;
 }
 
+gint
+gzochid_util_guint64_data_compare (gconstpointer a, gconstpointer b,
+				   gpointer user_data)
+{
+  return gzochid_util_guint64_compare (a, b);
+}
+
 gint
 gzochid_util_bytes_compare_null_first (gconstpointer o1, gconstpointer o2)
 {
