@@ -1,5 +1,5 @@
 /* mock-data.c: Test-time replacements for data.c routines.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -200,7 +200,7 @@ gzochid_data_dereference (gzochid_data_managed_reference *reference,
     }
   
   in = g_byte_array_sized_new (data->len);
-  g_byte_array_append (in, data->str, data->len);
+  g_byte_array_append (in, (const guint8 *) data->str, data->len);
   
   reference->obj = reference->serialization->deserializer 
     (reference->context, in, NULL);
