@@ -1,5 +1,5 @@
 ;; gzochi/test-tx.scm: Scheme unit tests for gzochi external transaction API
-;; Copyright (C) 2015 Julian Graham
+;; Copyright (C) 2020 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -47,9 +47,9 @@
 (test-begin "gzochi:join-transaction")
 
 (define join-participant #f)
-(module-set! (resolve-module '(gzochi tx)) 
-	     'primitive-join-transaction 
-	     (lambda (participant) (set! join-participant participant)))
+
+(gzochi:set-primitive-join-transaction! 
+ (lambda (participant) (set! join-participant participant)))
 
 (test-group "join"
   (let ((participant (make-test-transaction-participant)))
@@ -67,11 +67,10 @@
 (define abort-participant #f)
 (define abort-retryable? *unspecified*)
 
-(module-set! (resolve-module '(gzochi tx)) 
-	     'primitive-abort-transaction 
-	     (lambda* (participant #:optional retryable?) 
-	       (set! abort-participant participant)
-	       (set! abort-retryable? retryable?)))
+(gzochi:set-primitive-abort-transaction!
+ (lambda* (participant #:optional retryable?) 
+	  (set! abort-participant participant)
+	  (set! abort-retryable? retryable?)))
 
 (test-group "abort"
   (let ((participant (make-test-transaction-participant)))
