@@ -1,5 +1,5 @@
 /* test-auth.c: Test routines for auth.c in gzochid.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,10 +19,18 @@
 #include <glib-object.h>
 #include <stddef.h>
 
+#include "app.h"
 #include "auth_int.h"
 #include "config.h"
 #include "gzochid-auth.h"
 
+gzochid_auth_identity *
+gzochid_application_context_resolve_identity
+(GzochidApplicationContext *app_context, char *name)
+{
+  return gzochid_auth_identity_new (name);
+}
+
 static void
 test_identity_ref ()
 {
