@@ -1,5 +1,5 @@
 /* test-channel.c: Tests for channel.c in gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <fcntl.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <libguile.h>
 #include <stddef.h>
 #include <stdlib.h>
-#include <unistd.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 
 #include "app.h"
 #include "app-factory.h"
@@ -43,18 +43,18 @@ struct _channel_fixture
 typedef struct _channel_fixture channel_fixture;
 
 static gzochid_client_socket *
-create_pipe_client (int *read_fd)
+create_socket_client (int *read_fd)
 {
-  int pipefd[2] = { 0 };
-  int pipe_fd = pipe (pipefd);
-  GIOChannel *write_channel = g_io_channel_unix_new (pipefd[1]);  
+  int socketfd[2] = { 0 };
+  int socket_fd = socketpair(PF_LOCAL, SOCK_STREAM, 0, socketfd);
+  GIOChannel *write_channel = g_io_channel_unix_new (socketfd[1]);  
   gzochid_client_socket *client_socket = gzochid_game_server_protocol.accept
     (write_channel, "", NULL);
 
   g_io_channel_set_encoding (write_channel, NULL, NULL);
 
   if (read_fd != NULL)
-    *read_fd = pipefd[0];
+    *read_fd = socketfd[0];
 
   return client_socket;
 }
@@ -65,7 +65,7 @@ channel_fixture_setup (channel_fixture *fixture, gconstpointer user_data)
   int read_fd = 0;
   
   fixture->app_context = gzochid_create_test_application ();
-  fixture->client_socket = create_pipe_client (&read_fd);    
+  fixture->client_socket = create_socket_client (&read_fd);    
   fixture->game_client = _gzochid_client_socket_get_protocol_data
     (fixture->client_socket);
 
