@@ -1,5 +1,5 @@
 /* test-data.c: Test routines for data.c in gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ static void *test_deserializer
 (GzochidApplicationContext *context, GByteArray *in, GError **err)
 {
   deserialized = TRUE;
-  return g_string_new_len (in->data, in->len);
+  return g_string_new_len ((const gchar *) in->data, in->len);
 }
 
 static void test_finalizer (GzochidApplicationContext *context, void *ptr)
@@ -156,7 +156,6 @@ test_data_reference_by_ptr_failure_oids ()
 
   /* Swap in a mock oid allocation strategy. */
   
-  gzochid_oid_allocation_strategy_free (app_store->oid_strategy);
   app_store->oid_strategy = gzochid_oid_allocation_strategy_new
     (allocate_fail, NULL, NULL);
   
@@ -167,6 +166,7 @@ test_data_reference_by_ptr_failure_oids ()
     (data_reference_by_ptr_failure_oids_inner, context);
   gzochid_storage_engine_interface_mem.transaction_rollback (tx);  
 
+  gzochid_oid_allocation_strategy_free (app_store->oid_strategy);
   g_object_unref (context);
 }
 
@@ -186,7 +186,8 @@ test_data_reference_finalize ()
     (app_store->storage_context);
 
   gzochid_storage_engine_interface_mem.transaction_put 
-    (tx, app_store->oids, (char *) &zero, sizeof (guint64), "foo", 4);
+    (tx, app_store->oids, (char *) &zero, sizeof (guint64),
+     (unsigned char *) "foo", 4);
 
   gzochid_storage_engine_interface_mem.transaction_prepare (tx);  
   gzochid_storage_engine_interface_mem.transaction_commit (tx);  
