@@ -1,5 +1,5 @@
 /* test-dataclient.c: Tests for dataclient.c in gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -52,7 +52,9 @@ struct _dataclient_data_callback_data
   struct timeval timeout;
 
   dataclient_fixture *fixture;
+  gboolean failure;
   gboolean released;
+  gboolean success;
 };
 
 typedef struct _dataclient_data_callback_data dataclient_data_callback_data;
@@ -105,6 +107,7 @@ success_callback (GBytes *value, gpointer user_data)
 {
   dataclient_data_callback_data *callback_data = user_data;
   callback_data->value = g_bytes_ref (value);
+  callback_data->success = TRUE;
 }
 
 static void
@@ -112,6 +115,7 @@ failure_callback (struct timeval wait_time, gpointer user_data)
 {
   dataclient_data_callback_data *callback_data = user_data;
   callback_data->timeout = wait_time;
+  callback_data->failure = TRUE;
 }
 
 static void
@@ -201,16 +205,28 @@ static void
 test_request_value_simple (dataclient_fixture *fixture, gconstpointer user_data)
 {
   GBytes *key = g_bytes_new_static ("foo", 4);
+  dataclient_data_callback_data callback_data = { 0 };
+  gzochid_data_response response;
   
+  response.app = "test";
+  response.store = "oids";
+  response.success = TRUE;
+  response.data = g_bytes_new_static ("bar", 4);
+
   gzochid_dataclient_request_value
     (fixture->dataclient, "test", "oids", key, TRUE,
-     success_callback, NULL, failure_callback, NULL, release_callback, NULL);
+     success_callback, &callback_data, failure_callback, NULL,
+     release_callback, NULL);
 
   g_assert_cmpint (fixture->bytes_received->len, ==, 20);  
   g_assert
     (memcmp (fixture->bytes_received->data,
 	     "\x00\x11\x21test\x00oids\x00\x01\x00\x04""foo\x00", 20) == 0);
 
+  gzochid_dataclient_received_value (fixture->dataclient, &response);
+
+  g_bytes_unref (response.data);
+  g_bytes_unref (callback_data.value);  
   g_bytes_unref (key);
 }
 
@@ -322,15 +338,27 @@ test_request_next_key_simple (dataclient_fixture *fixture,
 			      gconstpointer user_data)
 {
   GBytes *key = g_bytes_new_static ("foo", 4);
-  
+  dataclient_data_callback_data callback_data = { 0 };
+  gzochid_data_response response;
+
+  response.app = "test";
+  response.store = "names";
+  response.success = TRUE;
+  response.data = g_bytes_new_static ("bar", 4);
+
   gzochid_dataclient_request_next_key
     (fixture->dataclient, "test", "names", key,
-     success_callback, NULL, failure_callback, NULL, release_callback, NULL);
+     success_callback, &callback_data, failure_callback, NULL,
+     release_callback, NULL);
 
   g_assert_cmpint (fixture->bytes_received->len, ==, 20);  
   g_assert (memcmp (fixture->bytes_received->data,
 		    "\x00\x11\x22test\x00names\x00\x00\x04""foo\x00", 20) == 0);
 
+  gzochid_dataclient_received_next_key (fixture->dataclient, &response);
+
+  g_bytes_unref (response.data);
+  g_bytes_unref (callback_data.value);
   g_bytes_unref (key);
 }
 
@@ -421,6 +449,26 @@ test_received_next_key_unexpected (dataclient_fixture *fixture,
   g_bytes_unref (response.data);
 }
 
+static void
+test_disconnected_simple (dataclient_fixture *fixture, gconstpointer user_data)
+{
+  GBytes *key = g_bytes_new_static ("foo", 4);
+  dataclient_data_callback_data callback_data = { 0 };
+
+  g_test_log_set_fatal_handler (ignore_warnings, NULL);
+
+  gzochid_dataclient_request_value
+    (fixture->dataclient, "test", "oids", key, TRUE,
+     success_callback, &callback_data, failure_callback, &callback_data,
+     release_callback, NULL);
+
+  gzochid_dataclient_disconnected (fixture->dataclient);
+
+  g_assert (callback_data.failure == TRUE);
+
+  g_bytes_unref (key);
+}
+
 static void
 test_submit_changeset_simple (dataclient_fixture *fixture,
 			      gconstpointer user_data)
@@ -463,13 +511,16 @@ test_submit_changeset_simple (dataclient_fixture *fixture,
   changeset = gzochid_data_changeset_new ("test", 0, changes);
   g_byte_array_append
     (expected_outbound_message_array,
-     "\x00\x0d\x32test\x00\x00\x00\x00\x00\x00\x00\x00\x00", 16);
-  g_byte_array_append (expected_outbound_message_array, "\x00\x47\x30", 3);
+     (const guint8 *) "\x00\x0d\x32test\x00\x00\x00\x00\x00\x00\x00\x00\x00",
+     16);
+  g_byte_array_append (expected_outbound_message_array,
+		       (const guint8 *) "\x00\x47\x30", 3);
   gzochid_data_protocol_changeset_write
     (changeset, expected_outbound_message_array);
   g_byte_array_append
     (expected_outbound_message_array,
-     "\x00\x0d\x34test\x00\x00\x00\x00\x00\x00\x00\x00\x00", 16);
+     (const guint8 *) "\x00\x0d\x34test\x00\x00\x00\x00\x00\x00\x00\x00\x00",
+     16);
   gzochid_data_changeset_free (changeset);
   
   gzochid_dataclient_submit_changeset (fixture->dataclient, "test", changes);
@@ -587,6 +638,11 @@ main (int argc, char *argv[])
      dataclient_fixture_setup, test_received_next_key_unexpected,
      dataclient_fixture_teardown);
 
+  g_test_add
+    ("/dataclient/disconnected/simple", dataclient_fixture, NULL,
+     dataclient_fixture_setup, test_disconnected_simple,
+     dataclient_fixture_teardown);
+
   g_test_add
     ("/dataclient/submit-changeset/simple", dataclient_fixture, NULL,
      dataclient_fixture_setup, test_submit_changeset_simple,
