@@ -1,5 +1,5 @@
 /* test-dataserver-protocol.c: Test routines for test-dataserver-protocol.c.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 
 #include "data-protocol.h"
 #include "dataserver.h"
@@ -34,6 +35,8 @@
 struct _GzochiMetadDataServer
 {
   GObject parent_instance;
+
+  gboolean set_error;
 };
 
 G_DEFINE_TYPE (GzochiMetadDataServer, gzochi_metad_data_server, G_TYPE_OBJECT);
@@ -48,6 +51,12 @@ gzochi_metad_data_server_init (GzochiMetadDataServer *self)
 {
 }
 
+GQuark
+gzochi_metad_dataserver_error_quark ()
+{
+  return g_quark_from_static_string ("gzochi-metad-dataserver-error-quark");
+}
+
 static GList *activity_log = NULL;
 
 static void
@@ -194,7 +203,11 @@ gzochi_metad_dataserver_end_changeset (GzochiMetadDataServer *dataserver,
 {
   activity_log = g_list_append
     (activity_log, g_strdup_printf ("END CHANGESET/%s/%" G_GUINT64_FORMAT, app,
-				    changeset_id)); 
+				    changeset_id));
+
+  if (dataserver->set_error)
+    g_set_error (err, GZOCHI_METAD_DATASERVER_ERROR,
+		 GZOCHI_METAD_DATASERVER_ERROR_FAILED, "An error occurred");
 }
 
 struct _metaserver_wrapper_client
@@ -256,13 +269,23 @@ struct _dataserver_protocol_fixture
 
 typedef struct _dataserver_protocol_fixture dataserver_protocol_fixture;
 
+static gboolean
+ignore_warnings (const gchar *log_domain, GLogLevelFlags log_level,
+                 const gchar *message, gpointer user_data)
+{
+  if (log_level & G_LOG_LEVEL_CRITICAL
+      || log_level & G_LOG_LEVEL_WARNING)
+    return FALSE;
+  else return log_level & G_LOG_FLAG_FATAL;
+}
+
 static void
 dataserver_protocol_fixture_set_up (dataserver_protocol_fixture *fixture,
 				    gconstpointer user_data)
 {
-  int pipefd[2] = { 0 };
-  int pipe_fd = pipe (pipefd);
-  GIOChannel *write_channel = g_io_channel_unix_new (pipefd[1]);
+  int socketfd[2] = { 0 };
+  int socket_fd = socketpair (PF_LOCAL, SOCK_STREAM, 0, socketfd);
+  GIOChannel *write_channel = g_io_channel_unix_new (socketfd[1]);
   metaserver_wrapper_client *wrapper_client =
     malloc (sizeof (metaserver_wrapper_client));
   gzochid_client_socket *client_socket = gzochid_client_socket_new
@@ -273,13 +296,14 @@ dataserver_protocol_fixture_set_up (dataserver_protocol_fixture *fixture,
   fixture->dataserver = gzochid_resolver_require
     (GZOCHI_METAD_TYPE_DATA_SERVER, NULL);
 
-  fixture->read_channel = g_io_channel_unix_new (pipefd[0]);
+  fixture->read_channel = g_io_channel_unix_new (socketfd[0]);
   fixture->write_channel = write_channel;
   fixture->client = wrapper_client;
   
   wrapper_client->dataserver_client = gzochi_metad_dataserver_client_new
     (fixture->dataserver, client_socket, 1);
 
+  g_io_channel_set_encoding (fixture->read_channel, NULL, NULL);
   g_io_channel_set_flags (fixture->read_channel, G_IO_FLAG_NONBLOCK, NULL);  
   g_io_channel_set_encoding (write_channel, NULL, NULL);
   g_io_channel_set_flags (write_channel, G_IO_FLAG_NONBLOCK, NULL);
@@ -306,7 +330,8 @@ test_client_can_dispatch_true (dataserver_protocol_fixture *fixture,
 {
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x18\x10\x02http://localhost:8080/", 27);
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x18\x10\x02http://localhost:8080/", 27);
 
   g_assert (wrapper_protocol.can_dispatch (bytes, fixture->client));
 
@@ -319,7 +344,7 @@ test_client_can_dispatch_false (dataserver_protocol_fixture *fixture,
 {
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x18\x10\x02http", 9);
+  g_byte_array_append (bytes, (const guint8 *) "\x00\x18\x10\x02http", 9);
 
   g_assert (! wrapper_protocol.can_dispatch (bytes, fixture->client));
 
@@ -333,7 +358,7 @@ test_client_dispatch_one_reserve_oids (dataserver_protocol_fixture *fixture,
   char buf[18];
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x05\x20test", 8);
+  g_byte_array_append (bytes, (const guint8 *) "\x00\x05\x20test", 8);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -357,7 +382,8 @@ test_client_dispatch_one_request_value (dataserver_protocol_fixture *fixture,
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (bytes, "\x00\x0f\x21test\x00oids\x00\x00\x00\x02""1", 18);
+    (bytes, (const guint8 *) "\x00\x0f\x21test\x00oids\x00\x00\x00\x02""1",
+     18);
   
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
@@ -381,7 +407,8 @@ test_client_dispatch_one_request_next_key
   char buf[22];
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x11\x22test\x00names\x00\x00\x04""foo", 20);
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x11\x22test\x00names\x00\x00\x04""foo", 20);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -403,7 +430,8 @@ test_client_dispatch_one_release_key (dataserver_protocol_fixture *fixture,
 {
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x0e\x40test\x00oids\x00\x00\x02""1", 17);
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x0e\x40test\x00oids\x00\x00\x02""1", 17);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -418,8 +446,8 @@ test_client_dispatch_one_release_range
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (bytes, "\x00\x19\x42test\x00names\x00\x00\x05""foo1\x00\x00\x05""foo2\x00",
-     28);
+    (bytes, (const guint8 *) "\x00\x19\x42test\x00names\x00\x00\x05"
+     "foo1\x00\x00\x05""foo2\x00", 28);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -435,7 +463,8 @@ test_client_dispatch_one_begin_changeset (dataserver_protocol_fixture *fixture,
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (bytes, "\x00\x0d\x32test\x00\x00\x00\x00\x00\x00\x00\x00\x7b", 16);
+    (bytes, (const guint8 *) "\x00\x0d\x32test\x00\x00\x00\x00\x00\x00\x00"
+     "\x00\x7b", 16);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -449,13 +478,18 @@ test_client_dispatch_one_process_changeset
 {
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x47\x30test", 8);
-  g_byte_array_append (bytes, "\x00\x00\x00\x00\x00\x00\x00\x7b", 8);
-  g_byte_array_append (bytes, "\x00\x04", 2);
-  g_byte_array_append (bytes, "oids\x00\x00\x02""1\x00\x00\x00", 11);
-  g_byte_array_append (bytes, "oids\x00\x00\x02""2\x00\x00\x04""foo", 15);
-  g_byte_array_append (bytes, "names\x00\x00\x04""foo\x00\x00\x00", 14);
-  g_byte_array_append (bytes, "names\x00\x00\x04""bar\x00\x00\x02""3", 16);
+  g_byte_array_append (bytes, (const guint8 *) "\x00\x47\x30test", 8);
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x00\x00\x00\x00\x00\x00\x7b", 8);
+  g_byte_array_append (bytes, (const guint8 *) "\x00\x04", 2);
+  g_byte_array_append
+    (bytes, (const guint8 *) "oids\x00\x00\x02""1\x00\x00\x00", 11);
+  g_byte_array_append
+    (bytes, (const guint8 *) "oids\x00\x00\x02""2\x00\x00\x04""foo", 15);
+  g_byte_array_append
+    (bytes, (const guint8 *) "names\x00\x00\x04""foo\x00\x00\x00", 14);
+  g_byte_array_append
+    (bytes, (const guint8 *) "names\x00\x00\x04""bar\x00\x00\x02""3", 16);
 
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
@@ -478,7 +512,8 @@ test_client_dispatch_one_end_changeset (dataserver_protocol_fixture *fixture,
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (bytes, "\x00\x0d\x34test\x00\x00\x00\x00\x00\x00\x00\x00\x7b", 16);
+    (bytes, (const guint8 *) "\x00\x0d\x34test\x00\x00\x00\x00\x00\x00\x00"
+     "\x00\x7b", 16);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -486,6 +521,28 @@ test_client_dispatch_one_end_changeset (dataserver_protocol_fixture *fixture,
   g_assert_cmpstr ((char *) activity_log->data, ==, "END CHANGESET/test/123");
 }
 
+static void
+test_client_dispatch_one_end_changeset_failure
+(dataserver_protocol_fixture *fixture, gconstpointer user_data)
+{
+  char buf[1];
+  GByteArray *bytes = g_byte_array_new ();
+
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x0d\x34test\x00\x00\x00\x00\x00\x00\x00"
+     "\x00\x7b", 16);
+
+  fixture->dataserver->set_error = TRUE;
+  g_test_log_set_fatal_handler (ignore_warnings, NULL);
+ 
+  wrapper_protocol.dispatch (bytes, fixture->client);
+  g_byte_array_unref (bytes);
+
+  g_assert_cmpint
+    (g_io_channel_read_chars (fixture->read_channel, buf, 1, NULL, NULL), ==,
+     G_IO_STATUS_EOF);
+}
+
 static void
 test_client_dispatch_multiple (dataserver_protocol_fixture *fixture,
 			       gconstpointer user_data)
@@ -493,9 +550,11 @@ test_client_dispatch_multiple (dataserver_protocol_fixture *fixture,
   GByteArray *bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (bytes, "\x00\x11\x40test\x00names\x00""\x00\x04""foo", 20);
+    (bytes, (const guint8 *) "\x00\x11\x40test\x00names\x00""\x00\x04"
+     "foo", 20);
   g_byte_array_append
-    (bytes, "\x00\x11\x40test\x00names\x00""\x00\x04""bar", 20);
+    (bytes, (const guint8 *) "\x00\x11\x40test\x00names\x00""\x00\x04"
+     "bar", 20);
   wrapper_protocol.dispatch (bytes, fixture->client);
   g_byte_array_unref (bytes);
 
@@ -582,6 +641,12 @@ main (int argc, char *argv[])
      dataserver_protocol_fixture_set_up, test_client_dispatch_one_end_changeset,
      dataserver_protocol_fixture_tear_down);
 
+  g_test_add
+    ("/client/dispatch/one/end-changeset/failure", dataserver_protocol_fixture,
+     NULL, dataserver_protocol_fixture_set_up,
+     test_client_dispatch_one_end_changeset_failure,
+     dataserver_protocol_fixture_tear_down);
+
   g_test_add ("/client/dispatch/multiple", dataserver_protocol_fixture, NULL,
 	      dataserver_protocol_fixture_set_up, test_client_dispatch_multiple,
 	      dataserver_protocol_fixture_tear_down);
