@@ -1,5 +1,5 @@
 /* test-dataserver.c: Tests for dataserver.c in gzochi-metad.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -111,8 +111,8 @@ struct _dataserver_fixture
 typedef struct _dataserver_fixture dataserver_fixture;
 
 static void
-put (gzochid_storage_store *store, char *key, size_t key_len, char *value,
-     size_t value_len)
+put (gzochid_storage_store *store, char *key, size_t key_len,
+     unsigned char *value, size_t value_len)
 {
   gzochid_storage_engine_interface *iface =
     &gzochid_storage_engine_interface_mem;
@@ -179,7 +179,7 @@ test_request_value (dataserver_fixture *fixture, gconstpointer user_data)
   test_storage_initialize (NULL);
   test_storage_open (test_context, "test/oids", GZOCHID_STORAGE_CREATE);
   
-  put (oids, "1", 2, "foo", 4);
+  put (oids, "1", 2, (unsigned char *) "foo", 4);
   
   response = gzochi_metad_dataserver_request_value
     (fixture->server, 1, "test", "oids", key, FALSE, NULL);
@@ -242,8 +242,8 @@ test_request_next_key (dataserver_fixture *fixture, gconstpointer user_data)
   test_storage_initialize (NULL);
   test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
-  put (names, "a", 2, "1", 2);
-  put (names, "b", 2, "2", 2);
+  put (names, "a", 2, (unsigned char *) "1", 2);
+  put (names, "b", 2, (unsigned char *) "2", 2);
 
   response1 = gzochi_metad_dataserver_request_next_key
     (fixture->server, 1, "test", "names", key, NULL);
@@ -275,7 +275,7 @@ test_request_next_key_null (dataserver_fixture *fixture,
   test_storage_initialize (NULL);
   test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
-  put (names, "a", 2, "1", 2);
+  put (names, "a", 2, (unsigned char *) "1", 2);
 
   response = gzochi_metad_dataserver_request_next_key
     (fixture->server, 1, "test", "names", NULL, NULL);
@@ -298,7 +298,7 @@ test_request_next_key_not_found (dataserver_fixture *fixture,
   test_storage_initialize (NULL);
   test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
-  put (names, "a", 2, "1", 2);
+  put (names, "a", 2, (unsigned char *) "1", 2);
 
   response = gzochi_metad_dataserver_request_next_key
     (fixture->server, 1, "test", "names", key, NULL);
@@ -322,9 +322,9 @@ test_request_next_key_failure (dataserver_fixture *fixture,
   test_storage_initialize (NULL);
   test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
-  put (names, "a", 2, "1", 2);
-  put (names, "b", 2, "2", 2);
-  put (names, "c", 2, "3", 2);
+  put (names, "a", 2, (unsigned char *) "1", 2);
+  put (names, "b", 2, (unsigned char *) "2", 2);
+  put (names, "c", 2, (unsigned char *) "3", 2);
 
   response1 = gzochi_metad_dataserver_request_next_key
     (fixture->server, 1, "test", "names", key1, NULL);
@@ -452,7 +452,7 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   GError *err = NULL;
   GArray *changes = g_array_sized_new
     (FALSE, FALSE, sizeof (gzochid_data_change), 6);
-  char *data = NULL;
+  unsigned char *data = NULL;
   
   gzochid_storage_engine_interface *iface =
     &gzochid_storage_engine_interface_mem;
@@ -519,11 +519,11 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   obtain_lock (fixture->server, "test", "names", binding_change2.key);
   obtain_lock (fixture->server, "test", "names", binding_change3.key);
   
-  put (oids, "1", 2, "foo", 4);
-  put (oids, "2", 2, "bar", 4);
+  put (oids, "1", 2, (unsigned char *) "foo", 4);
+  put (oids, "2", 2, (unsigned char *) "bar", 4);
 
-  put (names, "binding-1", 10, "1", 2);
-  put (names, "binding-2", 10, "2", 2);
+  put (names, "binding-1", 10, (unsigned char *) "1", 2);
+  put (names, "binding-2", 10, (unsigned char *) "2", 2);
 
   gzochi_metad_dataserver_begin_changeset
     (fixture->server, 1, "test", 123, &err);
@@ -542,7 +542,7 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   data = iface->transaction_get (transaction, oids, "1", 2, NULL);
 
   g_assert (data != NULL);
-  g_assert_cmpstr (data, ==, "bar");
+  g_assert_cmpstr ((const char *) data, ==, "bar");
   free (data);
 
   data = iface->transaction_get (transaction, oids, "2", 2, NULL);
@@ -551,13 +551,13 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   data = iface->transaction_get (transaction, oids, "3", 2, NULL);
 
   g_assert (data != NULL);
-  g_assert_cmpstr (data, ==, "baz");
+  g_assert_cmpstr ((const char *) data, ==, "baz");
   free (data);
 
   data = iface->transaction_get (transaction, names, "binding-1", 10, NULL);
 
   g_assert (data != NULL);
-  g_assert_cmpstr (data, ==, "2");
+  g_assert_cmpstr ((const char *) data, ==, "2");
   free (data);
   
   data = iface->transaction_get (transaction, names, "binding-2", 10, NULL);
@@ -566,7 +566,7 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   data = iface->transaction_get (transaction, names, "binding-3", 10, NULL);
 
   g_assert (data != NULL);  
-  g_assert_cmpstr (data, ==, "3");
+  g_assert_cmpstr ((const char *) data, ==, "3");
   free (data);
   
   iface->transaction_rollback (transaction);
