@@ -1,5 +1,5 @@
 /* test-durable-task.c: Test routines for durable-task.c in gzochid.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,83 +16,56 @@
  */
 
 #include <glib.h>
+#include <libguile.h>
+#include <stddef.h>
 #include <stdlib.h>
 
 #include "app.h"
+#include "app-factory.h"
+#include "app-store.h"
 #include "app-task.h"
+#include "config.h"
 #include "data.h"
 #include "durable-task.h"
 #include "game.h"
+#include "guile.h"
 #include "gzochid-auth.h"
 #include "oids.h"
 #include "oids-storage.h"
 #include "schedule.h"
+#include "scheme.h"
+#include "scheme-task.h"
 #include "storage-mem.h"
 
 static GMutex test_worker_mutex;
 static GCond test_worker_cond;
+static gboolean test_complete;
 
 struct _test_context
 {
-  gzochid_application_context *app_context;
+  GzochidApplicationContext *app_context;
   gzochid_auth_identity *identity;
-  guint64 handle_oid;
 };
 
 typedef struct _test_context test_context;
 
-static void 
-application_context_init (gzochid_application_context *context)
-{
-  context->storage_engine_interface = &gzochid_storage_engine_interface_mem;
-  context->storage_context = 
-    gzochid_storage_engine_interface_mem.initialize ("/dev/null");
-  context->meta = gzochid_storage_engine_interface_mem.open
-    (context->storage_context, "/dev/null", 0);
-  context->oids = gzochid_storage_engine_interface_mem.open 
-    (context->storage_context, "/dev/null", 0);
-  context->names = gzochid_storage_engine_interface_mem.open 
-    (context->storage_context, "/dev/null", 0);
-
-  context->oid_strategy = gzochid_storage_oid_strategy_new
-    (context->storage_engine_interface, context->storage_context,
-     context->meta);
-
-  context->identity_cache = gzochid_auth_identity_cache_new ();
-}
-
-static void
-application_context_clear (gzochid_application_context *context)
-{
-  gzochid_storage_engine_interface_mem.close_store (context->meta);
-  gzochid_storage_engine_interface_mem.close_store (context->oids);
-  gzochid_storage_engine_interface_mem.close_store (context->names);
-
-  gzochid_storage_engine_interface_mem.close_context (context->storage_context);
-  
-  gzochid_oid_allocation_strategy_free (context->oid_strategy);
-  
-  gzochid_auth_identity_cache_destroy (context->identity_cache);
-}
-
 static GString *test_worker_string;
 
 static void
-serialize_test_worker_string (gzochid_application_context *context,
+serialize_test_worker_string (GzochidApplicationContext *context,
 			      gpointer data, GByteArray *out, GError **error)
 {
 }
 
 static gpointer
-deserialize_test_worker_string (gzochid_application_context *context,
+deserialize_test_worker_string (GzochidApplicationContext *context,
 				GByteArray *in, GError **error)
 {
   return test_worker_string;
 }
 
 static void
-finalize_test_worker_string (gzochid_application_context *context,
-			     gpointer data)
+finalize_test_worker_string (GzochidApplicationContext *context, gpointer data)
 {
 }
 
@@ -104,13 +77,13 @@ static gzochid_io_serialization test_worker_string_serialization =
   };
 
 static void
-serialize_task_worker_noop (gzochid_application_context *context,
+serialize_task_worker_noop (GzochidApplicationContext *context,
 			    gzochid_application_worker worker, GByteArray *out)
 {
 }
 
 static void
-test_string_worker_a (gzochid_application_context *context,
+test_string_worker_a (GzochidApplicationContext *context,
 		      gzochid_auth_identity *identity, gpointer data)
 {
   GString *str = data;
@@ -118,7 +91,7 @@ test_string_worker_a (gzochid_application_context *context,
 }
 
 static gzochid_application_worker
-deserialize_task_worker_a (gzochid_application_context *context, GByteArray *in)
+deserialize_task_worker_a (GzochidApplicationContext *context, GByteArray *in)
 {
   return test_string_worker_a;
 }
@@ -134,7 +107,7 @@ static gzochid_application_task_serialization string_task_serialization_a =
   };
 
 static void
-test_string_worker_b (gzochid_application_context *context,
+test_string_worker_b (GzochidApplicationContext *context,
 		      gzochid_auth_identity *identity, gpointer data)
 {
   GString *str = data;
@@ -142,7 +115,7 @@ test_string_worker_b (gzochid_application_context *context,
 }
 
 static gzochid_application_worker
-deserialize_task_worker_b (gzochid_application_context *context, GByteArray *in)
+deserialize_task_worker_b (GzochidApplicationContext *context, GByteArray *in)
 {
   return test_string_worker_b;
 }
@@ -158,19 +131,20 @@ static gzochid_application_task_serialization string_task_serialization_b =
   };
 
 static void
-test_string_worker_c (gzochid_application_context *context,
+test_string_worker_c (GzochidApplicationContext *context,
 		      gzochid_auth_identity *identity, gpointer data)
 {
   GString *str = data;
   g_string_append (str, "c");
 
   g_mutex_lock (&test_worker_mutex);
+  test_complete = TRUE;
   g_cond_signal (&test_worker_cond);
   g_mutex_unlock (&test_worker_mutex);
 }
 
 static gzochid_application_worker
-deserialize_task_worker_c (gzochid_application_context *context, GByteArray *in)
+deserialize_task_worker_c (GzochidApplicationContext *context, GByteArray *in)
 {
   return test_string_worker_c;
 }
@@ -194,13 +168,13 @@ test_task_chain_inner0 (gpointer data)
   test_worker_string = g_string_new ("");
   
   gzochid_application_task *task_a = gzochid_application_task_new
-    (context->app_context, context->identity, test_string_worker_a,
+    (context->app_context, context->identity, test_string_worker_a, NULL,
      test_worker_string);
   gzochid_application_task *task_b = gzochid_application_task_new
-    (context->app_context, context->identity, test_string_worker_b,
+    (context->app_context, context->identity, test_string_worker_b, NULL,
      test_worker_string);
   gzochid_application_task *task_c = gzochid_application_task_new
-    (context->app_context, context->identity, test_string_worker_c,
+    (context->app_context, context->identity, test_string_worker_c, NULL,
      test_worker_string);
 
   gzochid_durable_application_task_handle *handle_a =
@@ -234,59 +208,85 @@ test_task_chain_inner0 (gpointer data)
   gzochid_application_task_unref (task_c);
 }
 
+static GzochidConfiguration *
+create_configuration ()
+{
+  GKeyFile *key_file = g_key_file_new ();
+  GzochidConfiguration *configuration = NULL;
+
+  g_key_file_set_value (key_file, "game", "lock.release.msec", "10");
+  g_key_file_set_value (key_file, "game", "tx.timeout", "2147483647");
+  
+  configuration = g_object_new
+    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, NULL);
+  
+  g_key_file_unref (key_file);
+
+  return configuration;
+}
+
 static void 
 test_task_chain_simple ()
 {
   test_context context;
-  gboolean submitted = FALSE, done = FALSE;
-
-  gzochid_application_context *app_context = 
-    gzochid_application_context_new ();
+  GzochidConfiguration *config = create_configuration ();  
+  GzochidApplicationContext *app_context =
+    gzochid_create_test_application_with_config (config);
   gzochid_auth_identity *identity = gzochid_auth_identity_new ("test");
-  GThreadPool *pool = gzochid_thread_pool_new (NULL, 1, TRUE, NULL);
-
-  application_context_init (app_context);
-  app_context->tx_timeout = (struct timeval) { INT_MAX, INT_MAX };
-  app_context->task_queue = gzochid_schedule_task_queue_new (pool);
-
-  gzochid_schedule_task_queue_start (app_context->task_queue);    
+  GzochidTaskQueue *task_queue = NULL;
 
+  g_object_get (app_context, "task-queue", &task_queue, NULL);
+  gzochid_schedule_task_queue_start (task_queue);
+  g_object_unref (task_queue);
+  
   context.app_context = app_context;
   context.identity = identity;
-  context.handle_oid = 0;
 
   g_mutex_lock (&test_worker_mutex);
   
+  gzochid_wait_for_start (app_context);
   gzochid_transaction_execute (test_task_chain_inner0, &context);
-  
-  g_cond_wait (&test_worker_cond, &test_worker_mutex);
+
+  while (!test_complete)
+    g_cond_wait (&test_worker_cond, &test_worker_mutex);
   g_mutex_unlock (&test_worker_mutex);
 
   g_assert_cmpstr (test_worker_string->str, ==, "abc");  
-
-  gzochid_schedule_task_queue_stop (app_context->task_queue);
-  gzochid_schedule_task_queue_free (app_context->task_queue);
-
-  g_thread_pool_free (pool, TRUE, TRUE);
-  application_context_clear (app_context);
-  gzochid_application_context_free (app_context);
+  g_string_free (test_worker_string, TRUE);
+  
   gzochid_auth_identity_unref (identity);
+  g_object_unref (app_context);
+
+  g_object_unref (config);
 }
 
-int
-main (int argc, char *argv[])
+static void
+inner_main (void *data, int argc, char *argv[])
 {
   g_cond_init (&test_worker_cond);
   g_mutex_init (&test_worker_mutex);
 
   gzochid_task_initialize_serialization_registry ();
+  gzochid_task_register_serialization
+    (&gzochid_task_chain_coordinator_task_serialization);
   gzochid_task_register_serialization (&string_task_serialization_a);
   gzochid_task_register_serialization (&string_task_serialization_b);
   gzochid_task_register_serialization (&string_task_serialization_c);
 
   g_test_init (&argc, &argv, NULL);
 
-  g_test_add_func ("/dutable-task/chain/simple", test_task_chain_simple);
+  g_test_add_func ("/durable-task/chain/simple", test_task_chain_simple);
 
-  return g_test_run ();
+  gzochid_guile_init ();
+  gzochid_scheme_initialize_bindings ();
+  gzochid_scheme_task_initialize_bindings ();
+  exit (g_test_run ());
+}
+
+int
+main (int argc, char *argv[])
+{
+  scm_boot_guile (argc, argv, inner_main, NULL);
+
+  return 0;
 }
