@@ -1,5 +1,5 @@
 /* test-metaclient.c: Tests for metaclient.c in gzochid.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -35,6 +35,11 @@
 #include "sessionclient.h"
 #include "taskclient.h"
 
+gzochid_client_protocol gzochid_channelclient_client_protocol;
+gzochid_client_protocol gzochid_dataclient_client_protocol;
+gzochid_client_protocol gzochid_sessionclient_client_protocol;
+gzochid_client_protocol gzochid_taskclient_client_protocol;
+
 struct _GzochidHttpServer
 {
   GObject parent_instance;
@@ -294,7 +299,6 @@ gzochid_task_client_init (GzochidTaskClient *self)
 struct _metaclient_fixture
 {
   GzochidResolutionContext *resolution_context;
-  GzochidMetaClient *metaclient;
 
   GzochidSocketServer *socket_server;
   gzochid_server_socket *server_socket;
@@ -380,11 +384,6 @@ metaclient_fixture_setup_inner (metaclient_fixture *fixture, GKeyFile *key_file)
 
   g_free (server_address);
 
-  fixture->metaclient = gzochid_resolver_require_full
-    (resolution_context, GZOCHID_TYPE_META_CLIENT, NULL);
-  
-  gzochid_metaclient_start (fixture->metaclient, NULL);
-
   g_object_unref (configuration);
 }
 
@@ -416,9 +415,6 @@ static void
 metaclient_fixture_teardown (metaclient_fixture *fixture,
 			     gconstpointer user_data)
 {
-  gzochid_metaclient_stop (fixture->metaclient);
-  
-  g_object_unref (fixture->metaclient);
   g_object_unref (fixture->resolution_context);
   g_object_unref (fixture->socket_server);
   
@@ -446,24 +442,40 @@ set_timeout (metaclient_fixture *fixture, guint interval)
 static void
 test_login_simple (metaclient_fixture *fixture, gconstpointer user_data)
 {
+  GzochidMetaClient *metaclient = gzochid_resolver_require_full
+    (fixture->resolution_context, GZOCHID_TYPE_META_CLIENT, NULL);
+  
+  gzochid_metaclient_start (metaclient, NULL);
+
   set_timeout (fixture, 1000);
   g_main_loop_run (fixture->socket_server->main_loop);
 
   g_assert_cmpint (fixture->bytes_received->len, ==, 5);  
   g_assert
     (memcmp (fixture->bytes_received->data, "\x00\x02\x10\x02\x00", 5) == 0);
+
+  gzochid_metaclient_stop (metaclient);
+  g_object_unref (metaclient);
 }
 
 static void
 test_login_http_server_enabled (metaclient_fixture *fixture,
 				gconstpointer user_data)
 {
+  GzochidMetaClient *metaclient = gzochid_resolver_require_full
+    (fixture->resolution_context, GZOCHID_TYPE_META_CLIENT, NULL);
+  
+  gzochid_metaclient_start (metaclient, NULL);
+
   set_timeout (fixture, 1000);
   g_main_loop_run (fixture->socket_server->main_loop);
 
   g_assert_cmpint (fixture->bytes_received->len, ==, 22);  
   g_assert (memcmp (fixture->bytes_received->data,
 		    "\x00\x13\x10\x02http://127.0.0.1/\x00", 22) == 0);
+
+  gzochid_metaclient_stop (metaclient);
+  g_object_unref (metaclient);
 }
 
 struct callback_data
@@ -495,11 +507,19 @@ static void
 test_nullify_connection (metaclient_fixture *fixture, gconstpointer user_data)
 {
   gzochid_event_source *event_source = NULL;
-  GzochidEventLoop *event_loop = gzochid_resolver_require_full
-    (fixture->resolution_context, GZOCHID_TYPE_EVENT_LOOP, NULL);
+  GzochidMetaClient *metaclient = NULL;
+  GzochidEventLoop *event_loop = g_object_new (GZOCHID_TYPE_EVENT_LOOP, NULL);
   struct callback_data callback_data;
 
-  g_object_get (fixture->metaclient, "event-source", &event_source, NULL);
+  gzochid_resolver_provide
+    (fixture->resolution_context, G_OBJECT(event_loop), NULL);
+  
+  metaclient = gzochid_resolver_require_full
+    (fixture->resolution_context, GZOCHID_TYPE_META_CLIENT, NULL);
+  
+  gzochid_metaclient_start (metaclient, NULL);
+  
+  g_object_get (metaclient, "event-source", &event_source, NULL);
   gzochid_event_attach
     (event_source, disconnected_event_handler, &callback_data);
   g_source_unref ((GSource *) event_source);
@@ -514,11 +534,13 @@ test_nullify_connection (metaclient_fixture *fixture, gconstpointer user_data)
 
   g_mutex_lock (&callback_data.mutex);  
   
-  gzochid_metaclient_nullify_connection (fixture->metaclient);
+  gzochid_metaclient_nullify_connection (metaclient);
 
-  g_cond_wait_until
-    (&callback_data.cond, &callback_data.mutex,
-     g_get_monotonic_time () + 100000);
+  while (g_cond_wait_until
+	 (&callback_data.cond, &callback_data.mutex,
+	  g_get_monotonic_time () + 1 * G_TIME_SPAN_SECOND))
+    if (callback_data.handled)
+      break;
 
   g_mutex_unlock (&callback_data.mutex);
   
@@ -530,6 +552,9 @@ test_nullify_connection (metaclient_fixture *fixture, gconstpointer user_data)
   g_cond_clear (&callback_data.cond);
   
   g_object_unref (event_loop);
+
+  gzochid_metaclient_stop (metaclient);
+  g_object_unref (metaclient);
 }
 
 static void
