@@ -1,5 +1,5 @@
 /* test-metaserver-protocol.c: Test routines for test-metaserver-protocol.c.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -442,13 +442,13 @@ metaserver_protocol_fixture_set_up (metaserver_protocol_fixture *fixture,
   _gzochid_server_socket_getsockname (fixture->server_socket, &addr, &addrlen);
   connect (socket_fd, &addr, addrlen);
 
-  g_assert (g_main_context_iteration
-	    (fixture->socket_server->main_context, TRUE));
+  while (g_main_context_iteration
+	 (fixture->socket_server->main_context, FALSE));
 
   g_assert (fixture->client_socket != NULL);
-
+  
   fixture->socket_channel = g_io_channel_unix_new (socket_fd);
-  g_io_channel_set_flags (fixture->socket_channel, G_IO_FLAG_NONBLOCK, NULL);
+  g_io_channel_set_encoding (fixture->socket_channel, NULL, NULL);
 }
 
 static void
@@ -503,7 +503,8 @@ test_client_can_dispatch_true (metaserver_protocol_fixture *fixture,
 
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x18\x10\x02http://localhost:8080/", 27);
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x18\x10\x02http://localhost:8080/", 27);
 
   g_assert
     (gzochi_metad_metaserver_client_protocol.can_dispatch (bytes, client));
@@ -520,7 +521,7 @@ test_client_can_dispatch_false (metaserver_protocol_fixture *fixture,
 
   GByteArray *bytes = g_byte_array_new ();
 
-  g_byte_array_append (bytes, "\x00\x18\x10\x02http", 9);
+  g_byte_array_append (bytes, (const guint8 *) "\x00\x18\x10\x02http", 9);
 
   g_assert
     (! gzochi_metad_metaserver_client_protocol.can_dispatch (bytes, client));
@@ -572,10 +573,18 @@ pump_login (GzochiMetadRootContext *root_context,
   
   gzochi_metad_metaserver_client_protocol.dispatch (bytes, client);
 
-  g_cond_wait_until
-    (&callback_data->cond, &callback_data->mutex,
-     g_get_monotonic_time () + 100000);
-  g_mutex_unlock (&callback_data->mutex);
+  while (TRUE)
+    {
+      g_cond_wait_until
+	(&callback_data->cond, &callback_data->mutex,
+	 g_get_monotonic_time () + 100000);
+
+      if (callback_data->handled)
+	{
+	  g_mutex_unlock (&callback_data->mutex);
+	  break;
+	}
+    }
 
   gzochid_event_loop_stop (event_loop);
   g_object_unref (event_loop);
@@ -590,25 +599,37 @@ test_client_dispatch_one_login (metaserver_protocol_fixture *fixture,
   
   char buf[27];
   GByteArray *bytes = g_byte_array_new ();
+  GError *err = NULL;
   struct callback_data callback_data;
+  size_t total_bytes = 0;
 
   g_mutex_init (&callback_data.mutex);
   g_cond_init (&callback_data.cond);
   callback_data.handled = FALSE;
   
-  g_byte_array_append (bytes, "\x00\x02\x10\x02\x00", 5);
+  g_byte_array_append (bytes, (const guint8 *) "\x00\x02\x10\x02\x00", 5);
   pump_login (fixture->root_context, client, bytes, &callback_data);
   g_byte_array_unref (bytes);
   
   g_assert (callback_data.handled);
   g_mutex_clear (&callback_data.mutex);
   g_cond_clear (&callback_data.cond);
-  
-  g_main_context_iteration (fixture->socket_server->main_context, FALSE);
-  
-  g_assert_cmpint
-    (g_io_channel_read_chars (fixture->socket_channel, buf, 27, NULL, NULL), ==,
-     G_IO_STATUS_NORMAL);
+
+  while (g_main_context_iteration
+	 (fixture->socket_server->main_context, FALSE));
+
+  while (total_bytes < 27)
+    {
+      size_t bytes_read = 0;
+      size_t remaining_bytes = 27 - total_bytes;
+
+      g_io_channel_read_chars
+	(fixture->socket_channel, buf + total_bytes, remaining_bytes,
+	 &bytes_read, &err);
+
+      g_assert_no_error (err);
+      total_bytes += bytes_read;
+    }
 
   g_assert (memcmp (buf, "\x00\x18\x11\x02http://localhost:8081/", 27) == 0);
   g_assert (channelserver_connected_flag);
@@ -625,13 +646,16 @@ test_client_dispatch_one_login_base_url (metaserver_protocol_fixture *fixture,
   
   char buf[27];
   GByteArray *bytes = g_byte_array_new ();
+  GError *err = NULL;
   struct callback_data callback_data;
+  size_t total_bytes = 0;
 
   g_mutex_init (&callback_data.mutex);
   g_cond_init (&callback_data.cond);
   callback_data.handled = FALSE;
   
-  g_byte_array_append (bytes, "\x00\x18\x10\x02http://localhost:8080/", 27);
+  g_byte_array_append
+    (bytes, (const guint8 *) "\x00\x18\x10\x02http://localhost:8080/", 27);
   pump_login (fixture->root_context, client, bytes, &callback_data);
   g_assert (callback_data.handled);
   
@@ -639,11 +663,21 @@ test_client_dispatch_one_login_base_url (metaserver_protocol_fixture *fixture,
   g_cond_clear (&callback_data.cond);
   g_byte_array_unref (bytes);
   
-  g_main_context_iteration (fixture->socket_server->main_context, FALSE);
+  while (g_main_context_iteration
+	 (fixture->socket_server->main_context, FALSE));
   
-  g_assert_cmpint
-    (g_io_channel_read_chars (fixture->socket_channel, buf, 27, NULL, NULL), ==,
-     G_IO_STATUS_NORMAL);
+  while (total_bytes < 27)
+    {
+      size_t bytes_read = 0;
+      size_t remaining_bytes = 27 - total_bytes;
+
+      g_io_channel_read_chars
+	(fixture->socket_channel, buf + total_bytes, remaining_bytes,
+	 &bytes_read, &err);
+
+      g_assert_no_error (err);
+      total_bytes += bytes_read;
+    }
 
   g_assert(memcmp (buf, "\x00\x18\x11\x02http://localhost:8081/", 27) == 0);
   g_assert (channelserver_connected_flag);
@@ -663,7 +697,8 @@ test_client_dispatch_one_channel (metaserver_protocol_fixture *fixture,
   GByteArray *input_bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (input_bytes, "\x00\x0c\x76""foo\x00\x00\x00\x00\x00\x00\x00\x00\x01", 15);
+    (input_bytes, (const guint8 *) "\x00\x0c\x76""foo\x00\x00\x00\x00\x00"
+     "\x00\x00\x00\x01", 15);
   gzochi_metad_metaserver_client_protocol.dispatch (input_bytes, client);  
   g_byte_array_unref (input_bytes);
   
@@ -683,7 +718,7 @@ test_client_dispatch_one_data (metaserver_protocol_fixture *fixture,
   GBytes *expected_bytes = g_bytes_new_static ("\x00\x04\x20""foo", 7);
   GByteArray *input_bytes = g_byte_array_new ();
 
-  g_byte_array_append (input_bytes, "\x00\x04\x20""foo", 7);
+  g_byte_array_append (input_bytes, (const guint8 *) "\x00\x04\x20""foo", 7);
   gzochi_metad_metaserver_client_protocol.dispatch (input_bytes, client);  
   g_byte_array_unref (input_bytes);
   
@@ -705,7 +740,8 @@ test_client_dispatch_one_session (metaserver_protocol_fixture *fixture,
   GByteArray *input_bytes = g_byte_array_new ();
 
   g_byte_array_append
-    (input_bytes, "\x00\x0c\x60""foo\x00\x00\x00\x00\x00\x00\x00\x00\x01", 15);
+    (input_bytes, (const guint8 *) "\x00\x0c\x60""foo\x00\x00\x00\x00\x00"
+     "\x00\x00\x00\x01", 15);
   gzochi_metad_metaserver_client_protocol.dispatch (input_bytes, client);  
   g_byte_array_unref (input_bytes);
   
@@ -725,7 +761,8 @@ test_client_dispatch_one_task (metaserver_protocol_fixture *fixture,
   GBytes *expected_bytes = g_bytes_new_static ("\x00\x04\x80""foo\x00", 7);
   GByteArray *input_bytes = g_byte_array_new ();
 
-  g_byte_array_append (input_bytes, "\x00\x04\x80""foo\x00", 7);
+  g_byte_array_append
+    (input_bytes, (const guint8 *) "\x00\x04\x80""foo\x00", 7);
   gzochi_metad_metaserver_client_protocol.dispatch (input_bytes, client);  
   g_byte_array_unref (input_bytes);
   
