@@ -1,5 +1,5 @@
 /* test-queue.c: Test routines for queue.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  */
 
 #include <glib.h>
+#include <glib-object.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,21 +31,21 @@
 /* Some toy serialization routines for strings. */
 
 static void
-serialize_string (gzochid_application_context *app_context, gpointer data,
+serialize_string (GzochidApplicationContext *app_context, gpointer data,
 		  GByteArray *out, GError **err)
 {
   gzochid_util_serialize_string (data, out);
 }
 
 static gpointer
-deserialize_string (gzochid_application_context *app_context, GByteArray *in,
+deserialize_string (GzochidApplicationContext *app_context, GByteArray *in,
 		    GError **err)
 {
   return gzochid_util_deserialize_string (in);
 }
 
 static void
-finalize_string (gzochid_application_context *app_context, gpointer data)
+finalize_string (GzochidApplicationContext *app_context, gpointer data)
 {
   free (data);
 }
@@ -52,12 +53,19 @@ finalize_string (gzochid_application_context *app_context, gpointer data)
 static gzochid_io_serialization string_serialization =
   { serialize_string, deserialize_string, finalize_string };
 
+void
+gzochid_mock_application_install_properties (GObjectClass *klass)
+{
+}
+
 static void
 test_queue_offer ()
 {
   char *data = NULL;
   GError *err = NULL;
-  gzochid_durable_queue *queue = gzochid_durable_queue_new (NULL);
+  GzochidApplicationContext *app_context = g_object_new
+    (GZOCHID_TYPE_APPLICATION_CONTEXT, NULL);
+  gzochid_durable_queue *queue = gzochid_durable_queue_new (app_context);
   
   gzochid_durable_queue_offer
     (queue, &string_serialization, strdup ("foo"), &err);
@@ -72,6 +80,7 @@ test_queue_offer ()
   g_assert_cmpstr (data, ==, "bar");
   
   gzochid_durable_queue_free (queue);
+  g_object_unref (app_context);
 }
 
 static void
@@ -79,7 +88,9 @@ test_queue_peek ()
 {
   char *data = NULL;
   GError *err = NULL;
-  gzochid_durable_queue *queue = gzochid_durable_queue_new (NULL);
+  GzochidApplicationContext *app_context = g_object_new
+    (GZOCHID_TYPE_APPLICATION_CONTEXT, NULL);
+  gzochid_durable_queue *queue = gzochid_durable_queue_new (app_context);
   
   gzochid_durable_queue_offer
     (queue, &string_serialization, strdup ("foo"), NULL);
@@ -95,13 +106,16 @@ test_queue_peek ()
   g_assert_no_error (err);
   
   gzochid_durable_queue_free (queue);
+  g_object_unref (app_context);
 }
 
 static void
 test_queue_pop ()
 {
   GError *err = NULL;
-  gzochid_durable_queue *queue = gzochid_durable_queue_new (NULL);
+  GzochidApplicationContext *app_context = g_object_new
+    (GZOCHID_TYPE_APPLICATION_CONTEXT, NULL);
+  gzochid_durable_queue *queue = gzochid_durable_queue_new (app_context);
   
   gzochid_durable_queue_offer
     (queue, &string_serialization, strdup ("foo"), NULL);
@@ -114,6 +128,7 @@ test_queue_pop ()
   g_assert_no_error (err);
   
   gzochid_durable_queue_free (queue);
+  g_object_unref (app_context);
 }
 
 int
