@@ -1,5 +1,5 @@
 /* test-scheme-task.c: Test routines for scheme-task.c in gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -109,7 +109,7 @@ persist_client_session (struct test_scheme_task_fixture *fixture,
   app_store->iface->transaction_put
     (tx, app_store->oids, "1", 2, out->data, out->len);
   app_store->iface->transaction_put
-    (tx, app_store->names, "s.session.1", 12, "1", 2);
+    (tx, app_store->names, "s.session.1", 12, (unsigned char *) "1", 2);
 
   app_store->iface->transaction_prepare (tx);
   app_store->iface->transaction_commit (tx);
