@@ -1,5 +1,5 @@
 /* test-socket.c: Test routines for socket.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,18 +16,14 @@
  */
 
 #include <assert.h>
-#include <fcntl.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <glib-unix.h>
 #include <stddef.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <unistd.h>
 
-#include "context.h"
 #include "log.h"
 #include "protocol.h"
 #include "socket.h"
@@ -132,9 +128,11 @@ test_socket_fixture_set_up (test_socket_fixture *fixture,
   _gzochid_server_socket_getsockname (fixture->server_socket, &addr, &addrlen);
   connect (fixture->client_socket_fd, &addr, addrlen);
 
-  g_assert
-    (g_main_context_iteration
-     (fixture->socket_server->main_context, FALSE));
+  do
+    {
+      g_main_context_iteration (fixture->socket_server->main_context, FALSE);
+    }
+  while (g_main_context_pending (fixture->socket_server->main_context));
 
   g_assert (fixture->client_socket != NULL);
 }
@@ -160,7 +158,11 @@ test_socket_client_dispatch (test_socket_fixture *fixture,
 {
   write (fixture->client_socket_fd, "\x01\x02\x03", 3);
 
-  g_main_context_iteration (fixture->socket_server->main_context, FALSE);
+  do
+    {
+      g_main_context_iteration (fixture->socket_server->main_context, FALSE);
+    }
+  while (g_main_context_pending (fixture->socket_server->main_context));
   
   g_assert_cmpint (fixture->state->can_dispatch_called, ==, 1);
   g_assert_cmpint (fixture->state->dispatch_called, ==, 1);
@@ -171,28 +173,28 @@ static void
 test_socket_client_listen ()
 {
   GError *err = NULL;
-  int pipe_flags = 0;
-  gint pipe_fds[2] = { 0 };
+  int flags = 0;
+  gint socket_fds[2] = { 0 };
   test_client_state state = { 0 };
   GzochidSocketServer *socket_server = g_object_new
     (GZOCHID_TYPE_SOCKET_SERVER, NULL);
   gzochid_client_socket *client_socket = NULL;
+  GIOChannel *read_channel = NULL;
 
-  g_unix_open_pipe (pipe_fds, 0, &err);
+  socketpair (PF_LOCAL, SOCK_STREAM, 0, socket_fds);
   g_assert_no_error (err);
 
-  pipe_flags = fcntl (pipe_fds[0], F_GETFL, 0);
-  fcntl (pipe_fds[0], F_SETFL, pipe_flags | O_NONBLOCK);
-
-  pipe_flags = fcntl (pipe_fds[1], F_GETFL, 0);
-  fcntl (pipe_fds[1], F_SETFL, pipe_flags | O_NONBLOCK);
-
+  read_channel = g_io_channel_unix_new (socket_fds[0]);
+  g_io_channel_set_encoding (read_channel, NULL, NULL);
+  g_io_channel_set_flags (read_channel, G_IO_FLAG_NONBLOCK, NULL);
+  g_io_channel_set_buffered (read_channel, FALSE);
+  
   client_socket = gzochid_client_socket_new
-    (g_io_channel_unix_new (pipe_fds[0]), "test", test_client_protocol, &state);
+    (read_channel, "test", test_client_protocol, &state);
 
   gzochid_client_socket_listen (socket_server, client_socket);
 
-  write (pipe_fds[1], "\x01\x02\x03", 3);
+  assert (write (socket_fds[1], "\x01\x02\x03", 3) == 3);
   
   g_main_context_iteration (socket_server->main_context, FALSE);
   g_assert_cmpint (state.can_dispatch_called, ==, 1);
@@ -207,7 +209,7 @@ test_reconnectable_socket_simple (test_socket_fixture *fixture,
 {
   gzochid_reconnectable_socket *rsock = gzochid_reconnectable_socket_new ();
 
-  gzochid_reconnectable_socket_write (rsock, "foo", 4);
+  gzochid_reconnectable_socket_write (rsock, (unsigned char *) "foo", 4);
   g_assert (!g_main_context_iteration
 	    (fixture->socket_server->main_context, FALSE));
 
