@@ -1,5 +1,5 @@
 /* test-storage-dataclient.c: Tests for storage-dataclient.c in gzochid.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -776,7 +776,8 @@ test_put_cached (dataclient_storage_fixture *fixture, gconstpointer user_data)
     (tx, fixture->store, "foo", 4, NULL);
   free (value);
 
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "baz", 4);
 
   g_assert (! tx->rollback);
   
@@ -812,7 +813,8 @@ test_put_cached_upgrade (dataclient_storage_fixture *fixture,
   value = fixture->iface->transaction_get (tx, fixture->store, "foo", 4, NULL);
   free (value);
 
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "baz", 4);
 
   g_assert (! tx->rollback);
   
@@ -842,7 +844,8 @@ test_put_uncached_success (dataclient_storage_fixture *fixture,
   fixture->dataclient->responses = g_list_append
     (fixture->dataclient->responses, response);
   
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "baz", 4);
 
   g_assert (! tx->rollback);
   
@@ -876,7 +879,8 @@ test_put_uncached_failure_success (dataclient_storage_fixture *fixture,
   fixture->dataclient->responses = g_list_append
     (fixture->dataclient->responses, response2);
   
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "baz", 4);
 
   g_assert (! tx->rollback);
   
@@ -898,7 +902,8 @@ test_put_uncached_timeout (dataclient_storage_fixture *fixture,
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin_timed
     (fixture->storage_context, (struct timeval) { 0, 100000 });
 
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar", 4);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "bar", 4);
 
   g_assert (tx->rollback);
   g_assert (tx->should_retry);
@@ -1073,7 +1078,7 @@ test_first_key_cached (dataclient_storage_fixture *fixture,
 		       gconstpointer user_data)
 {
   size_t key_len = 0;  
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin
     (fixture->storage_context);
 
@@ -1108,7 +1113,7 @@ test_first_key_uncached_success (dataclient_storage_fixture *fixture,
 				 gconstpointer user_data)
 {
   size_t key_len = 0;  
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin
     (fixture->storage_context);
 
@@ -1136,7 +1141,7 @@ test_first_key_uncached_failure_success (dataclient_storage_fixture *fixture,
 					 gconstpointer user_data)
 {
   size_t key_len = 0;  
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin_timed
     (fixture->storage_context, (struct timeval) { 0, 100000 });
   dataclient_storage_response *response1 = create_failure_response
@@ -1171,7 +1176,7 @@ static void
 test_first_key_uncached_timeout (dataclient_storage_fixture *fixture,
 				 gconstpointer user_data)
 {
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin_timed
     (fixture->storage_context, (struct timeval) { 0, 100000 });
 
@@ -1191,7 +1196,7 @@ test_next_key_cached (dataclient_storage_fixture *fixture,
 		      gconstpointer user_data)
 {
   size_t key_len = 0;  
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin
     (fixture->storage_context);
 
@@ -1228,7 +1233,7 @@ test_next_key_uncached_success (dataclient_storage_fixture *fixture,
 				gconstpointer user_data)
 {
   size_t key_len = 0;  
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin
     (fixture->storage_context);
 
@@ -1257,7 +1262,7 @@ test_next_key_uncached_failure_success (dataclient_storage_fixture *fixture,
 					gconstpointer user_data)
 {
   size_t key_len = 0;  
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin_timed
     (fixture->storage_context, (struct timeval) { 0, 100000 });
   dataclient_storage_response *response1 = create_failure_response
@@ -1293,7 +1298,7 @@ static void
 test_next_key_uncached_timeout (dataclient_storage_fixture *fixture,
 				gconstpointer user_data)
 {
-  unsigned char *key = NULL;
+  char *key = NULL;
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin_timed
     (fixture->storage_context, (struct timeval) { 0, 100000 });
 
@@ -1318,7 +1323,7 @@ test_lock_release_eviction (dataclient_storage_fixture *fixture,
   dataclient_storage_response *response = create_success_response (value_bytes);
   gzochid_storage_transaction *tx = fixture->iface->transaction_begin
     (fixture->storage_context);
-  char *val = NULL;
+  unsigned char *val = NULL;
   
   fixture->dataclient->responses = g_list_append
     (fixture->dataclient->responses, response);
@@ -1398,9 +1403,12 @@ test_compaction_simple (dataclient_storage_fixture *fixture,
   
   tx = fixture->iface->transaction_begin (fixture->storage_context);
   
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar1", 5);
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar2", 5);
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar3", 5);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "bar1", 5);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "bar2", 5);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "bar3", 5);
     
   fixture->iface->transaction_prepare (tx);
   fixture->iface->transaction_commit (tx);
@@ -1425,7 +1433,7 @@ test_cache_store_consistency (dataclient_storage_fixture *fixture,
   GBytes *value2_bytes = g_bytes_new_static ("baz", 4);
   GBytes *value3_bytes = g_bytes_new_static ("quux", 4);
 
-  char *value = NULL;
+  unsigned char *value = NULL;
   size_t value_len = 0;
   GBytes *value_bytes = NULL;
   
@@ -1454,7 +1462,8 @@ test_cache_store_consistency (dataclient_storage_fixture *fixture,
 
   g_mutex_lock (&fixture->dataclient->process_response_mutex);
   
-  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
+  fixture->iface->transaction_put
+    (tx, fixture->store, "foo", 4, (unsigned char *) "baz", 4);
 
   g_cond_wait (&fixture->dataclient->process_response_cond,
 	       &fixture->dataclient->process_response_mutex);
