@@ -1,5 +1,5 @@
 /* test-taskclient.c: Tests for taskclient.c in gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "app-task.h"
 #include "auth_int.h"
 #include "data.h"
+#include "descriptor.h"
 #include "durable-task.h"
 #include "event.h"
 #include "game.h"
@@ -42,22 +43,26 @@ gzochid_mock_application_install_properties (GObjectClass *object_class)
     (object_class, 1, g_param_spec_pointer
      ("app-state", "app-state", "app-state", G_PARAM_READWRITE));
   g_object_class_install_property
-    (object_class, 2, g_param_spec_boxed
+    (object_class, 2, g_param_spec_object
+     ("descriptor", "descriptor", "descriptor", G_TYPE_OBJECT,
+      G_PARAM_READWRITE));
+  g_object_class_install_property
+    (object_class, 3, g_param_spec_boxed
      ("event-source", "event-source", "event-source", G_TYPE_SOURCE,
       G_PARAM_READWRITE));
   g_object_class_install_property
-    (object_class, 3, g_param_spec_object
+    (object_class, 4, g_param_spec_object
      ("metaclient", "metaclient", "metaclient", G_TYPE_OBJECT,
       G_PARAM_READWRITE));
   g_object_class_install_property
-    (object_class, 4, g_param_spec_pointer
+    (object_class, 5, g_param_spec_pointer
      ("name", "name", "name", G_PARAM_READWRITE));
   g_object_class_install_property
-    (object_class, 5, g_param_spec_object 
+    (object_class, 6, g_param_spec_object 
      ("task-queue", "task-queue", "task-queue", G_TYPE_OBJECT,
       G_PARAM_READWRITE));
   g_object_class_install_property
-    (object_class, 6, g_param_spec_uint64
+    (object_class, 7, g_param_spec_uint64
      ("tx-timeout", "tx-timeout", "tx-timeout", 0, G_MAXUINT64, G_MAXUINT64,
       G_PARAM_READWRITE));
 }
@@ -228,6 +233,7 @@ gzochid_meta_client_init (GzochidMetaClient *self)
 
 struct _taskclient_fixture
 {
+  GzochidApplicationDescriptor *descriptor;
   GzochidGameServer *game_server;
   gzochid_reconnectable_socket *socket;
   GzochidMetaClient *metaclient;
@@ -312,7 +318,9 @@ taskclient_fixture_setup (taskclient_fixture *fixture, gconstpointer user_data)
 {
   guint64 tx_timeout_ms = 10000;
   gzochid_event_source *event_source = gzochid_event_source_new ();
-  
+
+  fixture->descriptor = g_object_new
+    (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
   fixture->game_server = g_object_new (GZOCHID_TYPE_GAME_SERVER, NULL);
   fixture->socket = gzochid_reconnectable_socket_new ();
   fixture->task_queue = g_object_new (GZOCHID_TYPE_TASK_QUEUE, NULL);
@@ -328,6 +336,7 @@ taskclient_fixture_setup (taskclient_fixture *fixture, gconstpointer user_data)
   fixture->app_context = g_object_new
     (GZOCHID_TYPE_APPLICATION_CONTEXT,
      "app-state", gzochid_application_state_new (),
+     "descriptor", fixture->descriptor,
      "event-source", event_source,
      "metaclient", fixture->metaclient,
      "name", "test",
@@ -382,6 +391,7 @@ taskclient_fixture_teardown (taskclient_fixture *fixture,
   g_object_get (fixture->app_context, "app-state", &state, NULL);
   gzochid_application_state_free (state);
 
+  g_object_unref (fixture->descriptor);
   g_object_unref (fixture->game_server);
   gzochid_reconnectable_socket_free (fixture->socket);
   g_object_unref (fixture->task_queue);
@@ -514,7 +524,8 @@ test_complete_task (taskclient_fixture *fixture, gconstpointer user_data)
   
   g_private_set (&name_key, NULL);
   
-  g_byte_array_append (expected_arr, "\x00\x0d\x86test\x00", 8);
+  g_byte_array_append
+    (expected_arr, (const guint8 *) "\x00\x0d\x86test\x00", 8);
   gzochi_common_io_write_long (task_id, expected_arr->data, 8);
   g_byte_array_set_size (expected_arr, 16);
   
@@ -553,10 +564,12 @@ test_resubmit_tasks (taskclient_fixture *fixture, gconstpointer user_data)
   gzochid_data_set_binding_to_oid
     (fixture->app_context, binding_str, task_id, NULL);
   
-  g_byte_array_append (expected_arr, "\x00\x0d\x84test\x00", 8);
+  g_byte_array_append
+    (expected_arr, (const guint8 *) "\x00\x0d\x84test\x00", 8);
   gzochi_common_io_write_long (task_id, expected_arr->data, 8);
   g_byte_array_set_size (expected_arr, 16);
-  g_byte_array_append (expected_arr, "\x00\x05\x82test\x00", 8);
+  g_byte_array_append
+    (expected_arr, (const guint8 *) "\x00\x05\x82test\x00", 8);
 
   g_private_set (&name_key, "assigning");
 
