@@ -1,5 +1,5 @@
 /* test-util.c: Test routines for util.c in gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -101,7 +101,7 @@ test_util_serialize_string ()
   g_assert_cmpint (out->data[2], ==, 0x00);
   g_assert_cmpint (out->data[3], ==, 0x0e);
 
-  g_assert_cmpstr (out->data + 4, ==, "Hello, world!");
+  g_assert_cmpstr ((const char *) out->data + 4, ==, "Hello, world!");
 
   g_byte_array_unref (out);
 }
@@ -252,11 +252,11 @@ test_util_deserialize_boolean ()
 {
   GByteArray *in = g_byte_array_new ();
 
-  g_byte_array_append (in, "\001", 1);
+  g_byte_array_append (in, (const guint8 *) "\001", 1);
 
   g_assert (gzochid_util_deserialize_boolean (in));
 
-  g_byte_array_append (in, "\000", 1);
+  g_byte_array_append (in, (const guint8 *) "\000", 1);
   g_assert (!gzochid_util_deserialize_boolean (in));
   
   g_byte_array_unref (in);
@@ -267,7 +267,7 @@ test_util_deserialize_int ()
 {
   GByteArray *in = g_byte_array_new ();
 
-  g_byte_array_append (in, "\001\002\003\004", 4);
+  g_byte_array_append (in, (const guint8 *) "\001\002\003\004", 4);
   g_assert_cmpint (gzochid_util_deserialize_int (in), ==, 16909060);
   g_byte_array_unref (in);
 }
@@ -277,7 +277,8 @@ test_util_deserialize_uint64 ()
 {
   GByteArray *in = g_byte_array_new ();
 
-  g_byte_array_append (in, "\001\002\003\004\005\006\007\008", 8);
+  g_byte_array_append
+    (in, (const guint8 *) "\001\002\003\004\005\006\007\008", 8);
   g_assert_cmpint (gzochid_util_deserialize_uint64 (in), ==, 72623859790382848);
   g_byte_array_unref (in);
 }
@@ -289,7 +290,7 @@ test_util_deserialize_bytes ()
   int len = 0;
   unsigned char *bytes = NULL;
 
-  g_byte_array_append (in, "\000\000\000\003\144\145\146", 7);
+  g_byte_array_append (in, (const guint8 *) "\000\000\000\003\144\145\146", 7);
 
   bytes = gzochid_util_deserialize_bytes (in, &len);
   
@@ -308,8 +309,8 @@ test_util_deserialize_string ()
   GByteArray *in = g_byte_array_new ();
   char *str = NULL;
 
-  g_byte_array_append (in, "\000\000\000\020", 4);
-  g_byte_array_append (in, "Goodbye, world!", 16);
+  g_byte_array_append (in, (const guint8 *) "\000\000\000\020", 4);
+  g_byte_array_append (in, (const guint8 *) "Goodbye, world!", 16);
 
   str = gzochid_util_deserialize_string (in);
   g_assert_cmpstr (str, ==, "Goodbye, world!");
@@ -325,7 +326,7 @@ test_util_deserialize_list ()
   GList *lst = NULL;
   
   g_byte_array_append
-    (in, "\000\000\000\003"
+    (in, (const guint8 *) "\000\000\000\003"
      "\000\000\000\002\144\000"
      "\000\000\000\002\145\000"
      "\000\000\000\002\146\000", 22);
@@ -349,7 +350,7 @@ test_util_deserialize_sequence ()
   GSequenceIter *iter = NULL;
 
   g_byte_array_append
-    (in, "\000\000\000\003"
+    (in, (const guint8 *) "\000\000\000\003"
      "\000\000\000\002\144\000"
      "\000\000\000\002\145\000"
      "\000\000\000\002\146\000", 22);
@@ -386,7 +387,7 @@ test_util_deserialize_hash_table ()
   GHashTable *ht = NULL;
   
   g_byte_array_append
-    (in, "\000\000\000\001"
+    (in, (const guint8 *) "\000\000\000\001"
      "\000\000\000\004\142\141\172\000"
      "\000\000\000\004\161\165\170\000", 20);
 
@@ -411,7 +412,8 @@ test_util_deserialize_timeval ()
   GByteArray *in = g_byte_array_new ();
   struct timeval tv;
 
-  g_byte_array_append (in, "\000\000\000\003\000\000\000\004", 8);
+  g_byte_array_append
+    (in, (const guint8 *) "\000\000\000\003\000\000\000\004", 8);
   tv = gzochid_util_deserialize_timeval (in);
   
   g_assert_cmpint (tv.tv_sec, ==, 3);
