@@ -1,5 +1,5 @@
 /* treefile.c: A toy storage engine implementation for gzochid.
- * Copyright (C) 2018 Julian Graham
+ * Copyright (C) 2020 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -260,11 +260,11 @@ ensure_store_tx (gzochid_storage_transaction *tx, gzochid_storage_store *store)
   return tree;
 }
 
-static char *
+static unsigned char *
 get_internal (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 	      char *key, size_t key_len, size_t *value_len)
 {
-  char *ret = NULL;
+  unsigned char *ret = NULL;
   GTree *tree = ensure_store_tx (tx, store);
   treefile_datum key_datum;
   treefile_datum *value_datum = NULL;
@@ -286,7 +286,7 @@ get_internal (gzochid_storage_transaction *tx, gzochid_storage_store *store,
   return ret;
 }
 
-static char *
+static unsigned char *
 transaction_get (gzochid_storage_transaction *tx, gzochid_storage_store *store,
 		 char *key, size_t key_len, size_t *value_len)
 {
@@ -357,7 +357,8 @@ transaction_rollback (gzochid_storage_transaction *tx)
 
 static void
 transaction_put (gzochid_storage_transaction *tx, gzochid_storage_store *store,
-		 char *key, size_t key_len, char *value, size_t value_len)
+		 char *key, size_t key_len, unsigned char *value,
+		 size_t value_len)
 {
   GTree *tree = ensure_store_tx (tx, store);
   treefile_datum *key_datum = malloc (sizeof (treefile_datum));
@@ -458,7 +459,7 @@ transaction_next_key (gzochid_storage_transaction *tx,
   else return NULL;
 }
 
-static char *
+static unsigned char *
 transaction_get_for_update (gzochid_storage_transaction *tx,
 			    gzochid_storage_store *store, char *key,
 			    size_t key_len, size_t *value_len)
